@@ -116,7 +116,7 @@ sub parse
 
     my %oOut;
     my $iIndex = 0;
-    my $bText = $strName eq 'text' || $strName eq 'p' || $strName eq 'title' ||
+    my $bText = $strName eq 'text' || $strName eq 'li' || $strName eq 'p' || $strName eq 'title' ||
                 $strName eq 'summary' || $strName eq 'table-cell' || $strName eq 'table-column' || $strName eq 'list-item' ||
                 $strName eq 'admonition';
 
@@ -356,7 +356,7 @@ sub nodeTest
 ####################################################################################################################################
 # nodeAdd
 #
-# Add a node to the current doc's child list
+# Add a node to to the current doc's child list
 ####################################################################################################################################
 sub nodeAdd
 {
