@@ -31,13 +31,13 @@ my $oRenderTag =
 {
     'markdown' =>
     {
-        'br' => ['\n', ''],
         'quote' => ['"', '"'],
         'b' => ['**', '**'],
         'i' => ['_', '_'],
         # 'bi' => ['_**', '**_'],
-        'list-item' => ["\n", ""],
-        'list-item' => ['- ', "\n"],
+        'ul' => ["\n", ""],
+        'ol' => ["\n", "\n"],
+        'li' => ['- ', "\n"],
         'id' => ['`', '`'],
         'file' => ['`', '`'],
         'path' => ['`', '`'],
@@ -56,14 +56,13 @@ my $oRenderTag =
 
     'text' =>
     {
-        'br' => ['\n', ''],
         'quote' => ['"', '"'],
-        'p' => ['', "\n\n"],
         'b' => ['', ''],
         'i' => ['', ''],
         # 'bi' => ['', ''],
-        'list' => ["", "\n"],
-        'list-item' => ['* ', "\n"],
+        'ul' => ["\n", "\n"],
+        'ol' => ["\n", "\n"],
+        'li' => ['* ', "\n"],
         'id' => ['', ''],
         'host' => ['', ''],
         'file' => ['', ''],
@@ -79,19 +78,50 @@ my $oRenderTag =
         'backrest' => [undef, ''],
         'proper' => ['', ''],
         'postgres' => ['PostgreSQL', ''],
-        'admonition' => ['', "\n\n"],
+        'admonition' => ["\n", "\n\n"],
+    },
+
+    'latex' =>
+    {
+        'quote' => ['``', '"'],
+        'b' => ['\textbf{', '}'],
+        'i' => ['\textit{', '}'],
+        # 'bi' => ['', ''],
+        'ul' => ["\\begin{itemize}\n", "\\end{itemize}\n"],
+        # 'ol' => ["\n", "\n"],
+        'li' => ['\item ', "\n"],
+        'id' => ['\textnormal{\texttt{', '}}'],
+        'host' => ['\textnormal{\textbf{', '}}'],
+        'file' => ['\textnormal{\texttt{', '}}'],
+        'path' => ['\textnormal{\texttt{', '}}'],
+        'cmd' => ['\textnormal{\texttt{', "}}"],
+        'user' => ['\textnormal{\texttt{', '}}'],
+        'br-option' => ['', ''],
+        # 'param' => ['\texttt{', '}'],
+        # 'setting' => ['\texttt{', '}'],
+        'br-option' => ['\textnormal{\texttt{', '}}'],
+        'br-setting' => ['\textnormal{\texttt{', '}}'],
+        'pg-option' => ['\textnormal{\texttt{', '}}'],
+        'pg-setting' => ['\textnormal{\texttt{', '}}'],
+        'code' => ['\textnormal{\texttt{', '}}'],
+        # 'code' => ['\texttt{', '}'],
+        # 'code-block' => ['', ''],
+        # 'exe' => [undef, ''],
+        'backrest' => [undef, ''],
+        'proper' => ['\textnormal{\texttt{', '}}'],
+        'postgres' => ['PostgreSQL', ''],
+        'admonition' => ["\n\\begin{leftbar}\n\\textit{\\textbf{", "}\n\\end{leftbar}\n"],
     },
 
     'html' =>
     {
-        'br' => ['<br/>', ''],
         'quote' => ['<q>', '</q>'],
         'b' => ['<b>', '</b>'],
         'i' => ['<i>', '</i>'],
-        'p' => ['', ''],
         # 'bi' => ['<i><b>', '</b></i>'],
-        'list' => ['<ul class="list-unordered">', '</ul>'],
-        'list-item' => ['<li class="list-unordered">', '</li>'],
+        'ul' => ['<ul>', '</ul>'],
+        'ol' => ['<ol>', '</ol>'],
+        'li' => ['<li>', '</li>'],
         'id' => ['<span class="id">', '</span>'],
         'host' => ['<span class="host">', '</span>'],
         'file' => ['<span class="file">', '</span>'],
@@ -151,15 +181,41 @@ sub new
     $$oRenderTag{text}{backrest}[0] = "{[project]}";
     $$oRenderTag{text}{exe}[0] = "{[project-exe]}";
 
+    $$oRenderTag{latex}{backrest}[0] = "{[project]}";
+    $$oRenderTag{latex}{exe}[0] = "\\textnormal\{\\texttt\{[project-exe]}}\}\}";
+
     $$oRenderTag{html}{backrest}[0] = "<span class=\"backrest\">{[project]}</span>";
     $$oRenderTag{html}{exe}[0] = "<span class=\"file\">{[project-exe]}</span>";
 
     if (defined($self->{strRenderOutKey}))
     {
         # Copy page data to self
-        my $oRenderOut = $self->{oManifest}->renderOutGet($self->{strType}, $self->{strRenderOutKey});
+        my $oRenderOut =
+            $self->{oManifest}->renderOutGet($self->{strType} eq 'latex' ? 'pdf' : $self->{strType}, $self->{strRenderOutKey});
 
-        if (defined($$oRenderOut{source}) && $$oRenderOut{source} eq 'release' && $self->{oManifest}->isBackRest())
+        # If these are the backrest docs then load the reference
+        if ($self->{oManifest}->isBackRest())
+        {
+            $self->{oReference} =
+                new pgBackRestDoc::Common::DocConfig(${$self->{oManifest}->sourceGet('reference')}{doc}, $self);
+        }
+
+        if (defined($$oRenderOut{source}) && $$oRenderOut{source} eq 'reference' && $self->{oManifest}->isBackRest())
+        {
+            if ($self->{strRenderOutKey} eq 'configuration')
+            {
+                $self->{oDoc} = $self->{oReference}->helpConfigDocGet();
+            }
+            elsif ($self->{strRenderOutKey} eq 'command')
+            {
+                $self->{oDoc} = $self->{oReference}->helpCommandDocGet();
+            }
+            else
+            {
+                confess &log(ERROR, "cannot render $self->{strRenderOutKey} from source $$oRenderOut{source}");
+            }
+        }
+        elsif (defined($$oRenderOut{source}) && $$oRenderOut{source} eq 'release' && $self->{oManifest}->isBackRest())
         {
             require pgBackRestDoc::Custom::DocCustomRelease;
             pgBackRestDoc::Custom::DocCustomRelease->import();
@@ -415,7 +471,7 @@ sub build
             $oNode->paramSet('depend-default', $strDependPrev);
         }
 
-        # Set log to true if this section has an execute list. This helps reduce the info logging by only showing sections that are
+        # Set log to true if this section has an execute list.  This helps reduce the info logging by only showing sections that are
         # likely to take a log time.
         $oNode->paramSet('log', $self->{bExe} && $oNode->nodeList('execute-list', false) > 0 ? true : false);
 
@@ -733,6 +789,10 @@ sub processTag
                 {
                     $strUrl = '#' . substr($strSection, 1);
                 }
+                elsif ($strType eq 'latex')
+                {
+                    $strUrl = $strSection;
+                }
                 else
                 {
                     $strUrl = lc($self->processText($oSection->nodeGet('title')->textGet()));
@@ -751,6 +811,17 @@ sub processTag
         {
             $strBuffer = '[' . $oTag->valueGet() . '](' . $strUrl . ')';
         }
+        elsif ($strType eq 'latex')
+        {
+            if ($oTag->paramTest('url'))
+            {
+                $strBuffer = "\\href{$strUrl}{" . $oTag->valueGet() . "}";
+            }
+            else
+            {
+                $strBuffer = "\\hyperref[$strUrl]{" . $oTag->valueGet() . "}";
+            }
+        }
         elsif ($strType eq 'text')
         {
             $strBuffer = $oTag->valueGet();
@@ -772,14 +843,14 @@ sub processTag
 
         $strBuffer .= $strStart;
 
-        # Admonitions in the help materials are tags of the text element rather than field elements of the document so special
+        # Admonitions in the reference materials are tags of the text element rather than field elements of the document so special
         # handling is required
         if ($strTag eq 'admonition')
         {
             $strBuffer .= $self->processAdmonitionStart($oTag);
         }
 
-        if ($strTag eq 'p' || $strTag eq 'title' || $strTag eq 'list-item' || $strTag eq 'code-block' || $strTag eq 'summary'
+        if ($strTag eq 'p' || $strTag eq 'title' || $strTag eq 'li' || $strTag eq 'code-block' || $strTag eq 'summary'
             || $strTag eq 'admonition')
         {
             $strBuffer .= $self->processText($oTag);
@@ -834,7 +905,7 @@ sub processAdmonitionStart
     my $strType = $self->{strType};
     my $strBuffer = '';
 
-    # Note that any changes to HTML or markdown display tags may also need to be made here
+    # Note that any changes to the way the HTML, markdown or latex display tags may also need to be made here
     if ($strType eq 'html')
     {
         my $strType = $oTag->paramGet('type');
@@ -845,6 +916,10 @@ sub processAdmonitionStart
     {
         $strBuffer = uc($oTag->paramGet('type')) . ": ";
     }
+    elsif ($strType eq 'latex')
+    {
+        $strBuffer = uc($oTag->paramGet('type')) . ": }";
+    }
 
     # Return from function and log return values if any
     return logDebugReturn
@@ -876,7 +951,7 @@ sub processAdmonitionEnd
     my $strType = $self->{strType};
     my $strBuffer = '';
 
-    # Note that any changes to HTML or markdown display tags may also need to be made here
+    # Note that any changes to the way the HTML, markdown or latex display tags may also need to be made here
     if ($strType eq 'html')
     {
         $strBuffer = '</div>';
@@ -911,7 +986,6 @@ sub processText
 
     my $strType = $self->{strType};
     my $strBuffer = '';
-    my $strLastTag = 'body';
 
     foreach my $oNode ($oText->nodeList(undef, false))
     {
@@ -922,30 +996,11 @@ sub processText
                 confess &log(ERROR, "unable to process quotes in string (use <quote> instead):\n${oNode}");
             }
 
-            # Skip text nodes with linefeeds since they happen between tags
-            if (index($oNode, "\n") == -1)
-            {
-                $strBuffer .= $oNode;
-            }
+            $strBuffer .= $oNode;
         }
         else
         {
-            # Add br tags to separate paragraphs and linefeeds to make the output more diffable. This is needed because of the hacky
-            # way config text is being rendered in the final document, i.e. by passing rendered HTML into divs rather than XML to be
-            # rendered at that time.
-            if ($strLastTag eq 'p' && $strType eq 'html')
-            {
-                $strBuffer .= "<br/>\n";
-
-                if ($oNode->nameGet() eq 'p')
-                {
-                    $strBuffer .= "<br/>\n";
-                }
-            }
-
             $strBuffer .= $self->processTag($oNode);
-
-            $strLastTag = $oNode->nameGet();
         }
     }
     #
@@ -964,9 +1019,40 @@ sub processText
         $strBuffer =~ s/^ //smg;
     # }
 
-    if ($strType eq 'html')
+    if ($strType eq 'latex')
     {
-        $strBuffer =~ s/ma[s]ter/ma&#115;ter/g;
+        $strBuffer =~ s/\&mdash\;/---/g;
+        $strBuffer =~ s/\&lt\;/\</g;
+        $strBuffer =~ s/\<\=/\$\\leq\$/g;
+        $strBuffer =~ s/\>\=/\$\\geq\$/g;
+        # $strBuffer =~ s/\_/\\_/g;
+
+        # If not a code-block, which is to be taken AS IS, then escape special characters in latex
+        if ($oText->nameGet() ne 'code-block')
+        {
+            # If the previous character is not already a slash (e.g. not already escaped) then insert a slash
+            $strBuffer =~ s/(?<!\\)\#/\\#/g;
+            $strBuffer =~ s/(?<!\\)\%/\\%/g;
+            $strBuffer =~ s/(?<!\\)\_/\\_/g;
+            # $strBuffer =~ s/(?<!\\)\$/\\\$/g;
+
+            # Escape square brackest in list items since they are used for reformatting the bullet item with what is in brackets,
+            # which is not the intention.
+            if ($oText->nameGet() eq 'list-item')
+            {
+                $strBuffer =~ s/\[/\{\[/g;
+                $strBuffer =~ s/\]/\]\}/g;
+            }
+
+            $strBuffer =~ s/\&copy\;/{\\textcopyright}/g;
+            $strBuffer =~ s/\&trade\;/{\\texttrademark}/g;
+            $strBuffer =~ s/\&reg\;/{\\textregistered}/g;
+
+            $strBuffer =~ s/\&rarr\;/{\\textrightarrow}/g;
+
+            # Escape all ampersands after making any other conversions above
+            $strBuffer =~ s/(?<!\\)\&/\\&/g;
+        }
     }
 
     if ($strType eq 'text')
