@@ -11,8 +11,9 @@ Archive Get File
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
-int archiveGetFile(
+// Copy a file from the archive to the specified destination
+void archiveGetFile(
     const Storage *storage, const String *archiveFile, const String *walDestination, bool durable, CipherType cipherType,
-    const String *cipherPass);
+    const String *cipherPassArchive);
 
 #endif
