@@ -29,9 +29,6 @@ archiveGetProtocol(const String *command, const VariantList *paramList, Protocol
 
     ASSERT(command != NULL);
 
-    // Get the repo storage in case it is remote and encryption settings need to be pulled down
-    storageRepo();
-
     // Attempt to satisfy the request -- we may get requests that are meant for other handlers
     bool found = true;
 
@@ -39,14 +36,16 @@ archiveGetProtocol(const String *command, const VariantList *paramList, Protocol
     {
         if (strEq(command, PROTOCOL_COMMAND_ARCHIVE_GET_STR))
         {
-            const String *walSegment = varStr(varLstGet(paramList, 0));
+            const String *archiveFileRequest = varStr(varLstGet(paramList, 0));
+            const String *archiveFileActual = varStr(varLstGet(paramList, 1));
+            const CipherType cipherType = (CipherType)varUIntForce(varLstGet(paramList, 2));
+            const String *cipherPassArchive = varStr(varLstGet(paramList, 3));
 
-            protocolServerResponse(
-                server,
-                VARINT(
-                    archiveGetFile(
-                        storageSpoolWrite(), walSegment, strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s", strPtr(walSegment)), true,
-                        cipherType(cfgOptionStr(cfgOptRepoCipherType)), cfgOptionStrNull(cfgOptRepoCipherPass))));
+            archiveGetFile(
+                storageSpoolWrite(), archiveFileActual, strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s", strZ(archiveFileRequest)), true,
+                cipherType, cipherPassArchive);
+
+            protocolServerResponse(server, NULL);
         }
         else
             found = false;
