@@ -104,7 +104,7 @@ backupType(const String *type)
     else if (strEq(type, BACKUP_TYPE_INCR_STR))
         result = backupTypeIncr;
     else
-        THROW_FMT(AssertError, "invalid backup type '%s'", strPtr(type));
+        THROW_FMT(AssertError, "invalid backup type '%s'", strZ(type));
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -122,22 +122,16 @@ const String *backupTypeStr(BackupType type)
     switch (type)
     {
         case backupTypeFull:
-        {
             result = BACKUP_TYPE_FULL_STR;
             break;
-        }
 
         case backupTypeDiff:
-        {
             result = BACKUP_TYPE_DIFF_STR;
             break;
-        }
 
         case backupTypeIncr:
-        {
             result = BACKUP_TYPE_INCR_STR;
             break;
-        }
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -164,8 +158,8 @@ backupLinkLatest(const String *backupLabel)
         if (storageFeature(storageRepoWrite(), storageFeatureSymLink))
         {
             THROW_ON_SYS_ERROR_FMT(
-                symlink(strPtr(backupLabel), strPtr(latestLink)) == -1, FileOpenError,
-                "unable to create symlink '%s' to '%s'", strPtr(latestLink), strPtr(backupLabel));
+                symlink(strZ(backupLabel), strZ(latestLink)) == -1, FileOpenError, "unable to create symlink '%s' to '%s'",
+                strZ(latestLink), strZ(backupLabel));
         }
 
         // Sync backup path if required
