@@ -7,32 +7,28 @@ Local Command
 #include "command/archive/push/protocol.h"
 #include "command/backup/protocol.h"
 #include "command/restore/protocol.h"
+#include "command/verify/protocol.h"
 #include "common/debug.h"
-#include "common/io/handleRead.h"
-#include "common/io/handleWrite.h"
+#include "common/io/fdRead.h"
+#include "common/io/fdWrite.h"
 #include "common/log.h"
-#include "config/config.h"
+#include "config/config.intern.h"
 #include "config/protocol.h"
 #include "protocol/helper.h"
 #include "protocol/server.h"
 
 /**********************************************************************************************************************************/
 void
-cmdLocal(int handleRead, int handleWrite)
+cmdLocal(int fdRead, int fdWrite)
 {
     FUNCTION_LOG_VOID(logLevelDebug);
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Configure two retries for local commands
-        VariantList *retryInterval = varLstNew();
-        varLstAdd(retryInterval, varNewUInt64(0));
-        varLstAdd(retryInterval, varNewUInt64(15000));
-
         String *name = strNewFmt(PROTOCOL_SERVICE_LOCAL "-%u", cfgOptionUInt(cfgOptProcess));
-        IoRead *read = ioHandleReadNew(name, handleRead, (TimeMSec)(cfgOptionDbl(cfgOptProtocolTimeout) * 1000));
+        IoRead *read = ioFdReadNew(name, fdRead, cfgOptionUInt64(cfgOptProtocolTimeout));
         ioReadOpen(read);
-        IoWrite *write = ioHandleWriteNew(name, handleWrite);
+        IoWrite *write = ioFdWriteNew(name, fdWrite, cfgOptionUInt64(cfgOptProtocolTimeout));
         ioWriteOpen(write);
 
         ProtocolServer *server = protocolServerNew(name, PROTOCOL_SERVICE_LOCAL_STR, read, write);
@@ -40,7 +36,8 @@ cmdLocal(int handleRead, int handleWrite)
         protocolServerHandlerAdd(server, archivePushProtocol);
         protocolServerHandlerAdd(server, backupProtocol);
         protocolServerHandlerAdd(server, restoreProtocol);
-        protocolServerProcess(server, retryInterval);
+        protocolServerHandlerAdd(server, verifyProtocol);
+        protocolServerProcess(server, cfgCommandJobRetry());
     }
     MEM_CONTEXT_TEMP_END();
 
