@@ -22,29 +22,54 @@ typedef struct IoFilterGroup IoFilterGroup;
 #include "common/type/string.h"
 
 /***********************************************************************************************************************************
-Constructor
+Constructors
 ***********************************************************************************************************************************/
 IoFilterGroup *ioFilterGroupNew(void);
 
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
+// Add a filter
 IoFilterGroup *ioFilterGroupAdd(IoFilterGroup *this, IoFilter *filter);
+
+// Insert a filter before an index
 IoFilterGroup *ioFilterGroupInsert(IoFilterGroup *this, unsigned int listIdx, IoFilter *filter);
+
+// Clear filters
 IoFilterGroup *ioFilterGroupClear(IoFilterGroup *this);
+
+// Open filter group
 void ioFilterGroupOpen(IoFilterGroup *this);
+
+// Process filters
 void ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output);
+
+// Close filter group and gather results
 void ioFilterGroupClose(IoFilterGroup *this);
 
 /***********************************************************************************************************************************
-Getters
+Getters/Setters
 ***********************************************************************************************************************************/
+// Is the filter group done processing?
 bool ioFilterGroupDone(const IoFilterGroup *this);
+
+// Should the same input be passed again? A buffer of input can produce multiple buffers of output, e.g. when a file containing all
+// zeroes is being decompressed.
 bool ioFilterGroupInputSame(const IoFilterGroup *this);
+
+// Get all filters and their parameters so they can be passed to a remote
 Variant *ioFilterGroupParamAll(const IoFilterGroup *this);
+
+// Get filter results
 const Variant *ioFilterGroupResult(const IoFilterGroup *this, const String *filterType);
+
+// Get all filter results
 const Variant *ioFilterGroupResultAll(const IoFilterGroup *this);
+
+// Set all filter results
 void ioFilterGroupResultAllSet(IoFilterGroup *this, const Variant *filterResult);
+
+// Return total number of filters
 unsigned int ioFilterGroupSize(const IoFilterGroup *this);
 
 /***********************************************************************************************************************************
