@@ -295,6 +295,7 @@ bufLimitSet(Buffer *this, size_t limit)
 
     ASSERT(this != NULL);
     ASSERT(limit <= this->sizeAlloc);
+    ASSERT(limit >= this->used);
 
     this->size = limit;
     this->sizeLimit = true;
@@ -355,7 +356,7 @@ bufToLog(const Buffer *this)
 {
     String *result = strNewFmt(
         "{used: %zu, size: %zu%s", this->used, this->size,
-        this->sizeLimit ? strPtr(strNewFmt(", sizeAlloc: %zu}", this->sizeAlloc)) : "}");
+        this->sizeLimit ? strZ(strNewFmt(", sizeAlloc: %zu}", this->sizeAlloc)) : "}");
 
     return result;
 }
