@@ -48,7 +48,7 @@ Buffer *bufDup(const Buffer *buffer);
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
-//Append the contents of another buffer
+// Append the contents of another buffer
 Buffer *bufCat(Buffer *this, const Buffer *cat);
 
 // Append a C buffer
@@ -81,6 +81,14 @@ bufSize(const Buffer *this)
     return ((const BufferConst *)this)->size;
 }
 
+// Allocated buffer size. This may be different from bufSize() if a limit has been set.
+__attribute__((always_inline)) static inline size_t
+bufSizeAlloc(const Buffer *this)
+{
+    ASSERT_INLINE(this != NULL);
+    return ((const BufferConst *)this)->sizeAlloc;
+}
+
 // Amount of the buffer actually used. This will be updated automatically when possible but if the buffer is modified by using
 // bufPtr() then the user is responsible for updating used.
 __attribute__((always_inline)) static inline size_t
@@ -94,6 +102,13 @@ void bufUsedInc(Buffer *this, size_t inc);
 void bufUsedSet(Buffer *this, size_t used);
 void bufUsedZero(Buffer *this);
 
+// Is the buffer empty?
+__attribute__((always_inline)) static inline bool
+bufEmpty(const Buffer *this)
+{
+    return bufUsed(this) == 0;
+}
+
 // Remaining space in the buffer
 __attribute__((always_inline)) static inline size_t
 bufRemains(const Buffer *this)
@@ -159,7 +174,7 @@ By convention all buffer constant identifiers are appended with _BUF.
 
 // Create a buffer constant inline from a String
 #define BUFSTR(string)                                                                                                             \
-    BUF((unsigned char *)strPtr(string), strSize(string))
+    BUF((unsigned char *)strZ(string), strSize(string))
 
 // Create a buffer constant inline from a constant zero-terminated string
 #define BUFSTRDEF(stringdef)                                                                                                       \
