@@ -68,14 +68,14 @@ jsonToBool(const String *json)
     FUNCTION_TEST_END();
 
     unsigned int jsonPos = 0;
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
-    bool result = jsonToBoolInternal(strPtr(json), &jsonPos);
+    bool result = jsonToBoolInternal(strZ(json), &jsonPos);
 
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
     if (jsonPos != strSize(json))
-        THROW_FMT(JsonFormatError, "unexpected characters after boolean at '%s'", strPtr(json) + jsonPos);
+        THROW_FMT(JsonFormatError, "unexpected characters after boolean at '%s'", strZ(json) + jsonPos);
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -117,9 +117,9 @@ jsonToNumberInternal(const char *json, unsigned int *jsonPos)
         MEM_CONTEXT_PRIOR_BEGIN()
         {
             if (intSigned)
-                result = varNewInt64(cvtZToInt64(strPtr(resultStr)));
+                result = varNewInt64(cvtZToInt64(strZ(resultStr)));
             else
-                result = varNewUInt64(cvtZToUInt64(strPtr(resultStr)));
+                result = varNewUInt64(cvtZToUInt64(strZ(resultStr)));
         }
         MEM_CONTEXT_PRIOR_END();
     }
@@ -136,14 +136,14 @@ jsonToNumber(const String *json)
     FUNCTION_TEST_END();
 
     unsigned int jsonPos = 0;
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
-    Variant *result = jsonToNumberInternal(strPtr(json), &jsonPos);
+    Variant *result = jsonToNumberInternal(strZ(json), &jsonPos);
 
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
     if (jsonPos != strSize(json))
-        THROW_FMT(JsonFormatError, "unexpected characters after number at '%s'", strPtr(json) + jsonPos);
+        THROW_FMT(JsonFormatError, "unexpected characters after number at '%s'", strZ(json) + jsonPos);
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -259,35 +259,35 @@ jsonToStrInternal(const char *json, unsigned int *jsonPos)
                 switch (json[*jsonPos])
                 {
                     case '"':
-                            strCatChr(result, '"');
+                        strCatChr(result, '"');
                         break;
 
                     case '\\':
-                            strCatChr(result, '\\');
+                        strCatChr(result, '\\');
                         break;
 
                     case '/':
-                            strCatChr(result, '/');
+                        strCatChr(result, '/');
                         break;
 
                     case 'n':
-                            strCatChr(result, '\n');
+                        strCatChr(result, '\n');
                         break;
 
                     case 'r':
-                            strCatChr(result, '\r');
+                        strCatChr(result, '\r');
                         break;
 
                     case 't':
-                            strCatChr(result, '\t');
+                        strCatChr(result, '\t');
                         break;
 
                     case 'b':
-                            strCatChr(result, '\b');
+                        strCatChr(result, '\b');
                         break;
 
                     case 'f':
-                            strCatChr(result, '\f');
+                        strCatChr(result, '\f');
                         break;
 
                     default:
@@ -329,19 +329,19 @@ jsonToStr(const String *json)
     FUNCTION_TEST_END();
 
     unsigned int jsonPos = 0;
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
     String *result = NULL;
 
-    if (strncmp(strPtr(json), NULL_Z, 4) == 0)
+    if (strncmp(strZ(json), NULL_Z, 4) == 0)
         jsonPos += 4;
     else
-        result = jsonToStrInternal(strPtr(json), &jsonPos);
+        result = jsonToStrInternal(strZ(json), &jsonPos);
 
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
     if (jsonPos != strSize(json))
-        THROW_FMT(JsonFormatError, "unexpected characters after string at '%s'", strPtr(json) + jsonPos);
+        THROW_FMT(JsonFormatError, "unexpected characters after string at '%s'", strZ(json) + jsonPos);
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -406,17 +406,17 @@ jsonToKv(const String *json)
     FUNCTION_TEST_END();
 
     unsigned int jsonPos = 0;
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
-    if (strPtr(json)[jsonPos] != '{')
-        THROW_FMT(JsonFormatError, "expected '{' at '%s'", strPtr(json) + jsonPos);
+    if (strZ(json)[jsonPos] != '{')
+        THROW_FMT(JsonFormatError, "expected '{' at '%s'", strZ(json) + jsonPos);
 
-    KeyValue *result = jsonToKvInternal(strPtr(json), &jsonPos);
+    KeyValue *result = jsonToKvInternal(strZ(json), &jsonPos);
 
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
     if (jsonPos != strSize(json))
-        THROW_FMT(JsonFormatError, "unexpected characters after object at '%s'", strPtr(json) + jsonPos);
+        THROW_FMT(JsonFormatError, "unexpected characters after object at '%s'", strZ(json) + jsonPos);
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -481,14 +481,14 @@ jsonToVarLst(const String *json)
     FUNCTION_TEST_END();
 
     unsigned int jsonPos = 0;
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
-    VariantList *result = jsonToVarLstInternal(strPtr(json), &jsonPos);
+    VariantList *result = jsonToVarLstInternal(strZ(json), &jsonPos);
 
-    jsonConsumeWhiteSpace(strPtr(json), &jsonPos);
+    jsonConsumeWhiteSpace(strZ(json), &jsonPos);
 
     if (jsonPos != strSize(json))
-        THROW_FMT(JsonFormatError, "unexpected characters after array at '%s'", strPtr(json) + jsonPos);
+        THROW_FMT(JsonFormatError, "unexpected characters after array at '%s'", strZ(json) + jsonPos);
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -515,26 +515,20 @@ jsonToVarInternal(const char *json, unsigned int *jsonPos)
     {
         // String
         case '"':
-        {
             result = varNewStr(jsonToStrInternal(json, jsonPos));
             break;
-        }
 
         // Integer
         case '-':
         case '0' ... '9':
-        {
             result = jsonToNumberInternal(json, jsonPos);
             break;
-        }
 
         // Boolean
         case 't':
         case 'f':
-        {
             result = varNewBool(jsonToBoolInternal(json, jsonPos));
             break;
-        }
 
         // Null
         case 'n':
@@ -549,24 +543,18 @@ jsonToVarInternal(const char *json, unsigned int *jsonPos)
 
         // Array
         case '[':
-        {
             result = varNewVarLst(jsonToVarLstInternal(json, jsonPos));
             break;
-        }
 
         // Object
         case '{':
-        {
             result = varNewKv(jsonToKvInternal(json, jsonPos));
             break;
-        }
 
         // Object
         default:
-        {
             THROW_FMT(JsonFormatError, "invalid type at '%s'", json + *jsonPos);
             break;
-        }
     }
 
     jsonConsumeWhiteSpace(json, jsonPos);
@@ -581,10 +569,15 @@ jsonToVar(const String *json)
         FUNCTION_LOG_PARAM(STRING, json);
     FUNCTION_LOG_END();
 
-    const char *jsonPtr = strPtr(json);
+    const char *jsonPtr = strZ(json);
     unsigned int jsonPos = 0;
 
-    FUNCTION_LOG_RETURN(VARIANT, jsonToVarInternal(jsonPtr, &jsonPos));
+    Variant *result = jsonToVarInternal(jsonPtr, &jsonPos);
+
+    if (jsonPos != strSize(json))
+        THROW_FMT(JsonFormatError, "unexpected characters after JSON at '%s'", strZ(json) + jsonPos);
+
+    FUNCTION_LOG_RETURN(VARIANT, result);
 }
 
 /**********************************************************************************************************************************/
@@ -678,7 +671,7 @@ jsonFromStrInternal(String *json, const String *string)
 
         for (unsigned int stringIdx = 0; stringIdx < strSize(string); stringIdx++)
         {
-            char stringChr = strPtr(string)[stringIdx];
+            char stringChr = strZ(string)[stringIdx];
 
             switch (stringChr)
             {
@@ -700,46 +693,32 @@ jsonFromStrInternal(String *json, const String *string)
                     switch (stringChr)
                     {
                         case '"':
-                        {
                             strCatZ(json, "\\\"");
                             break;
-                        }
 
                         case '\\':
-                        {
                             strCatZ(json, "\\\\");
                             break;
-                        }
 
                         case '\n':
-                        {
                             strCatZ(json, "\\n");
                             break;
-                        }
 
                         case '\r':
-                        {
                             strCatZ(json, "\\r");
                             break;
-                        }
 
                         case '\t':
-                        {
                             strCatZ(json, "\\t");
                             break;
-                        }
 
                         case '\b':
-                        {
                             strCatZ(json, "\\b");
                             break;
-                        }
 
                         case '\f':
-                        {
                             strCatZ(json, "\\f");
                             break;
-                        }
                     }
 
                     break;
@@ -749,7 +728,7 @@ jsonFromStrInternal(String *json, const String *string)
                 {
                     // If escape string is zero size then start it
                     if (noEscapeSize == 0)
-                        noEscape = strPtr(string) + stringIdx;
+                        noEscape = strZ(string) + stringIdx;
 
                     noEscapeSize++;
                     break;
@@ -808,7 +787,7 @@ jsonFromKvInternal(const KeyValue *kv)
                 strCatZ(result, ",");
 
             // Keys are always strings in the output, so add starting quote and colon.
-            strCatFmt(result, "\"%s\":", strPtr(key));
+            strCatFmt(result, "\"%s\":", strZ(key));
 
             // NULL value
             if (value == NULL)
@@ -818,17 +797,15 @@ jsonFromKvInternal(const KeyValue *kv)
                 switch (varType(value))
                 {
                     case varTypeKeyValue:
-                    {
                         strCat(result, jsonFromKvInternal(kvDup(varKv(value))));
                         break;
-                    }
 
                     case varTypeVariantList:
                     {
                         // If the array is empty, then do not add formatting, else process the array.
                         if (varVarLst(value) == NULL)
                             strCat(result, NULL_STR);
-                        else if (varLstSize(varVarLst(value)) == 0)
+                        else if (varLstEmpty(varVarLst(value)))
                             strCatZ(result, "[]");
                         else
                         {
@@ -873,16 +850,12 @@ jsonFromKvInternal(const KeyValue *kv)
 
                     // String
                     case varTypeString:
-                    {
                         jsonFromStrInternal(result, varStr(value));
                         break;
-                    }
 
                     default:
-                    {
                         strCat(result, varStrForce(value));
                         break;
-                    }
                 }
             }
         }
@@ -969,7 +942,7 @@ jsonFromVar(const Variant *var)
             const VariantList *vl = varVarLst(var);
 
             // If not an empty array
-            if (varLstSize(vl) > 0)
+            if (!varLstEmpty(vl))
             {
                 strCatZ(jsonStr, "[");
 
