@@ -165,13 +165,14 @@ typedef struct InfoLoadData
 } InfoLoadData;
 
 static void
-infoLoadCallback(void *data, const String *section, const String *key, const String *value)
+infoLoadCallback(void *data, const String *section, const String *key, const String *value, const Variant *valueVar)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, data);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
         FUNCTION_TEST_PARAM(STRING, value);
+        FUNCTION_TEST_PARAM(VARIANT, valueVar);
     FUNCTION_TEST_END();
 
     ASSERT(data != NULL);
@@ -206,18 +207,20 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
     // Process backrest section
     if (strEq(section, INFO_SECTION_BACKREST_STR))
     {
+        ASSERT(valueVar != NULL);
+
         // Validate format
         if (strEq(key, INFO_KEY_FORMAT_STR))
         {
-            if (jsonToUInt(value) != REPOSITORY_FORMAT)
-                THROW_FMT(FormatError, "expected format %d but found %d", REPOSITORY_FORMAT, cvtZToInt(strPtr(value)));
+            if (varUInt64(valueVar) != REPOSITORY_FORMAT)
+                THROW_FMT(FormatError, "expected format %d but found %" PRIu64, REPOSITORY_FORMAT, varUInt64(valueVar));
         }
         // Store pgBackRest version
         else if (strEq(key, INFO_KEY_VERSION_STR))
         {
             MEM_CONTEXT_BEGIN(loadData->info->memContext)
             {
-                loadData->info->backrestVersion = jsonToStr(value);
+                loadData->info->backrestVersion = strDup(varStr(valueVar));
             }
             MEM_CONTEXT_END();
         }
@@ -226,7 +229,7 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
         {
             MEM_CONTEXT_BEGIN(loadData->memContext)
             {
-                loadData->checksumExpected = jsonToStr(value);
+                loadData->checksumExpected = strDup(varStr(valueVar));
             }
             MEM_CONTEXT_END();
         }
@@ -234,19 +237,21 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
     // Process cipher section
     else if (strEq(section, INFO_SECTION_CIPHER_STR))
     {
+        ASSERT(valueVar != NULL);
+
         // No validation needed for cipher-pass, just store it
         if (strEq(key, INFO_KEY_CIPHER_PASS_STR))
         {
             MEM_CONTEXT_BEGIN(loadData->info->memContext)
             {
-                loadData->info->cipherPass = jsonToStr(value);
+                loadData->info->cipherPass = strDup(varStr(valueVar));
             }
             MEM_CONTEXT_END();
         }
     }
     // Else pass to callback for processing
     else
-        loadData->callbackFunction(loadData->callbackData, section, key, value);
+        loadData->callbackFunction(loadData->callbackData, section, key, valueVar);
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -301,12 +306,12 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
             const String *checksumActual = varStr(ioFilterResult(data.checksumActual));
 
             if (data.checksumExpected == NULL)
-                THROW_FMT(ChecksumError, "invalid checksum, actual '%s' but no checksum found", strPtr(checksumActual));
+                THROW_FMT(ChecksumError, "invalid checksum, actual '%s' but no checksum found", strZ(checksumActual));
             else if (!strEq(data.checksumExpected, checksumActual))
             {
                 THROW_FMT(
-                    ChecksumError, "invalid checksum, actual '%s' but expected '%s'", strPtr(checksumActual),
-                    strPtr(data.checksumExpected));
+                    ChecksumError, "invalid checksum, actual '%s' but expected '%s'", strZ(checksumActual),
+                    strZ(data.checksumExpected));
             }
         }
         MEM_CONTEXT_TEMP_END();
@@ -346,6 +351,9 @@ infoSaveValue(InfoSave *infoSaveData, const String *section, const String *key,
     ASSERT(section != NULL);
     ASSERT(key != NULL);
     ASSERT(jsonValue != NULL);
+    ASSERT(strSize(jsonValue) != 0);
+    // The JSON value must not be an array because this may be confused with a section in the ini file
+    ASSERT(strZ(jsonValue)[0] != '[');
 
     // Save section
     if (infoSaveData->sectionLast == NULL || !strEq(section, infoSaveData->sectionLast))
@@ -525,7 +533,7 @@ infoLoad(const String *error, InfoLoadCallback *callbackFunction, void *callback
                 if (loadErrorType == NULL)
                 {
                     loadErrorType = errorType();
-                    loadErrorMessage = strNewFmt("%s:", strPtr(error));
+                    loadErrorMessage = strNewFmt("%s:", strZ(error));
                 }
                 // Else if the error type is different
                 else if (loadErrorType != errorType())
@@ -552,7 +560,7 @@ infoLoad(const String *error, InfoLoadCallback *callbackFunction, void *callback
 
         // Error when no file was loaded
         if (!loaded)
-            THROWP(loadErrorType, strPtr(loadErrorMessage));
+            THROWP(loadErrorType, strZ(loadErrorMessage));
     }
     MEM_CONTEXT_TEMP_END();
 
