@@ -14,10 +14,10 @@ Backup Info Handler
 #include "common/ini.h"
 #include "common/log.h"
 #include "common/memContext.h"
-#include "common/object.h"
 #include "common/regExp.h"
 #include "common/type/json.h"
 #include "common/type/list.h"
+#include "common/type/object.h"
 #include "info/infoBackup.h"
 #include "info/manifest.h"
 #include "postgres/interface.h"
@@ -64,10 +64,11 @@ struct InfoBackup
     List *backup;                                                   // List of current backups and their associated data
 };
 
+OBJECT_DEFINE_MOVE(INFO_BACKUP);
 OBJECT_DEFINE_FREE(INFO_BACKUP);
 
 /***********************************************************************************************************************************
-Create new object
+Internal constructor
 ***********************************************************************************************************************************/
 static InfoBackup *
 infoBackupNewInternal(void)
@@ -85,19 +86,18 @@ infoBackupNewInternal(void)
     FUNCTION_TEST_RETURN(this);
 }
 
-/***********************************************************************************************************************************
-Create new object without loading it from a file
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 InfoBackup *
-infoBackupNew(unsigned int pgVersion, uint64_t pgSystemId, const String *cipherPassSub)
+infoBackupNew(unsigned int pgVersion, uint64_t pgSystemId, unsigned int pgCatalogVersion, const String *cipherPassSub)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(UINT, pgVersion);
         FUNCTION_LOG_PARAM(UINT64, pgSystemId);
+        FUNCTION_LOG_PARAM(UINT, pgCatalogVersion);
         FUNCTION_TEST_PARAM(STRING, cipherPassSub);
     FUNCTION_LOG_END();
 
-    ASSERT(pgVersion > 0 && pgSystemId > 0);
+    ASSERT(pgVersion > 0 && pgSystemId > 0 && pgCatalogVersion > 0);
 
     InfoBackup *this = NULL;
 
@@ -107,7 +107,7 @@ infoBackupNew(unsigned int pgVersion, uint64_t pgSystemId, const String *cipherP
 
         // Initialize the pg data
         this->infoPg = infoPgNew(infoPgBackup, cipherPassSub);
-        infoBackupPgSet(this, pgVersion, pgSystemId);
+        infoBackupPgSet(this, pgVersion, pgSystemId, pgCatalogVersion);
     }
     MEM_CONTEXT_NEW_END();
 
@@ -118,13 +118,13 @@ infoBackupNew(unsigned int pgVersion, uint64_t pgSystemId, const String *cipherP
 Create new object and load contents from a file
 ***********************************************************************************************************************************/
 static void
-infoBackupLoadCallback(void *data, const String *section, const String *key, const String *value)
+infoBackupLoadCallback(void *data, const String *section, const String *key, const Variant *value)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, data);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
-        FUNCTION_TEST_PARAM(STRING, value);
+        FUNCTION_TEST_PARAM(VARIANT, value);
     FUNCTION_TEST_END();
 
     ASSERT(data != NULL);
@@ -137,7 +137,7 @@ infoBackupLoadCallback(void *data, const String *section, const String *key, con
     // Process current backup list
     if (strEq(section, INFO_BACKUP_SECTION_BACKUP_CURRENT_STR))
     {
-        const KeyValue *backupKv = jsonToKv(value);
+        const KeyValue *backupKv = varKv(value);
 
         MEM_CONTEXT_BEGIN(lstMemContext(infoBackup->backup))
         {
@@ -150,7 +150,7 @@ infoBackupLoadCallback(void *data, const String *section, const String *key, con
                 .backupInfoSize = varUInt64(kvGet(backupKv, INFO_BACKUP_KEY_BACKUP_INFO_SIZE_VAR)),
                 .backupInfoSizeDelta = varUInt64(kvGet(backupKv, INFO_BACKUP_KEY_BACKUP_INFO_SIZE_DELTA_VAR)),
                 .backupLabel = strDup(key),
-                .backupPgId = cvtZToUInt(strPtr(varStrForce(kvGet(backupKv, INFO_KEY_DB_ID_VAR)))),
+                .backupPgId = cvtZToUInt(strZ(varStrForce(kvGet(backupKv, INFO_KEY_DB_ID_VAR)))),
 
                 // When reading timestamps, read as uint64 to ensure always positive value (guarantee no backups before 1970)
                 .backupTimestampStart = (time_t)varUInt64(kvGet(backupKv, INFO_BACKUP_KEY_BACKUP_TIMESTAMP_START_VAR)),
@@ -184,7 +184,8 @@ infoBackupLoadCallback(void *data, const String *section, const String *key, con
     FUNCTION_TEST_RETURN_VOID();
 }
 
-static InfoBackup *
+/**********************************************************************************************************************************/
+InfoBackup *
 infoBackupNewLoad(IoRead *read)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -293,9 +294,7 @@ infoBackupSave(InfoBackup *this, IoWrite *write)
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-Get PostgreSQL info
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 InfoPg *
 infoBackupPg(const InfoBackup *this)
 {
@@ -308,26 +307,22 @@ infoBackupPg(const InfoBackup *this)
     FUNCTION_TEST_RETURN(this->infoPg);
 }
 
-/***********************************************************************************************************************************
-Set the infoPg data
-***********************************************************************************************************************************/
 InfoBackup *
-infoBackupPgSet(InfoBackup *this, unsigned int pgVersion, uint64_t pgSystemId)
+infoBackupPgSet(InfoBackup *this, unsigned int pgVersion, uint64_t pgSystemId, unsigned int pgCatalogVersion)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(INFO_BACKUP, this);
         FUNCTION_LOG_PARAM(UINT, pgVersion);
         FUNCTION_LOG_PARAM(UINT64, pgSystemId);
+        FUNCTION_LOG_PARAM(UINT, pgCatalogVersion);
     FUNCTION_LOG_END();
 
-    this->infoPg = infoPgSet(this->infoPg, infoPgBackup, pgVersion, pgSystemId);
+    this->infoPg = infoPgSet(this->infoPg, infoPgBackup, pgVersion, pgSystemId, pgCatalogVersion);
 
     FUNCTION_LOG_RETURN(INFO_BACKUP, this);
 }
 
-/***********************************************************************************************************************************
-Get total current backups
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 unsigned int
 infoBackupDataTotal(const InfoBackup *this)
 {
@@ -340,9 +335,7 @@ infoBackupDataTotal(const InfoBackup *this)
     FUNCTION_TEST_RETURN(lstSize(this->backup));
 }
 
-/***********************************************************************************************************************************
-Return a structure of the backup data from a specific index
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 InfoBackupData
 infoBackupData(const InfoBackup *this, unsigned int backupDataIdx)
 {
@@ -353,12 +346,25 @@ infoBackupData(const InfoBackup *this, unsigned int backupDataIdx)
 
     ASSERT(this != NULL);
 
-    FUNCTION_LOG_RETURN(INFO_BACKUP_DATA, *((InfoBackupData *)lstGet(this->backup, backupDataIdx)));
+    FUNCTION_LOG_RETURN(INFO_BACKUP_DATA, *(InfoBackupData *)lstGet(this->backup, backupDataIdx));
 }
 
-/***********************************************************************************************************************************
-Add a backup to the current list
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
+InfoBackupData *
+infoBackupDataByLabel(const InfoBackup *this, const String *backupLabel)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(INFO_BACKUP, this);
+        FUNCTION_TEST_PARAM(STRING, backupLabel);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(backupLabel != NULL);
+
+    FUNCTION_TEST_RETURN(lstFind(this->backup, &backupLabel));
+}
+
+/**********************************************************************************************************************************/
 void
 infoBackupDataAdd(const InfoBackup *this, const Manifest *manifest)
 {
@@ -422,7 +428,7 @@ infoBackupDataAdd(const InfoBackup *this, const Manifest *manifest)
                 .optionBackupStandby = manData->backupOptionStandby != NULL ? varBool(manData->backupOptionStandby) : false,
                 .optionChecksumPage = manData->backupOptionChecksumPage != NULL ?
                     varBool(manData->backupOptionChecksumPage) : false,
-                .optionCompress = manData->backupOptionCompress,
+                .optionCompress = manData->backupOptionCompressType != compressTypeNone,
                 .optionHardlink = manData->backupOptionHardLink,
                 .optionOnline = manData->backupOptionOnline,
             };
@@ -447,9 +453,7 @@ infoBackupDataAdd(const InfoBackup *this, const Manifest *manifest)
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-Delete a backup from the current backup list
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 void
 infoBackupDataDelete(const InfoBackup *this, const String *backupDeleteLabel)
 {
@@ -471,9 +475,7 @@ infoBackupDataDelete(const InfoBackup *this, const String *backupDeleteLabel)
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-Return a list of current backup labels, applying a regex expression if provided
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 StringList *
 infoBackupDataLabelList(const InfoBackup *this, const String *expression)
 {
@@ -508,9 +510,40 @@ infoBackupDataLabelList(const InfoBackup *this, const String *expression)
     FUNCTION_LOG_RETURN(STRING_LIST, result);
 }
 
-/***********************************************************************************************************************************
-Return the cipher passphrase
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
+StringList *
+infoBackupDataDependentList(const InfoBackup *this, const String *backupLabel)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(INFO_BACKUP, this);
+        FUNCTION_LOG_PARAM(STRING, backupLabel);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(backupLabel != NULL);
+
+    // Return the given label as the only dependency or the given label and a list of labels that depend on it
+    StringList *result = strLstNew();
+    strLstAdd(result, backupLabel);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // For each backup label from oldest to newest in the current section, add each dependency to the list
+        for (unsigned int backupLabelIdx = 0; backupLabelIdx < infoBackupDataTotal(this); backupLabelIdx++)
+        {
+            InfoBackupData backupData = infoBackupData(this, backupLabelIdx);
+
+            // If the backupPrior is in the dependency chain add the label to the list
+            if (backupData.backupPrior != NULL && strLstExists(result, backupData.backupPrior))
+                strLstAdd(result, backupData.backupLabel);
+        }
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(STRING_LIST, result);
+}
+
+/**********************************************************************************************************************************/
 const String *
 infoBackupCipherPass(const InfoBackup *this)
 {
@@ -523,9 +556,7 @@ infoBackupCipherPass(const InfoBackup *this)
     FUNCTION_TEST_RETURN(infoPgCipherPass(this->infoPg));
 }
 
-/***********************************************************************************************************************************
-Helper function to load backup info files
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 typedef struct InfoBackupLoadFileData
 {
     MemContext *memContext;                                         // Mem context
@@ -552,7 +583,7 @@ infoBackupLoadFileCallback(void *data, unsigned int try)
     if (try < 2)
     {
         // Construct filename based on try
-        const String *fileName = try == 0 ? loadData->fileName : strNewFmt("%s" INFO_COPY_EXT, strPtr(loadData->fileName));
+        const String *fileName = try == 0 ? loadData->fileName : strNewFmt("%s" INFO_COPY_EXT, strZ(loadData->fileName));
 
         // Attempt to load the file
         IoRead *read = storageReadIo(storageNewReadP(loadData->storage, fileName));
@@ -594,7 +625,7 @@ infoBackupLoadFile(const Storage *storage, const String *fileName, CipherType ci
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        const char *fileNamePath = strPtr(storagePathP(storage, fileName));
+        const char *fileNamePath = strZ(storagePathP(storage, fileName));
 
         TRY_BEGIN()
         {
@@ -618,9 +649,7 @@ infoBackupLoadFile(const Storage *storage, const String *fileName, CipherType ci
     FUNCTION_LOG_RETURN(INFO_BACKUP, data.infoBackup);
 }
 
-/***********************************************************************************************************************************
-Load backup info and update it by adding valid backups from the repo or removing backups no longer in the repo
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 InfoBackup *
 infoBackupLoadFileReconstruct(const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass)
 {
@@ -646,9 +675,34 @@ infoBackupLoadFileReconstruct(const Storage *storage, const String *fileName, Ci
                 .expression = backupRegExpP(.full = true, .differential = true, .incremental = true)),
             sortOrderAsc);
 
-        // Get the current list of backups from backup.info
+        // Get the list of current backups and remove backups from current that are no longer in the repository
         StringList *backupCurrentList = strLstSort(infoBackupDataLabelList(infoBackup, NULL), sortOrderAsc);
 
+        for (unsigned int backupCurrIdx = 0; backupCurrIdx < strLstSize(backupCurrentList); backupCurrIdx++)
+        {
+            String *backupLabel = strLstGet(backupCurrentList, backupCurrIdx);
+            String *manifestFileName = strNewFmt(STORAGE_REPO_BACKUP "/%s/" BACKUP_MANIFEST_FILE, strZ(backupLabel));
+
+            // If the manifest does not exist on disk and this backup has not already been deleted from the current list in the
+            // infoBackup object, then remove it and its dependencies
+            if (!storageExistsP(storage, manifestFileName) && infoBackupDataByLabel(infoBackup, backupLabel) != NULL)
+            {
+                StringList *backupList = strLstSort(infoBackupDataDependentList(infoBackup, backupLabel), sortOrderDesc);
+
+                for (unsigned int backupIdx = 0; backupIdx < strLstSize(backupList); backupIdx++)
+                {
+                    String *removeBackup = strLstGet(backupList, backupIdx);
+
+                    LOG_WARN_FMT("backup '%s' missing manifest removed from " INFO_BACKUP_FILE, strZ(removeBackup));
+
+                    infoBackupDataDelete(infoBackup, removeBackup);
+                }
+            }
+        }
+
+        // Get the updated current list of backups from backup.info
+        backupCurrentList = strLstSort(infoBackupDataLabelList(infoBackup, NULL), sortOrderAsc);
+
         // For each backup in the repo, check if it exists in backup.info
         for (unsigned int backupIdx = 0; backupIdx < strLstSize(backupList); backupIdx++)
         {
@@ -657,7 +711,7 @@ infoBackupLoadFileReconstruct(const Storage *storage, const String *fileName, Ci
             // If it does not exist in the list of current backups, then if it is valid, add it
             if (!strLstExists(backupCurrentList, backupLabel))
             {
-                String *manifestFileName = strNewFmt(STORAGE_REPO_BACKUP "/%s/" BACKUP_MANIFEST_FILE, strPtr(backupLabel));
+                String *manifestFileName = strNewFmt(STORAGE_REPO_BACKUP "/%s/" BACKUP_MANIFEST_FILE, strZ(backupLabel));
 
                 // Check if a completed backup exists (backup.manifest only - ignore .copy)
                 if (storageExistsP(storage, manifestFileName))
@@ -673,11 +727,14 @@ infoBackupLoadFileReconstruct(const Storage *storage, const String *fileName, Ci
                     {
                         InfoPgData pgHistory = infoPgData(infoBackup->infoPg, pgIdx);
 
-                        // If there is an exact match with the history, system and version then add it to the current backup list
+                        // If there is an exact match with the history, system and version and there is no backup-prior dependency
+                        // or there is a backup-prior and it is in the list, then add this backup to the current backup list
                         if (manData->pgId == pgHistory.id && manData->pgSystemId == pgHistory.systemId &&
-                            manData->pgVersion == pgHistory.version)
+                            manData->pgVersion == pgHistory.version &&
+                            (manData->backupLabelPrior == NULL ||
+                                infoBackupDataByLabel(infoBackup, manData->backupLabelPrior) != NULL))
                         {
-                            LOG_WARN_FMT("backup '%s' found in repository added to " INFO_BACKUP_FILE, strPtr(backupLabel));
+                            LOG_WARN_FMT("backup '%s' found in repository added to " INFO_BACKUP_FILE, strZ(backupLabel));
                             infoBackupDataAdd(infoBackup, manifest);
                             found = true;
                             break;
@@ -685,35 +742,17 @@ infoBackupLoadFileReconstruct(const Storage *storage, const String *fileName, Ci
                     }
 
                     if (!found)
-                        LOG_WARN_FMT("invalid backup '%s' cannot be added to current backups", strPtr(manData->backupLabel));
+                        LOG_WARN_FMT("invalid backup '%s' cannot be added to current backups", strZ(manData->backupLabel));
                 }
             }
         }
-
-        // Get the updated list of current backups and remove backups from current that are no longer in the repository
-        backupCurrentList = infoBackupDataLabelList(infoBackup, NULL);
-
-        for (unsigned int backupCurrIdx = 0; backupCurrIdx < strLstSize(backupCurrentList); backupCurrIdx++)
-        {
-            String *backupLabel = strLstGet(backupCurrentList, backupCurrIdx);
-            String *manifestFileName = strNewFmt(STORAGE_REPO_BACKUP "/%s/" BACKUP_MANIFEST_FILE, strPtr(backupLabel));
-
-            // Remove backup from the current list in the infoBackup object
-            if (!storageExistsP(storage, manifestFileName))
-            {
-                LOG_WARN_FMT("backup '%s' missing manifest removed from " INFO_BACKUP_FILE, strPtr(backupLabel));
-                infoBackupDataDelete(infoBackup, backupLabel);
-            }
-        }
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN(INFO_BACKUP, infoBackup);
 }
 
-/***********************************************************************************************************************************
-Helper function to save backup info files
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 void
 infoBackupSaveFile(
     InfoBackup *infoBackup, const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass)
@@ -739,19 +778,16 @@ infoBackupSaveFile(
         infoBackupSave(infoBackup, write);
 
         // Make a copy of the file
-        storageCopy(
-            storageNewReadP(storage, fileName), storageNewWriteP(storage, strNewFmt("%s" INFO_COPY_EXT, strPtr(fileName))));
+        storageCopy(storageNewReadP(storage, fileName), storageNewWriteP(storage, strNewFmt("%s" INFO_COPY_EXT, strZ(fileName))));
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-Render as string for logging
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 String *
 infoBackupDataToLog(const InfoBackupData *this)
 {
-    return strNewFmt("{label: %s, pgId: %u}", strPtr(this->backupLabel), this->backupPgId);
+    return strNewFmt("{label: %s, pgId: %u}", strZ(this->backupLabel), this->backupPgId);
 }
