@@ -9,8 +9,9 @@ Protocol Helper
 #include "common/debug.h"
 #include "common/exec.h"
 #include "common/memContext.h"
-#include "config/config.h"
+#include "config/config.intern.h"
 #include "config/exec.h"
+#include "config/parse.h"
 #include "config/protocol.h"
 #include "postgres/version.h"
 #include "protocol/helper.h"
@@ -69,10 +70,13 @@ protocolHelperInit(void)
 
 /**********************************************************************************************************************************/
 bool
-repoIsLocal(void)
+repoIsLocal(unsigned int repoIdx)
 {
-    FUNCTION_TEST_VOID();
-    FUNCTION_TEST_RETURN(!cfgOptionTest(cfgOptRepoHost));
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(UINT, repoIdx);
+    FUNCTION_LOG_END();
+
+    FUNCTION_LOG_RETURN(BOOL, !cfgOptionIdxTest(cfgOptRepoHost, repoIdx));
 }
 
 /**********************************************************************************************************************************/
@@ -81,7 +85,18 @@ repoIsLocalVerify(void)
 {
     FUNCTION_TEST_VOID();
 
-    if (!repoIsLocal())
+    repoIsLocalVerifyIdx(cfgOptionGroupIdxDefault(cfgOptGrpRepo));
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
+/**********************************************************************************************************************************/
+void
+repoIsLocalVerifyIdx(unsigned int repoIdx)
+{
+    FUNCTION_TEST_VOID();
+
+    if (!repoIsLocal(repoIdx))
         THROW_FMT(HostInvalidError, "%s command must be run on the repository host", cfgCommandName(cfgCommand()));
 
     FUNCTION_TEST_RETURN_VOID();
@@ -89,15 +104,13 @@ repoIsLocalVerify(void)
 
 /**********************************************************************************************************************************/
 bool
-pgIsLocal(unsigned int hostId)
+pgIsLocal(unsigned int pgIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(UINT, hostId);
+        FUNCTION_LOG_PARAM(UINT, pgIdx);
     FUNCTION_LOG_END();
 
-    ASSERT(hostId > 0);
-
-    FUNCTION_LOG_RETURN(BOOL, !cfgOptionTest(cfgOptPgHost + hostId - 1));
+    FUNCTION_LOG_RETURN(BOOL, !cfgOptionIdxTest(cfgOptPgHost, pgIdx));
 }
 
 /**********************************************************************************************************************************/
@@ -106,7 +119,7 @@ pgIsLocalVerify(void)
 {
     FUNCTION_TEST_VOID();
 
-    if (!pgIsLocal(1))
+    if (!pgIsLocal(cfgOptionGroupIdxDefault(cfgOptGrpPg)))
         THROW_FMT(HostInvalidError, "%s command must be run on the " PG_NAME " host", cfgCommandName(cfgCommand()));
 
     FUNCTION_TEST_RETURN_VOID();
@@ -116,16 +129,14 @@ pgIsLocalVerify(void)
 Get the command line required for local protocol execution
 ***********************************************************************************************************************************/
 static StringList *
-protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostId, unsigned int protocolId)
+protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostIdx, unsigned int processId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(ENUM, protocolStorageType);
-        FUNCTION_LOG_PARAM(UINT, hostId);
-        FUNCTION_LOG_PARAM(UINT, protocolId);
+        FUNCTION_LOG_PARAM(UINT, hostIdx);
+        FUNCTION_LOG_PARAM(UINT, processId);
     FUNCTION_LOG_END();
 
-    ASSERT(hostId > 0);
-
     StringList *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -134,10 +145,12 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostId,
         KeyValue *optionReplace = kvNew();
 
         // Add the process id -- used when more than one process will be called
-        kvPut(optionReplace, VARSTR(CFGOPT_PROCESS_STR), VARUINT(protocolId));
+        kvPut(optionReplace, VARSTR(CFGOPT_PROCESS_STR), VARUINT(processId));
 
-        // Add the host id
-        kvPut(optionReplace, VARSTR(CFGOPT_HOST_ID_STR), VARUINT(hostId));
+        // Add the pg default. Don't do this for repos because the repo default should come from the user or the local should
+        // handle all the repos equally. Repos don't get special handling like pg primaries or standbys.
+        if (protocolStorageType == protocolStorageTypePg)
+            kvPut(optionReplace, VARSTRDEF(CFGOPT_PG), VARUINT(cfgOptionGroupIdxToKey(cfgOptGrpPg, hostIdx)));
 
         // Add the remote type
         kvPut(optionReplace, VARSTR(CFGOPT_REMOTE_TYPE_STR), VARSTR(protocolStorageTypeStr(protocolStorageType)));
@@ -162,16 +175,14 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostId,
 
 /**********************************************************************************************************************************/
 ProtocolClient *
-protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostId, unsigned int protocolId)
+protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx, unsigned int processId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(ENUM, protocolStorageType);
-        FUNCTION_LOG_PARAM(UINT, hostId);
-        FUNCTION_LOG_PARAM(UINT, protocolId);
+        FUNCTION_LOG_PARAM(UINT, hostIdx);
+        FUNCTION_LOG_PARAM(UINT, processId);
     FUNCTION_LOG_END();
 
-    ASSERT(hostId > 0);
-
     protocolHelperInit();
 
     // Allocate the client cache
@@ -188,10 +199,10 @@ protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostId, u
         MEM_CONTEXT_END();
     }
 
-    ASSERT(protocolId <= protocolHelper.clientLocalSize);
+    ASSERT(processId <= protocolHelper.clientLocalSize);
 
     // Create protocol object
-    ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientLocal[protocolId - 1];
+    ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientLocal[processId - 1];
 
     if (protocolHelperClient->client == NULL)
     {
@@ -199,14 +210,13 @@ protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostId, u
         {
             // Execute the protocol command
             protocolHelperClient->exec = execNew(
-                cfgExe(), protocolLocalParam(protocolStorageType, hostId, protocolId),
-                strNewFmt(PROTOCOL_SERVICE_LOCAL "-%u process", protocolId),
-                (TimeMSec)(cfgOptionDbl(cfgOptProtocolTimeout) * 1000));
+                cfgExe(), protocolLocalParam(protocolStorageType, hostIdx, processId),
+                strNewFmt(PROTOCOL_SERVICE_LOCAL "-%u process", processId), cfgOptionUInt64(cfgOptProtocolTimeout));
             execOpen(protocolHelperClient->exec);
 
             // Create protocol object
             protocolHelperClient->client = protocolClientNew(
-                strNewFmt(PROTOCOL_SERVICE_LOCAL "-%u protocol", protocolId),
+                strNewFmt(PROTOCOL_SERVICE_LOCAL "-%u protocol", processId),
                 PROTOCOL_SERVICE_LOCAL_STR, execIoRead(protocolHelperClient->exec), execIoWrite(protocolHelperClient->exec));
 
             protocolClientMove(protocolHelperClient->client, execMemContext(protocolHelperClient->exec));
@@ -217,15 +227,73 @@ protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostId, u
     FUNCTION_LOG_RETURN(PROTOCOL_CLIENT, protocolHelperClient->client);
 }
 
+/***********************************************************************************************************************************
+Free the protocol client and underlying exec'd process. Log any errors as warnings since it is not worth terminating the process
+while closing a local/remote that has already completed its work. The warning will be an indication that something is not right.
+***********************************************************************************************************************************/
+static void
+protocolHelperClientFree(ProtocolHelperClient *protocolHelperClient)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM_P(VOID, protocolHelperClient);
+    FUNCTION_LOG_END();
+
+    if (protocolHelperClient->client != NULL)
+    {
+        // Try to shutdown the protocol but only warn on error
+        TRY_BEGIN()
+        {
+            protocolClientFree(protocolHelperClient->client);
+        }
+        CATCH_ANY()
+        {
+            LOG_WARN(errorMessage());
+        }
+        TRY_END();
+
+        // Try to end the child process but only warn on error
+        TRY_BEGIN()
+        {
+            execFree(protocolHelperClient->exec);
+        }
+        CATCH_ANY()
+        {
+            LOG_WARN(errorMessage());
+        }
+        TRY_END();
+
+        protocolHelperClient->client = NULL;
+        protocolHelperClient->exec = NULL;
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/**********************************************************************************************************************************/
+void
+protocolLocalFree(unsigned int processId)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(UINT, processId);
+    FUNCTION_LOG_END();
+
+    if (protocolHelper.clientLocal != NULL)
+    {
+        ASSERT(processId <= protocolHelper.clientLocalSize);
+        protocolHelperClientFree(&protocolHelper.clientLocal[processId - 1]);
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Get the command line required for remote protocol execution
 ***********************************************************************************************************************************/
 static StringList *
-protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protocolId, unsigned int hostIdx)
+protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(ENUM, protocolStorageType);
-        FUNCTION_LOG_PARAM(UINT, protocolId);
         FUNCTION_LOG_PARAM(UINT, hostIdx);
     FUNCTION_LOG_END();
 
@@ -242,131 +310,110 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protoc
     strLstAddZ(result, "PasswordAuthentication=no");
 
     // Append port if specified
-    ConfigOption optHostPort = isRepo ? cfgOptRepoHostPort : cfgOptPgHostPort + hostIdx;
+    ConfigOption optHostPort = isRepo ? cfgOptRepoHostPort : cfgOptPgHostPort;
 
-    if (cfgOptionTest(optHostPort))
+    if (cfgOptionIdxTest(optHostPort, hostIdx))
     {
         strLstAddZ(result, "-p");
-        strLstAdd(result, strNewFmt("%u", cfgOptionUInt(optHostPort)));
+        strLstAdd(result, strNewFmt("%u", cfgOptionIdxUInt(optHostPort, hostIdx)));
     }
 
     // Append user/host
     strLstAdd(
         result,
         strNewFmt(
-            "%s@%s", strPtr(cfgOptionStr(isRepo ? cfgOptRepoHostUser : cfgOptPgHostUser + hostIdx)),
-            strPtr(cfgOptionStr(isRepo ? cfgOptRepoHost : cfgOptPgHost + hostIdx))));
+            "%s@%s", strZ(cfgOptionIdxStr(isRepo ? cfgOptRepoHostUser : cfgOptPgHostUser, hostIdx)),
+            strZ(cfgOptionIdxStr(isRepo ? cfgOptRepoHost : cfgOptPgHost, hostIdx))));
 
     // Option replacements
     KeyValue *optionReplace = kvNew();
 
     // Replace config options with the host versions
-    unsigned int optConfig = isRepo ? cfgOptRepoHostConfig : cfgOptPgHostConfig + hostIdx;
+    unsigned int optConfig = isRepo ? cfgOptRepoHostConfig : cfgOptPgHostConfig;
 
-    kvPut(optionReplace, VARSTR(CFGOPT_CONFIG_STR), cfgOptionSource(optConfig) != cfgSourceDefault  ? cfgOption(optConfig) : NULL);
+    kvPut(
+        optionReplace, VARSTR(CFGOPT_CONFIG_STR),
+        cfgOptionIdxSource(optConfig, hostIdx) != cfgSourceDefault ? VARSTR(cfgOptionIdxStr(optConfig, hostIdx)) : NULL);
 
-    unsigned int optConfigIncludePath = isRepo ? cfgOptRepoHostConfigIncludePath : cfgOptPgHostConfigIncludePath + hostIdx;
+    unsigned int optConfigIncludePath = isRepo ? cfgOptRepoHostConfigIncludePath : cfgOptPgHostConfigIncludePath;
 
     kvPut(
         optionReplace, VARSTR(CFGOPT_CONFIG_INCLUDE_PATH_STR),
-        cfgOptionSource(optConfigIncludePath) != cfgSourceDefault ? cfgOption(optConfigIncludePath) : NULL);
+        cfgOptionIdxSource(optConfigIncludePath, hostIdx) != cfgSourceDefault ?
+            VARSTR(cfgOptionIdxStr(optConfigIncludePath, hostIdx)) : NULL);
 
-    unsigned int optConfigPath = isRepo ? cfgOptRepoHostConfigPath : cfgOptPgHostConfigPath + hostIdx;
+    unsigned int optConfigPath = isRepo ? cfgOptRepoHostConfigPath : cfgOptPgHostConfigPath;
 
     kvPut(
         optionReplace, VARSTR(CFGOPT_CONFIG_PATH_STR),
-        cfgOptionSource(optConfigPath) != cfgSourceDefault ? cfgOption(optConfigPath) : NULL);
-
-    // Set local so host settings configured on the remote will not accidentally be picked up
-    kvPut(
-        optionReplace,
-        protocolStorageType == protocolStorageTypeRepo ? VARSTR(CFGOPT_REPO1_LOCAL_STR) : VARSTR(CFGOPT_PG1_LOCAL_STR),
-        BOOL_TRUE_VAR);
+        cfgOptionIdxSource(optConfigPath, hostIdx) != cfgSourceDefault ? VARSTR(cfgOptionIdxStr(optConfigPath, hostIdx)) : NULL);
 
     // Update/remove repo/pg options that are sent to the remote
-    ConfigDefineCommand commandDefId = cfgCommandDefIdFromId(cfgCommand());
-    const String *repoHostPrefix = STR(cfgDefOptionName(cfgDefOptRepoHost));
-    const String *repoPrefix = strNewFmt("%s-", PROTOCOL_REMOTE_TYPE_REPO);
-    const String *pgHostPrefix = STR(cfgDefOptionName(cfgDefOptPgHost));
-    const String *pgPrefix = strNewFmt("%s-", PROTOCOL_REMOTE_TYPE_PG);
-
     for (ConfigOption optionId = 0; optionId < CFG_OPTION_TOTAL; optionId++)
     {
-        ConfigDefineOption optionDefId = cfgOptionDefIdFromId(optionId);
-        const String *optionDefName = STR(cfgDefOptionName(optionDefId));
+        // Skip options that are not part of a group
+        if (!cfgOptionGroup(optionId))
+            continue;
+
         bool remove = false;
+        bool skipHostZero = false;
 
-        // Remove repo host options that are not needed on the remote.  The remote is not expecting to see host settings and it
-        // could get confused about the locality of the repo, i.e. local or remote.
-        if (strBeginsWith(optionDefName, repoHostPrefix))
-        {
-            remove = true;
-        }
         // Remove repo options when the remote type is pg since they won't be used
-        else if (strBeginsWith(optionDefName, repoPrefix))
+        if (cfgOptionGroupId(optionId) == cfgOptGrpRepo)
         {
-            if (protocolStorageType == protocolStorageTypePg)
-                remove = true;
+            remove = protocolStorageType == protocolStorageTypePg;
         }
-        // Remove pg host options that are not needed on the remote.  The remote is not expecting to see host settings and it could
-        // get confused about the locality of pg, i.e. local or remote.
-        else if (strBeginsWith(optionDefName, pgHostPrefix))
-        {
-            remove = true;
-        }
-        else if (strBeginsWith(optionDefName, pgPrefix))
+        // Remove pg host options that are not needed on the remote
+        else
         {
+            ASSERT(cfgOptionGroupId(optionId) == cfgOptGrpPg);
+
             // Remove unrequired/defaulted pg options when the remote type is repo since they won't be used
             if (protocolStorageType == protocolStorageTypeRepo)
             {
-                remove = !cfgDefOptionRequired(commandDefId, optionDefId) || cfgDefOptionDefault(commandDefId, optionDefId) != NULL;
+                remove = !cfgParseOptionRequired(cfgCommand(), optionId) || cfgParseOptionDefault(cfgCommand(), optionId) != NULL;
             }
-            // Else move/remove pg options with index > 0 since they won't be used
-            else if (cfgOptionIndex(optionId) > 0)
+            // Move pg options to host index 0 (key 1) so they will be in the default index on the remote host
+            else
             {
-                // If the option index matches the host-id then this is a pg option that the remote needs.  Since the remote expects
-                // to find pg options in index 0, copy the option to index 0.
-                if (cfgOptionIndex(optionId) == hostIdx)
+                if (hostIdx != 0)
                 {
                     kvPut(
-                        optionReplace, VARSTRZ(cfgOptionName(optionId - hostIdx)),
-                        cfgOptionSource(optionId) != cfgSourceDefault ? cfgOption(optionId) : NULL);
+                        optionReplace, VARSTRZ(cfgOptionIdxName(optionId, 0)),
+                        cfgOptionIdxSource(optionId, hostIdx) != cfgSourceDefault ? cfgOptionIdx(optionId, hostIdx) : NULL);
                 }
 
-                // Remove pg options that are not needed on the remote.  The remote is only going to look at index 0 so the options
-                // in higher indexes will not be used and just add clutter which makes debugging harder.
                 remove = true;
+                skipHostZero = true;
             }
         }
 
         // Remove options that have been marked for removal if they are not already null or invalid. This is more efficient because
         // cfgExecParam() won't have to search through as large a list looking for overrides.
-        if (remove && cfgOptionTest(optionId))
-            kvPut(optionReplace, VARSTRZ(cfgOptionName(optionId)), NULL);
+        if (remove)
+        {
+            // Loop through option indexes
+            for (unsigned int optionIdx = 0; optionIdx < cfgOptionIdxTotal(optionId); optionIdx++)
+            {
+                if (cfgOptionIdxTest(optionId, optionIdx) && !(skipHostZero && optionIdx == 0))
+                    kvPut(optionReplace, VARSTRZ(cfgOptionIdxName(optionId, optionIdx)), NULL);
+            }
+        }
     }
 
-    // Don't pass host-id to the remote.  The host will always be in index 0.
-    kvPut(optionReplace, VARSTR(CFGOPT_HOST_ID_STR), NULL);
+    // Set repo default so the remote only operates on a single repo
+    if (protocolStorageType == protocolStorageTypeRepo)
+        kvPut(optionReplace, VARSTRDEF(CFGOPT_REPO), VARUINT(cfgOptionGroupIdxToKey(cfgOptGrpRepo, hostIdx)));
 
-    // Add the process id (or use the current process id if it is valid)
+    // Add the process id if not set. This means that the remote is being started from the main process and should always get a
+    // process id of 0.
     if (!cfgOptionTest(cfgOptProcess))
-        kvPut(optionReplace, VARSTR(CFGOPT_PROCESS_STR), VARINT((int)protocolId));
+        kvPut(optionReplace, VARSTR(CFGOPT_PROCESS_STR), VARINT(0));
 
     // Don't pass log-path or lock-path since these are host specific
     kvPut(optionReplace, VARSTR(CFGOPT_LOG_PATH_STR), NULL);
     kvPut(optionReplace, VARSTR(CFGOPT_LOCK_PATH_STR), NULL);
 
-    // ??? Don't pass restore options which the remote doesn't need and are likely to contain spaces because they might get mangled
-    // on the way to the remote depending on how SSH is set up on the server.  This code should be removed when option passing with
-    // spaces is resolved.
-    kvPut(optionReplace, VARSTR(CFGOPT_TYPE_STR), NULL);
-    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_STR), NULL);
-    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_EXCLUSIVE_STR), NULL);
-    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_ACTION_STR), NULL);
-    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_STR), NULL);
-    kvPut(optionReplace, VARSTR(CFGOPT_TARGET_TIMELINE_STR), NULL);
-    kvPut(optionReplace, VARSTR(CFGOPT_RECOVERY_OPTION_STR), NULL);
-
     // Only enable file logging on the remote when requested
     kvPut(
         optionReplace, VARSTR(CFGOPT_LOG_LEVEL_FILE_STR),
@@ -382,7 +429,7 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protoc
     kvPut(optionReplace, VARSTR(CFGOPT_REMOTE_TYPE_STR), VARSTR(protocolStorageTypeStr(protocolStorageType)));
 
     StringList *commandExec = cfgExecParam(cfgCommand(), cfgCmdRoleRemote, optionReplace, false, true);
-    strLstInsert(commandExec, 0, cfgOptionStr(isRepo ? cfgOptRepoHostCmd : cfgOptPgHostCmd + hostIdx));
+    strLstInsert(commandExec, 0, cfgOptionIdxStr(isRepo ? cfgOptRepoHostCmd : cfgOptPgHostCmd, hostIdx));
     strLstAdd(result, strLstJoin(commandExec, " "));
 
     FUNCTION_LOG_RETURN(STRING_LIST, result);
@@ -390,15 +437,13 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int protoc
 
 /**********************************************************************************************************************************/
 ProtocolClient *
-protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
+protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(ENUM, protocolStorageType);
-        FUNCTION_LOG_PARAM(UINT, hostId);
+        FUNCTION_LOG_PARAM(UINT, hostIdx);
     FUNCTION_LOG_END();
 
-    ASSERT(hostId > 0);
-
     // Is this a repo remote?
     bool isRepo = protocolStorageType == protocolStorageTypeRepo;
 
@@ -409,12 +454,7 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
     {
         MEM_CONTEXT_BEGIN(protocolHelper.memContext)
         {
-            // The number of remotes allowed is the greater of allowed repo or pg configs + 1 (0 is reserved for connections from
-            // the main process).  Since these are static and only one will be true it presents a problem for coverage.  We think
-            // that pg remotes will always be greater but we'll protect that assumption with an assertion.
-            ASSERT(cfgDefOptionIndexTotal(cfgDefOptPgPath) >= cfgDefOptionIndexTotal(cfgDefOptRepoPath));
-
-            protocolHelper.clientRemoteSize = cfgDefOptionIndexTotal(cfgDefOptPgPath) + 1;
+            protocolHelper.clientRemoteSize = cfgOptionGroupIdxTotal(isRepo ? cfgOptGrpRepo : cfgOptGrpPg) + 1;
             protocolHelper.clientRemote = memNew(protocolHelper.clientRemoteSize * sizeof(ProtocolHelperClient));
 
             for (unsigned int clientIdx = 0; clientIdx < protocolHelper.clientRemoteSize; clientIdx++)
@@ -426,51 +466,48 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
     // Determine protocol id for the remote.  If the process option is set then use that since we want the remote protocol id to
     // match the local protocol id. Otherwise set to 0 since the remote is being started from a main process and there should only
     // be one remote per host.
-    unsigned int protocolId = 0;
-
-    // Use hostId to determine where to cache to remote
-    unsigned int protocolIdx = hostId - 1;
+    unsigned int processId = 0;
 
     if (cfgOptionTest(cfgOptProcess))
-        protocolId = cfgOptionUInt(cfgOptProcess);
+        processId = cfgOptionUInt(cfgOptProcess);
 
-    CHECK(protocolIdx < protocolHelper.clientRemoteSize);
+    CHECK(hostIdx < protocolHelper.clientRemoteSize);
 
     // Create protocol object
-    ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientRemote[protocolIdx];
+    ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientRemote[hostIdx];
 
     if (protocolHelperClient->client == NULL)
     {
         MEM_CONTEXT_BEGIN(protocolHelper.memContext)
         {
-            unsigned int optHost = isRepo ? cfgOptRepoHost : cfgOptPgHost + hostId - 1;
+            unsigned int optHost = isRepo ? cfgOptRepoHost : cfgOptPgHost;
 
             // Execute the protocol command
             protocolHelperClient->exec = execNew(
-                cfgOptionStr(cfgOptCmdSsh), protocolRemoteParam(protocolStorageType, protocolId, hostId - 1),
-                strNewFmt(PROTOCOL_SERVICE_REMOTE "-%u process on '%s'", protocolId, strPtr(cfgOptionStr(optHost))),
-                (TimeMSec)(cfgOptionDbl(cfgOptProtocolTimeout) * 1000));
+                cfgOptionStr(cfgOptCmdSsh), protocolRemoteParam(protocolStorageType, hostIdx),
+                strNewFmt(PROTOCOL_SERVICE_REMOTE "-%u process on '%s'", processId, strZ(cfgOptionIdxStr(optHost, hostIdx))),
+                cfgOptionUInt64(cfgOptProtocolTimeout));
             execOpen(protocolHelperClient->exec);
 
             // Create protocol object
             protocolHelperClient->client = protocolClientNew(
-                strNewFmt(PROTOCOL_SERVICE_REMOTE "-%u protocol on '%s'", protocolId, strPtr(cfgOptionStr(optHost))),
+                strNewFmt(PROTOCOL_SERVICE_REMOTE "-%u protocol on '%s'", processId, strZ(cfgOptionIdxStr(optHost, hostIdx))),
                 PROTOCOL_SERVICE_REMOTE_STR, execIoRead(protocolHelperClient->exec), execIoWrite(protocolHelperClient->exec));
 
             // Get cipher options from the remote if none are locally configured
-            if (isRepo && strEq(cfgOptionStr(cfgOptRepoCipherType), CIPHER_TYPE_NONE_STR))
+            if (isRepo && strEq(cfgOptionIdxStr(cfgOptRepoCipherType, hostIdx), CIPHER_TYPE_NONE_STR))
             {
                 // Options to query
                 VariantList *param = varLstNew();
-                varLstAdd(param, varNewStr(CFGOPT_REPO1_CIPHER_TYPE_STR));
-                varLstAdd(param, varNewStr(CFGOPT_REPO1_CIPHER_PASS_STR));
+                varLstAdd(param, varNewStrZ(cfgOptionIdxName(cfgOptRepoCipherType, hostIdx)));
+                varLstAdd(param, varNewStrZ(cfgOptionIdxName(cfgOptRepoCipherPass, hostIdx)));
 
                 VariantList *optionList = configProtocolOption(protocolHelperClient->client, param);
 
                 if (!strEq(varStr(varLstGet(optionList, 0)), CIPHER_TYPE_NONE_STR))
                 {
-                    cfgOptionSet(cfgOptRepoCipherType, cfgSourceConfig, varLstGet(optionList, 0));
-                    cfgOptionSet(cfgOptRepoCipherPass, cfgSourceConfig, varLstGet(optionList, 1));
+                    cfgOptionIdxSet(cfgOptRepoCipherType, hostIdx, cfgSourceConfig, varLstGet(optionList, 0));
+                    cfgOptionIdxSet(cfgOptRepoCipherPass, hostIdx, cfgSourceConfig, varLstGet(optionList, 1));
                 }
             }
 
@@ -484,27 +521,14 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostId)
 
 /**********************************************************************************************************************************/
 void
-protocolRemoteFree(unsigned int hostId)
+protocolRemoteFree(unsigned int hostIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(UINT, hostId);
+        FUNCTION_LOG_PARAM(UINT, hostIdx);
     FUNCTION_LOG_END();
 
-    ASSERT(hostId > 0);
-
     if (protocolHelper.clientRemote != NULL)
-    {
-        ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientRemote[hostId - 1];
-
-        if (protocolHelperClient->client != NULL)
-        {
-            protocolClientFree(protocolHelperClient->client);
-            execFree(protocolHelperClient->exec);
-
-            protocolHelperClient->client = NULL;
-            protocolHelperClient->exec = NULL;
-        }
-    }
+        protocolHelperClientFree(&protocolHelper.clientRemote[hostIdx]);
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -544,7 +568,7 @@ protocolStorageTypeEnum(const String *type)
     else if (strEq(type, PROTOCOL_REMOTE_TYPE_REPO_STR))
         FUNCTION_TEST_RETURN(protocolStorageTypeRepo);
 
-    THROW_FMT(AssertError, "invalid protocol storage type '%s'", strPtr(type));
+    THROW_FMT(AssertError, "invalid protocol storage type '%s'", strZ(type));
 }
 
 const String *
@@ -576,21 +600,11 @@ protocolFree(void)
     {
         // Free remotes
         for (unsigned int clientIdx = 0; clientIdx < protocolHelper.clientRemoteSize; clientIdx++)
-            protocolRemoteFree(clientIdx + 1);
+            protocolRemoteFree(clientIdx);
 
         // Free locals
-        for (unsigned int clientIdx  = 0; clientIdx < protocolHelper.clientLocalSize; clientIdx++)
-        {
-            ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientLocal[clientIdx];
-
-            if (protocolHelperClient->client != NULL)
-            {
-                protocolClientFree(protocolHelperClient->client);
-                execFree(protocolHelperClient->exec);
-
-                *protocolHelperClient = (ProtocolHelperClient){.exec = NULL};
-            }
-        }
+        for (unsigned int clientIdx = 1; clientIdx <= protocolHelper.clientLocalSize; clientIdx++)
+            protocolLocalFree(clientIdx);
     }
 
     FUNCTION_LOG_RETURN_VOID();
