@@ -11,6 +11,8 @@ Azure Storage
 #include "common/debug.h"
 #include "common/io/http/client.h"
 #include "common/io/http/common.h"
+#include "common/io/socket/client.h"
+#include "common/io/tls/client.h"
 #include "common/log.h"
 #include "common/memContext.h"
 #include "common/regExp.h"
@@ -25,12 +27,6 @@ Storage type
 ***********************************************************************************************************************************/
 STRING_EXTERN(STORAGE_AZURE_TYPE_STR,                               STORAGE_AZURE_TYPE);
 
-/***********************************************************************************************************************************
-Azure default hosts
-***********************************************************************************************************************************/
-#define AZURE_HOST                                                  "core.windows.net"
-#define AZURE_BLOB_HOST                                             "blob." AZURE_HOST
-
 /***********************************************************************************************************************************
 Azure http headers
 ***********************************************************************************************************************************/
@@ -132,7 +128,7 @@ storageAzureAuth(
                 if (!strBeginsWithZ(headerKey, "x-ms-"))
                     continue;
 
-                strCatFmt(headerCanonical, "%s:%s\n", strPtr(headerKey), strPtr(httpHeaderGet(httpHeader, headerKey)));
+                strCatFmt(headerCanonical, "%s:%s\n", strZ(headerKey), strZ(httpHeaderGet(httpHeader, headerKey)));
             }
 
             // Generate canonical query
@@ -141,13 +137,13 @@ storageAzureAuth(
             if (query != NULL)
             {
                 StringList *queryKeyList = httpQueryList(query);
-                ASSERT(strLstSize(queryKeyList) > 0);
+                ASSERT(!strLstEmpty(queryKeyList));
 
                 for (unsigned int queryKeyIdx = 0; queryKeyIdx < strLstSize(queryKeyList); queryKeyIdx++)
                 {
                     const String *queryKey = strLstGet(queryKeyList, queryKeyIdx);
 
-                    strCatFmt(queryCanonical, "\n%s:%s", strPtr(queryKey), strPtr(httpQueryGet(query, queryKey)));
+                    strCatFmt(queryCanonical, "\n%s:%s", strZ(queryKey), strZ(httpQueryGet(query, queryKey)));
                 }
             }
 
@@ -171,12 +167,12 @@ storageAzureAuth(
                 "%s"                                                    // Canonicalized headers
                 "/%s%s"                                                 // Canonicalized account/uri
                 "%s",                                                   // Canonicalized query
-                strPtr(verb), strEq(contentLength, ZERO_STR) ? "" : strPtr(contentLength), contentMd5 == NULL ? "" : strPtr(contentMd5),
-                strPtr(dateTime), strPtr(headerCanonical), strPtr(this->account), strPtr(uri), strPtr(queryCanonical));
+                strZ(verb), strEq(contentLength, ZERO_STR) ? "" : strZ(contentLength), contentMd5 == NULL ? "" : strZ(contentMd5),
+                strZ(dateTime), strZ(headerCanonical), strZ(this->account), strZ(uri), strZ(queryCanonical));
 
             // Generate authorization header
-            Buffer *keyBin = bufNew(decodeToBinSize(encodeBase64, strPtr(this->sharedKey)));
-            decodeToBin(encodeBase64, strPtr(this->sharedKey), bufPtr(keyBin));
+            Buffer *keyBin = bufNew(decodeToBinSize(encodeBase64, strZ(this->sharedKey)));
+            decodeToBin(encodeBase64, strZ(this->sharedKey), bufPtr(keyBin));
             bufUsedSet(keyBin, bufSize(keyBin));
 
             char authHmacBase64[45];
@@ -185,7 +181,7 @@ storageAzureAuth(
                 HASH_TYPE_SHA256_SIZE, authHmacBase64);
 
             httpHeaderPut(
-                httpHeader, HTTP_HEADER_AUTHORIZATION_STR, strNewFmt("SharedKey %s:%s", strPtr(this->account), authHmacBase64));
+                httpHeader, HTTP_HEADER_AUTHORIZATION_STR, strNewFmt("SharedKey %s:%s", strZ(this->account), authHmacBase64));
         }
         // SAS authentication
         else
@@ -219,7 +215,7 @@ storageAzureRequestAsync(StorageAzure *this, const String *verb, StorageAzureReq
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Prepend uri prefix
-        param.uri = param.uri == NULL ? this->uriPrefix : strNewFmt("%s%s", strPtr(this->uriPrefix), strPtr(param.uri));
+        param.uri = param.uri == NULL ? this->uriPrefix : strNewFmt("%s%s", strZ(this->uriPrefix), strZ(param.uri));
 
         // Create header list and add content length
         HttpHeader *requestHeader = param.header == NULL ?
@@ -228,7 +224,7 @@ storageAzureRequestAsync(StorageAzure *this, const String *verb, StorageAzureReq
         // Set content length
         httpHeaderAdd(
             requestHeader, HTTP_HEADER_CONTENT_LENGTH_STR,
-            param.content == NULL || bufUsed(param.content) == 0 ? ZERO_STR : strNewFmt("%zu", bufUsed(param.content)));
+            param.content == NULL || bufEmpty(param.content) ? ZERO_STR : strNewFmt("%zu", bufUsed(param.content)));
 
         // Calculate content-md5 header if there is content
         if (param.content != NULL)
@@ -335,15 +331,13 @@ storageAzureListInternal(
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        const String *marker = NULL;
-
         // Build the base prefix by stripping off the initial /
         const String *basePrefix;
 
         if (strSize(path) == 1)
             basePrefix = EMPTY_STR;
         else
-            basePrefix = strNewFmt("%s/", strPtr(strSub(path, 1)));
+            basePrefix = strNewFmt("%s/", strZ(strSub(path, 1)));
 
         // Get the expression prefix when possible to limit initial results
         const String *expressionPrefix = regExpPrefix(expression);
@@ -358,38 +352,65 @@ storageAzureListInternal(
             if (strEmpty(basePrefix))
                 queryPrefix = expressionPrefix;
             else
-                queryPrefix = strNewFmt("%s%s", strPtr(basePrefix), strPtr(expressionPrefix));
+                queryPrefix = strNewFmt("%s%s", strZ(basePrefix), strZ(expressionPrefix));
         }
 
-        // Loop as long as a continuation token returned
+        // Create query
+        HttpQuery *query = httpQueryNewP();
+
+        // Add the delimiter to not recurse
+        if (!recurse)
+            httpQueryAdd(query, AZURE_QUERY_DELIMITER_STR, FSLASH_STR);
+
+        // Add resource type
+        httpQueryAdd(query, AZURE_QUERY_RESTYPE_STR, AZURE_QUERY_VALUE_CONTAINER_STR);
+
+        // Add list comp
+        httpQueryAdd(query, AZURE_QUERY_COMP_STR, AZURE_QUERY_VALUE_LIST_STR);
+
+        // Don't specify empty prefix because it is the default
+        if (!strEmpty(queryPrefix))
+            httpQueryAdd(query, AZURE_QUERY_PREFIX_STR, queryPrefix);
+
+        // Loop as long as a continuation marker returned
+        HttpRequest *request = NULL;
+
         do
         {
             // Use an inner mem context here because we could potentially be retrieving millions of files so it is a good idea to
             // free memory at regular intervals
             MEM_CONTEXT_TEMP_BEGIN()
             {
-                HttpQuery *query = httpQueryNewP();
+                HttpResponse *response = NULL;
 
-                // Add continuation token from the prior loop if any
-                if (marker != NULL)
-                    httpQueryAdd(query, AZURE_QUERY_MARKER_STR, marker);
+                // If there is an outstanding async request then wait for the response
+                if (request != NULL)
+                {
+                    response = storageAzureResponseP(request);
 
-                // Add the delimiter to not recurse
-                if (!recurse)
-                    httpQueryAdd(query, AZURE_QUERY_DELIMITER_STR, FSLASH_STR);
+                    httpRequestFree(request);
+                    request = NULL;
+                }
+                // Else get the response immediately from a sync request
+                else
+                    response = storageAzureRequestP(this, HTTP_VERB_GET_STR, .query = query);
 
-                // Add resource type
-                httpQueryAdd(query, AZURE_QUERY_RESTYPE_STR, AZURE_QUERY_VALUE_CONTAINER_STR);
+                XmlNode *xmlRoot = xmlDocumentRoot(xmlDocumentNewBuf(httpResponseContent(response)));
 
-                // Add list comp
-                httpQueryAdd(query, AZURE_QUERY_COMP_STR, AZURE_QUERY_VALUE_LIST_STR);
+                // If a continuation marker exists then send an async request to get more data
+                const String *continuationMarker = xmlNodeContent(xmlNodeChild(xmlRoot, AZURE_XML_TAG_NEXT_MARKER_STR, false));
 
-                // Don't specify empty prefix because it is the default
-                if (!strEmpty(queryPrefix))
-                    httpQueryAdd(query, AZURE_QUERY_PREFIX_STR, queryPrefix);
+                if (!strEq(continuationMarker, EMPTY_STR))
+                {
+                    httpQueryPut(query, AZURE_QUERY_MARKER_STR, continuationMarker);
 
-                XmlNode *xmlRoot = xmlDocumentRoot(
-                    xmlDocumentNewBuf(httpResponseContent(storageAzureRequestP(this, HTTP_VERB_GET_STR, .query = query))));
+                    // Store request in the outer temp context
+                    MEM_CONTEXT_PRIOR_BEGIN()
+                    {
+                        request = storageAzureRequestAsyncP(this, HTTP_VERB_GET_STR, .query = query);
+                    }
+                    MEM_CONTEXT_PRIOR_END();
+                }
 
                 // Get subpath list
                 XmlNode *blobs = xmlNodeChild(xmlRoot, AZURE_XML_TAG_BLOBS_STR, true);
@@ -426,17 +447,10 @@ storageAzureListInternal(
                     callback(
                         this, callbackData, file, storageTypeFile, xmlNodeChild(fileNode, AZURE_XML_TAG_PROPERTIES_STR, true));
                 }
-
-                // Get the continuation token and store it in the outer temp context
-                MEM_CONTEXT_PRIOR_BEGIN()
-                {
-                    marker = xmlNodeContent(xmlNodeChild(xmlRoot, AZURE_XML_TAG_NEXT_MARKER_STR, false));
-                }
-                MEM_CONTEXT_PRIOR_END();
             }
             MEM_CONTEXT_TEMP_END();
         }
-        while (!strEq(marker, EMPTY_STR));
+        while (request != NULL);
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -469,7 +483,7 @@ storageAzureInfo(THIS_VOID, const String *file, StorageInfoLevel level, StorageI
     if (result.level >= storageInfoLevelBasic && result.exists)
     {
         result.type = storageTypeFile;
-        result.size = cvtZToUInt64(strPtr(httpHeaderGet(httpResponseHeader(httpResponse), HTTP_HEADER_CONTENT_LENGTH_STR)));
+        result.size = cvtZToUInt64(strZ(httpHeaderGet(httpResponseHeader(httpResponse), HTTP_HEADER_CONTENT_LENGTH_STR)));
         result.timeModified = httpDateToTime(httpHeaderGet(httpResponseHeader(httpResponse), HTTP_HEADER_LAST_MODIFIED_STR));
     }
 
@@ -517,7 +531,7 @@ storageAzureInfoListCallback(StorageAzure *this, void *callbackData, const Strin
         {
             ASSERT(xml != NULL);
 
-            info.size =  cvtZToUInt64(strPtr(xmlNodeContent(xmlNodeChild(xml, AZURE_XML_TAG_CONTENT_LENGTH_STR, true))));
+            info.size =  cvtZToUInt64(strZ(xmlNodeContent(xmlNodeChild(xml, AZURE_XML_TAG_CONTENT_LENGTH_STR, true))));
             info.timeModified = httpDateToTime(xmlNodeContent(xmlNodeChild(xml, AZURE_XML_TAG_LAST_MODIFIED_STR, true)));
         }
     }
@@ -601,7 +615,9 @@ storageAzureNewWrite(THIS_VOID, const String *file, StorageInterfaceNewWritePara
 /**********************************************************************************************************************************/
 typedef struct StorageAzurePathRemoveData
 {
-    const String *path;
+    MemContext *memContext;                                         // Mem context to create requests in
+    HttpRequest *request;                                           // Async remove request
+    const String *path;                                             // Root path of remove
 } StorageAzurePathRemoveData;
 
 static void
@@ -619,11 +635,26 @@ storageAzurePathRemoveCallback(StorageAzure *this, void *callbackData, const Str
     ASSERT(callbackData != NULL);
     ASSERT(name != NULL);
 
+    StorageAzurePathRemoveData *data = (StorageAzurePathRemoveData *)callbackData;
+
+    // Get response from prior async request
+    if (data->request != NULL)
+    {
+        storageAzureResponseP(data->request, .allowMissing = true);
+
+        httpRequestFree(data->request);
+        data->request = NULL;
+    }
+
     // Only delete files since paths don't really exist
     if (type == storageTypeFile)
     {
-        StorageAzurePathRemoveData *data = (StorageAzurePathRemoveData *)callbackData;
-        storageInterfaceRemoveP(this, strNewFmt("%s/%s", strEq(data->path, FSLASH_STR) ? "" : strPtr(data->path), strPtr(name)));
+        MEM_CONTEXT_BEGIN(data->memContext)
+        {
+            data->request = storageAzureRequestAsyncP(
+                this, HTTP_VERB_DELETE_STR, strNewFmt("%s/%s", strEq(data->path, FSLASH_STR) ? "" : strZ(data->path), strZ(name)));
+        }
+        MEM_CONTEXT_END();
     }
 
     FUNCTION_TEST_RETURN_VOID();
@@ -646,8 +677,12 @@ storageAzurePathRemove(THIS_VOID, const String *path, bool recurse, StorageInter
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        StorageAzurePathRemoveData data = {.path = path};
+        StorageAzurePathRemoveData data = {.memContext = memContextCurrent(), .path = path};
         storageAzureListInternal(this, path, NULL, true, storageAzurePathRemoveCallback, &data);
+
+        // Check response on last async request
+        if (data.request != NULL)
+            storageAzureResponseP(data.request, .allowMissing = true);
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -689,8 +724,8 @@ static const StorageInterface storageInterfaceAzure =
 Storage *
 storageAzureNew(
     const String *path, bool write, StoragePathExpressionCallback pathExpressionFunction, const String *container,
-    const String *account, StorageAzureKeyType keyType, const String *key, size_t blockSize, const String *host, unsigned int port,
-    TimeMSec timeout, bool verifyPeer, const String *caFile, const String *caPath)
+    const String *account, StorageAzureKeyType keyType, const String *key, size_t blockSize, const String *host,
+    const String *endpoint, unsigned int port, TimeMSec timeout, bool verifyPeer, const String *caFile, const String *caPath)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, path);
@@ -702,6 +737,7 @@ storageAzureNew(
         FUNCTION_TEST_PARAM(STRING, key);
         FUNCTION_LOG_PARAM(SIZE, blockSize);
         FUNCTION_LOG_PARAM(STRING, host);
+        FUNCTION_LOG_PARAM(STRING, endpoint);
         FUNCTION_LOG_PARAM(UINT, port);
         FUNCTION_LOG_PARAM(TIME_MSEC, timeout);
         FUNCTION_LOG_PARAM(BOOL, verifyPeer);
@@ -728,9 +764,8 @@ storageAzureNew(
             .container = strDup(container),
             .account = strDup(account),
             .blockSize = blockSize,
-            .host = host == NULL ? strNewFmt("%s." AZURE_BLOB_HOST, strPtr(account)) : host,
-            .uriPrefix = host == NULL ?
-                strNewFmt("/%s", strPtr(container)) : strNewFmt("/%s/%s", strPtr(account), strPtr(container)),
+            .host = host == NULL ? strNewFmt("%s.%s", strZ(account), strZ(endpoint)) : host,
+            .uriPrefix = host == NULL ? strNewFmt("/%s", strZ(container)) : strNewFmt("/%s/%s", strZ(account), strZ(container)),
         };
 
         // Store shared key or parse sas query
@@ -740,7 +775,8 @@ storageAzureNew(
             driver->sasKey = httpQueryNewStr(key);
 
         // Create the http client used to service requests
-        driver->httpClient = httpClientNew(driver->host, port, timeout, verifyPeer, caFile, caPath);
+        driver->httpClient = httpClientNew(
+            tlsClientNew(sckClientNew(driver->host, port, timeout), driver->host, timeout, verifyPeer, caFile, caPath), timeout);
 
         // Create list of redacted headers
         driver->headerRedactList = strLstNew();
