@@ -9,7 +9,6 @@ Storage Helper
 #include "common/io/io.h"
 #include "common/memContext.h"
 #include "common/regExp.h"
-#include "config/define.h"
 #include "config/config.h"
 #include "protocol/helper.h"
 #include "storage/azure/storage.h"
@@ -48,8 +47,8 @@ static struct StorageHelper
     Storage *storageLocalWrite;                                     // Local write storage
     Storage **storagePg;                                            // PostgreSQL read-only storage
     Storage **storagePgWrite;                                       // PostgreSQL write storage
-    Storage *storageRepo;                                           // Repository read-only storage
-    Storage *storageRepoWrite;                                      // Repository write storage
+    Storage **storageRepo;                                          // Repository read-only storage
+    Storage **storageRepoWrite;                                     // Repository write storage
     Storage *storageSpool;                                          // Spool read-only storage
     Storage *storageSpoolWrite;                                     // Spool write storage
 
@@ -164,26 +163,26 @@ storageLocalWrite(void)
 Get pg storage for the specified host id
 ***********************************************************************************************************************************/
 static Storage *
-storagePgGet(unsigned int hostId, bool write)
+storagePgGet(unsigned int pgIdx, bool write)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, hostId);
+        FUNCTION_TEST_PARAM(UINT, pgIdx);
         FUNCTION_TEST_PARAM(BOOL, write);
     FUNCTION_TEST_END();
 
     Storage *result = NULL;
 
     // Use remote storage
-    if (!pgIsLocal(hostId))
+    if (!pgIsLocal(pgIdx))
     {
         result = storageRemoteNew(
             STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, NULL,
-            protocolRemoteGet(protocolStorageTypePg, hostId), cfgOptionUInt(cfgOptCompressLevelNetwork));
+            protocolRemoteGet(protocolStorageTypePg, pgIdx), cfgOptionUInt(cfgOptCompressLevelNetwork));
     }
     // Use Posix storage
     else
     {
-        result = storagePosixNewP(cfgOptionStr(cfgOptPgPath + hostId - 1), .write = write);
+        result = storagePosixNewP(cfgOptionIdxStr(cfgOptPgPath, pgIdx), .write = write);
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -191,69 +190,69 @@ storagePgGet(unsigned int hostId, bool write)
 
 /**********************************************************************************************************************************/
 const Storage *
-storagePgId(unsigned int hostId)
+storagePgIdx(unsigned int pgIdx)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, hostId);
+        FUNCTION_TEST_PARAM(UINT, pgIdx);
     FUNCTION_TEST_END();
 
-    if (storageHelper.storagePg == NULL || storageHelper.storagePg[hostId - 1] == NULL)
+    if (storageHelper.storagePg == NULL || storageHelper.storagePg[pgIdx] == NULL)
     {
         storageHelperInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
             if (storageHelper.storagePg == NULL)
-                storageHelper.storagePg = memNewPtrArray(cfgDefOptionIndexTotal(cfgDefOptPgPath));
+                storageHelper.storagePg = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpPg));
 
-            storageHelper.storagePg[hostId - 1] = storagePgGet(hostId, false);
+            storageHelper.storagePg[pgIdx] = storagePgGet(pgIdx, false);
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storagePg[hostId - 1]);
+    FUNCTION_TEST_RETURN(storageHelper.storagePg[pgIdx]);
 }
 
 const Storage *
 storagePg(void)
 {
     FUNCTION_TEST_VOID();
-    FUNCTION_TEST_RETURN(storagePgId(cfgOptionTest(cfgOptHostId) ? cfgOptionUInt(cfgOptHostId) : 1));
+    FUNCTION_TEST_RETURN(storagePgIdx(cfgOptionGroupIdxDefault(cfgOptGrpPg)));
 }
 
 const Storage *
-storagePgIdWrite(unsigned int hostId)
+storagePgIdxWrite(unsigned int pgIdx)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, hostId);
+        FUNCTION_TEST_PARAM(UINT, pgIdx);
     FUNCTION_TEST_END();
 
     // Writes not allowed in dry-run mode
     if (!storageHelper.dryRunInit || storageHelper.dryRun)
         THROW(AssertError, WRITABLE_WHILE_DRYRUN);
 
-    if (storageHelper.storagePgWrite == NULL || storageHelper.storagePgWrite[hostId - 1] == NULL)
+    if (storageHelper.storagePgWrite == NULL || storageHelper.storagePgWrite[pgIdx] == NULL)
     {
         storageHelperInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
             if (storageHelper.storagePgWrite == NULL)
-                storageHelper.storagePgWrite = memNewPtrArray(cfgDefOptionIndexTotal(cfgDefOptPgPath));
+                storageHelper.storagePgWrite = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpPg));
 
-            storageHelper.storagePgWrite[hostId - 1] = storagePgGet(hostId, true);
+            storageHelper.storagePgWrite[pgIdx] = storagePgGet(pgIdx, true);
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storagePgWrite[hostId - 1]);
+    FUNCTION_TEST_RETURN(storageHelper.storagePgWrite[pgIdx]);
 }
 
 const Storage *
 storagePgWrite(void)
 {
     FUNCTION_TEST_VOID();
-    FUNCTION_TEST_RETURN(storagePgIdWrite(cfgOptionTest(cfgOptHostId) ? cfgOptionUInt(cfgOptHostId) : 1));
+    FUNCTION_TEST_RETURN(storagePgIdxWrite(cfgOptionGroupIdxDefault(cfgOptGrpPg)));
 }
 
 /***********************************************************************************************************************************
@@ -295,7 +294,7 @@ storageRepoPathExpression(const String *expression, const String *path)
     {
         // Construct the base path
         if (storageHelper.stanza != NULL)
-            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s", strPtr(storageHelper.stanza));
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s", strZ(storageHelper.stanza));
         else
             result = strNew(STORAGE_PATH_ARCHIVE);
 
@@ -306,25 +305,25 @@ storageRepoPathExpression(const String *expression, const String *path)
             String *file = strLstSize(pathSplit) == 2 ? strLstGet(pathSplit, 1) : NULL;
 
             if (file != NULL && regExpMatch(storageHelper.walRegExp, file))
-                strCatFmt(result, "/%s/%s/%s", strPtr(strLstGet(pathSplit, 0)), strPtr(strSubN(file, 0, 16)), strPtr(file));
+                strCatFmt(result, "/%s/%s/%s", strZ(strLstGet(pathSplit, 0)), strZ(strSubN(file, 0, 16)), strZ(file));
             else
-                strCatFmt(result, "/%s", strPtr(path));
+                strCatFmt(result, "/%s", strZ(path));
         }
     }
     else if (strEq(expression, STORAGE_REPO_BACKUP_STR))
     {
         // Construct the base path
         if (storageHelper.stanza != NULL)
-            result = strNewFmt(STORAGE_PATH_BACKUP "/%s", strPtr(storageHelper.stanza));
+            result = strNewFmt(STORAGE_PATH_BACKUP "/%s", strZ(storageHelper.stanza));
         else
             result = strNew(STORAGE_PATH_BACKUP);
 
         // Append subpath if provided
         if (path != NULL)
-            strCatFmt(result, "/%s", strPtr(path));
+            strCatFmt(result, "/%s", strZ(path));
     }
     else
-        THROW_FMT(AssertError, "invalid expression '%s'", strPtr(expression));
+        THROW_FMT(AssertError, "invalid expression '%s'", strZ(expression));
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -333,80 +332,95 @@ storageRepoPathExpression(const String *expression, const String *path)
 Get the repo storage
 ***********************************************************************************************************************************/
 static Storage *
-storageRepoGet(const String *type, bool write)
+storageRepoGet(unsigned int repoIdx, bool write)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STRING, type);
+        FUNCTION_TEST_PARAM(UINT, repoIdx);
         FUNCTION_TEST_PARAM(BOOL, write);
     FUNCTION_TEST_END();
 
-    ASSERT(type != NULL);
-
     Storage *result = NULL;
 
     // Use remote storage
-    if (!repoIsLocal())
+    if (!repoIsLocal(repoIdx))
     {
         result = storageRemoteNew(
             STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression,
-            protocolRemoteGet(protocolStorageTypeRepo, 1), cfgOptionUInt(cfgOptCompressLevelNetwork));
+            protocolRemoteGet(protocolStorageTypeRepo, repoIdx), cfgOptionUInt(cfgOptCompressLevelNetwork));
     }
     // Use Azure storage
-    else if (strEqZ(type, STORAGE_AZURE_TYPE))
-    {
-        result = storageAzureNew(
-            cfgOptionStr(cfgOptRepoPath), write, storageRepoPathExpression, cfgOptionStr(cfgOptRepoAzureContainer),
-            cfgOptionStr(cfgOptRepoAzureAccount),
-            strEqZ(cfgOptionStr(cfgOptRepoAzureKeyType), STORAGE_AZURE_KEY_TYPE_SHARED) ?
-                storageAzureKeyTypeShared : storageAzureKeyTypeSas,
-            cfgOptionStr(cfgOptRepoAzureKey), STORAGE_AZURE_BLOCKSIZE_MIN, cfgOptionStrNull(cfgOptRepoAzureHost),
-            cfgOptionUInt(cfgOptRepoAzurePort), ioTimeoutMs(), cfgOptionBool(cfgOptRepoAzureVerifyTls),
-            cfgOptionStrNull(cfgOptRepoAzureCaFile), cfgOptionStrNull(cfgOptRepoAzureCaPath));
-    }
-    // Use CIFS storage
-    else if (strEqZ(type, STORAGE_CIFS_TYPE))
-    {
-        result = storageCifsNew(
-            cfgOptionStr(cfgOptRepoPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression);
-    }
-    // Use Posix storage
-    else if (strEqZ(type, STORAGE_POSIX_TYPE))
-    {
-        result = storagePosixNewP(
-            cfgOptionStr(cfgOptRepoPath), .write = write, .pathExpressionFunction = storageRepoPathExpression);
-    }
-    // Use S3 storage
-    else if (strEqZ(type, STORAGE_S3_TYPE))
-    {
-        // Set the default port
-        unsigned int port = cfgOptionUInt(cfgOptRepoS3Port);
-
-        // Extract port from the endpoint and host if it is present
-        const String *endPoint = cfgOptionHostPort(cfgOptRepoS3Endpoint, &port);
-        const String *host = cfgOptionHostPort(cfgOptRepoS3Host, &port);
-
-        // If the port option was set explicitly then use it in preference to appended ports
-        if (cfgOptionSource(cfgOptRepoS3Port) != cfgSourceDefault)
-            port = cfgOptionUInt(cfgOptRepoS3Port);
-
-        result = storageS3New(
-            cfgOptionStr(cfgOptRepoPath), write, storageRepoPathExpression, cfgOptionStr(cfgOptRepoS3Bucket), endPoint,
-            strEqZ(cfgOptionStr(cfgOptRepoS3UriStyle), STORAGE_S3_URI_STYLE_HOST) ? storageS3UriStyleHost : storageS3UriStylePath,
-            cfgOptionStr(cfgOptRepoS3Region), cfgOptionStr(cfgOptRepoS3Key), cfgOptionStr(cfgOptRepoS3KeySecret),
-            cfgOptionStrNull(cfgOptRepoS3Token), STORAGE_S3_PARTSIZE_MIN, STORAGE_S3_DELETE_MAX, host, port, ioTimeoutMs(),
-            cfgOptionBool(cfgOptRepoS3VerifyTls), cfgOptionStrNull(cfgOptRepoS3CaFile), cfgOptionStrNull(cfgOptRepoS3CaPath));
-    }
     else
-        THROW_FMT(AssertError, "invalid storage type '%s'", strPtr(type));
+    {
+        const String *type = cfgOptionIdxStr(cfgOptRepoType, repoIdx);
+
+        if (strEqZ(type, STORAGE_AZURE_TYPE))
+        {
+            result = storageAzureNew(
+                cfgOptionIdxStr(cfgOptRepoPath, repoIdx), write, storageRepoPathExpression,
+                cfgOptionIdxStr(cfgOptRepoAzureContainer, repoIdx), cfgOptionIdxStr(cfgOptRepoAzureAccount, repoIdx),
+                strEqZ(cfgOptionIdxStr(cfgOptRepoAzureKeyType, repoIdx), STORAGE_AZURE_KEY_TYPE_SHARED) ?
+                    storageAzureKeyTypeShared : storageAzureKeyTypeSas,
+                cfgOptionIdxStr(cfgOptRepoAzureKey, repoIdx), STORAGE_AZURE_BLOCKSIZE_MIN,
+                cfgOptionIdxStrNull(cfgOptRepoAzureHost, repoIdx), cfgOptionIdxStr(cfgOptRepoAzureEndpoint, repoIdx),
+                cfgOptionIdxUInt(cfgOptRepoAzurePort, repoIdx), ioTimeoutMs(), cfgOptionIdxBool(cfgOptRepoAzureVerifyTls, repoIdx),
+                cfgOptionIdxStrNull(cfgOptRepoAzureCaFile, repoIdx),
+                cfgOptionIdxStrNull(cfgOptRepoAzureCaPath, repoIdx));
+        }
+        // Use CIFS storage
+        else if (strEqZ(type, STORAGE_CIFS_TYPE))
+        {
+            result = storageCifsNew(
+                cfgOptionIdxStr(cfgOptRepoPath, repoIdx), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write,
+                storageRepoPathExpression);
+        }
+        // Use Posix storage
+        else if (strEqZ(type, STORAGE_POSIX_TYPE))
+        {
+            result = storagePosixNewP(
+                cfgOptionIdxStr(cfgOptRepoPath, repoIdx), .write = write, .pathExpressionFunction = storageRepoPathExpression);
+        }
+        // Use S3 storage
+        else
+        {
+            // Storage must be S3
+            CHECK(strEqZ(type, STORAGE_S3_TYPE));
+
+            // Set the default port
+            unsigned int port = cfgOptionIdxUInt(cfgOptRepoS3Port, repoIdx);
+
+            // Extract port from the endpoint and host if it is present
+            const String *endPoint = cfgOptionIdxHostPort(cfgOptRepoS3Endpoint, repoIdx, &port);
+            const String *host = cfgOptionIdxHostPort(cfgOptRepoS3Host, repoIdx, &port);
+
+            // If the port option was set explicitly then use it in preference to appended ports
+            if (cfgOptionIdxSource(cfgOptRepoS3Port, repoIdx) != cfgSourceDefault)
+                port = cfgOptionIdxUInt(cfgOptRepoS3Port, repoIdx);
+
+            result = storageS3New(
+                cfgOptionIdxStr(cfgOptRepoPath, repoIdx), write, storageRepoPathExpression,
+                cfgOptionIdxStr(cfgOptRepoS3Bucket, repoIdx), endPoint,
+                strEqZ(cfgOptionIdxStr(cfgOptRepoS3UriStyle, repoIdx), STORAGE_S3_URI_STYLE_HOST) ?
+                    storageS3UriStyleHost : storageS3UriStylePath,
+                cfgOptionIdxStr(cfgOptRepoS3Region, repoIdx),
+                strEqZ(cfgOptionIdxStr(cfgOptRepoS3KeyType, repoIdx), STORAGE_S3_KEY_TYPE_SHARED) ?
+                    storageS3KeyTypeShared : storageS3KeyTypeAuto,
+                cfgOptionIdxStrNull(cfgOptRepoS3Key, repoIdx), cfgOptionIdxStrNull(cfgOptRepoS3KeySecret, repoIdx),
+                cfgOptionIdxStrNull(cfgOptRepoS3Token, repoIdx), cfgOptionIdxStrNull(cfgOptRepoS3Role, repoIdx),
+                STORAGE_S3_PARTSIZE_MIN, host, port, ioTimeoutMs(), cfgOptionIdxBool(cfgOptRepoS3VerifyTls, repoIdx),
+                cfgOptionIdxStrNull(cfgOptRepoS3CaFile, repoIdx), cfgOptionIdxStrNull(cfgOptRepoS3CaPath, repoIdx));
+        }
+    }
 
     FUNCTION_TEST_RETURN(result);
 }
 
 /**********************************************************************************************************************************/
 const Storage *
-storageRepo(void)
+storageRepoIdx(unsigned int repoIdx)
 {
-    FUNCTION_TEST_VOID();
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, repoIdx);
+    FUNCTION_TEST_END();
 
     if (storageHelper.storageRepo == NULL)
     {
@@ -416,18 +430,36 @@ storageRepo(void)
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageRepo = storageRepoGet(cfgOptionStr(cfgOptRepoType), false);
+            storageHelper.storageRepo = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpRepo));
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storageRepo);
+    if (storageHelper.storageRepo[repoIdx] == NULL)
+    {
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storageRepo[repoIdx] = storageRepoGet(repoIdx, false);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storageRepo[repoIdx]);
 }
 
 const Storage *
-storageRepoWrite(void)
+storageRepo(void)
 {
     FUNCTION_TEST_VOID();
+    FUNCTION_TEST_RETURN(storageRepoIdx(cfgOptionGroupIdxDefault(cfgOptGrpRepo)));
+}
+
+const Storage *
+storageRepoIdxWrite(unsigned int repoIdx)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, repoIdx);
+    FUNCTION_TEST_END();
 
     // Writes not allowed in dry-run mode
     if (!storageHelper.dryRunInit || storageHelper.dryRun)
@@ -441,12 +473,28 @@ storageRepoWrite(void)
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageRepoWrite = storageRepoGet(cfgOptionStr(cfgOptRepoType), true);
+            storageHelper.storageRepoWrite = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpRepo));
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storageRepoWrite);
+    if (storageHelper.storageRepoWrite[repoIdx] == NULL)
+    {
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storageRepoWrite[repoIdx] = storageRepoGet(repoIdx, true);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storageRepoWrite[repoIdx]);
+}
+
+const Storage *
+storageRepoWrite(void)
+{
+    FUNCTION_TEST_VOID();
+    FUNCTION_TEST_RETURN(storageRepoIdxWrite(cfgOptionGroupIdxDefault(cfgOptGrpRepo)));
 }
 
 /***********************************************************************************************************************************
@@ -468,19 +516,19 @@ storageSpoolPathExpression(const String *expression, const String *path)
     if (strEqZ(expression, STORAGE_SPOOL_ARCHIVE_IN))
     {
         if (path == NULL)
-            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/in", strPtr(storageHelper.stanza));
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/in", strZ(storageHelper.stanza));
         else
-            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/in/%s", strPtr(storageHelper.stanza), strPtr(path));
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/in/%s", strZ(storageHelper.stanza), strZ(path));
     }
     else if (strEqZ(expression, STORAGE_SPOOL_ARCHIVE_OUT))
     {
         if (path == NULL)
-            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/out", strPtr(storageHelper.stanza));
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/out", strZ(storageHelper.stanza));
         else
-            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/out/%s", strPtr(storageHelper.stanza), strPtr(path));
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/out/%s", strZ(storageHelper.stanza), strZ(path));
     }
     else
-        THROW_FMT(AssertError, "invalid expression '%s'", strPtr(expression));
+        THROW_FMT(AssertError, "invalid expression '%s'", strZ(expression));
 
     FUNCTION_TEST_RETURN(result);
 }
