@@ -1,5 +1,5 @@
 /***********************************************************************************************************************************
-S3 Storage File write
+S3 Storage File Write
 ***********************************************************************************************************************************/
 #include "build.auto.h"
 
@@ -7,7 +7,7 @@ S3 Storage File write
 #include "common/io/write.intern.h"
 #include "common/log.h"
 #include "common/memContext.h"
-#include "common/object.h"
+#include "common/type/object.h"
 #include "common/type/xml.h"
 #include "storage/s3/write.h"
 #include "storage/write.intern.h"
@@ -37,6 +37,7 @@ typedef struct StorageWriteS3
     StorageWriteInterface interface;                                // Interface
     StorageS3 *storage;                                             // Storage that created this object
 
+    HttpRequest *request;                                           // Async request
     size_t partSize;
     Buffer *partBuffer;
     const String *uploadId;
@@ -46,9 +47,9 @@ typedef struct StorageWriteS3
 /***********************************************************************************************************************************
 Macros for function logging
 ***********************************************************************************************************************************/
-#define FUNCTION_LOG_STORAGE_WRITE_S3_TYPE                                                                                  \
+#define FUNCTION_LOG_STORAGE_WRITE_S3_TYPE                                                                                         \
     StorageWriteS3 *
-#define FUNCTION_LOG_STORAGE_WRITE_S3_FORMAT(value, buffer, bufferSize)                                                     \
+#define FUNCTION_LOG_STORAGE_WRITE_S3_FORMAT(value, buffer, bufferSize)                                                            \
     objToLog(value, "StorageWriteS3", buffer, bufferSize)
 
 /***********************************************************************************************************************************
@@ -86,21 +87,48 @@ storageWriteS3Part(StorageWriteS3 *this)
         FUNCTION_LOG_PARAM(STORAGE_WRITE_S3, this);
     FUNCTION_LOG_END();
 
+    ASSERT(this != NULL);
+
+    // If there is an outstanding async request then wait for the response and store the part id
+    if (this->request != NULL)
+    {
+        strLstAdd(
+            this->uploadPartList, httpHeaderGet(httpResponseHeader(storageS3ResponseP(this->request)), HTTP_HEADER_ETAG_STR));
+        ASSERT(strLstGet(this->uploadPartList, strLstSize(this->uploadPartList) - 1) != NULL);
+
+        httpRequestFree(this->request);
+        this->request = NULL;
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+static void
+storageWriteS3PartAsync(StorageWriteS3 *this)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(STORAGE_WRITE_S3, this);
+    FUNCTION_LOG_END();
+
     ASSERT(this != NULL);
     ASSERT(this->partBuffer != NULL);
     ASSERT(bufSize(this->partBuffer) > 0);
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
+        // Complete prior async request, if any
+        storageWriteS3Part(this);
+
         // Get the upload id if we have not already
         if (this->uploadId == NULL)
         {
             // Initiate mult-part upload
             XmlNode *xmlRoot = xmlDocumentRoot(
                 xmlDocumentNewBuf(
-                    storageS3Request(
-                        this->storage, HTTP_VERB_POST_STR, this->interface.name,
-                        httpQueryAdd(httpQueryNew(), S3_QUERY_UPLOADS_STR, EMPTY_STR), NULL, true, false).response));
+                    httpResponseContent(
+                        storageS3RequestP(
+                            this->storage, HTTP_VERB_POST_STR, this->interface.name,
+                            .query = httpQueryAdd(httpQueryNewP(), S3_QUERY_UPLOADS_STR, EMPTY_STR)))));
 
             // Store the upload id
             MEM_CONTEXT_BEGIN(this->memContext)
@@ -111,19 +139,17 @@ storageWriteS3Part(StorageWriteS3 *this)
             MEM_CONTEXT_END();
         }
 
-        // Upload the part and add etag to part list
-        HttpQuery *query = httpQueryNew();
+        // Upload the part async
+        HttpQuery *query = httpQueryNewP();
         httpQueryAdd(query, S3_QUERY_UPLOAD_ID_STR, this->uploadId);
         httpQueryAdd(query, S3_QUERY_PART_NUMBER_STR, strNewFmt("%u", strLstSize(this->uploadPartList) + 1));
 
-        strLstAdd(
-            this->uploadPartList,
-            httpHeaderGet(
-                storageS3Request(
-                    this->storage, HTTP_VERB_PUT_STR, this->interface.name, query, this->partBuffer, true, false).responseHeader,
-                HTTP_HEADER_ETAG_STR));
-
-        ASSERT(strLstGet(this->uploadPartList, strLstSize(this->uploadPartList) - 1) != NULL);
+        MEM_CONTEXT_BEGIN(this->memContext)
+        {
+            this->request = storageS3RequestAsyncP(
+                this->storage, HTTP_VERB_PUT_STR, this->interface.name, .query = query, .content = this->partBuffer);
+        }
+        MEM_CONTEXT_END();
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -145,6 +171,7 @@ storageWriteS3(THIS_VOID, const Buffer *buffer)
 
     ASSERT(this != NULL);
     ASSERT(this->partBuffer != NULL);
+    ASSERT(buffer != NULL);
 
     size_t bytesTotal = 0;
 
@@ -160,7 +187,7 @@ storageWriteS3(THIS_VOID, const Buffer *buffer)
         // If the part buffer is full then write it
         if (bufRemains(this->partBuffer) == 0)
         {
-            storageWriteS3Part(this);
+            storageWriteS3PartAsync(this);
             bufUsedZero(this->partBuffer);
         }
     }
@@ -192,8 +219,11 @@ storageWriteS3Close(THIS_VOID)
             if (this->uploadId != NULL)
             {
                 // If there is anything left in the part buffer then write it
-                if (bufUsed(this->partBuffer) > 0)
-                    storageWriteS3Part(this);
+                if (!bufEmpty(this->partBuffer))
+                    storageWriteS3PartAsync(this);
+
+                // Complete prior async request, if any
+                storageWriteS3Part(this);
 
                 // Generate the xml part list
                 XmlDocument *partList = xmlDocumentNew(S3_XML_TAG_COMPLETE_MULTIPART_UPLOAD_STR);
@@ -206,16 +236,14 @@ storageWriteS3Close(THIS_VOID)
                 }
 
                 // Finalize the multi-part upload
-                storageS3Request(
+                storageS3RequestP(
                     this->storage, HTTP_VERB_POST_STR, this->interface.name,
-                    httpQueryAdd(httpQueryNew(), S3_QUERY_UPLOAD_ID_STR, this->uploadId), xmlDocumentBuf(partList), true, false);
+                    .query = httpQueryAdd(httpQueryNewP(), S3_QUERY_UPLOAD_ID_STR, this->uploadId),
+                    .content = xmlDocumentBuf(partList));
             }
             // Else upload all the data in a single put
             else
-            {
-                storageS3Request(
-                    this->storage, HTTP_VERB_PUT_STR, this->interface.name, NULL, this->partBuffer, true, false);
-            }
+                storageS3RequestP(this->storage, HTTP_VERB_PUT_STR, this->interface.name, .content = this->partBuffer);
 
             bufFree(this->partBuffer);
             this->partBuffer = NULL;
@@ -226,9 +254,7 @@ storageWriteS3Close(THIS_VOID)
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-New object
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 StorageWrite *
 storageWriteS3New(StorageS3 *storage, const String *name, size_t partSize)
 {
