@@ -47,7 +47,6 @@ test.pl [options] doc|test
    --vm                 docker container to build/test
    --param              parameters to pass to test.pl
    --sudo               test requires sudo
-   --no-tempfs          do not create tempfs mount
 
  General Options:
    --help               display usage and exit
@@ -56,19 +55,15 @@ test.pl [options] doc|test
 ####################################################################################################################################
 # Command line parameters
 ####################################################################################################################################
-my $strVm = "none";
-my $strVmArch;
+my $strVm;
 my @stryParam;
-my $bNoTempFs;
 my $bSudo;
 my $bHelp;
 
 GetOptions ('help' => \$bHelp,
             'param=s@' => \@stryParam,
-            'no-tempfs' => \$bNoTempFs,
             'sudo' => \$bSudo,
-            'vm=s' => \$strVm,
-            'vm-arch=s' => \$strVmArch)
+            'vm=s' => \$strVm)
     or pod2usage(2);
 
 ####################################################################################################################################
@@ -121,6 +116,12 @@ eval
         pod2usage();
     }
 
+    # VM must be defined
+    if (!defined($strVm))
+    {
+        confess &log(ERROR, '--vm is required');
+    }
+
     ################################################################################################################################
     # Paths
     ################################################################################################################################
@@ -132,18 +133,14 @@ eval
 
     processBegin('install common packages');
     processExec('sudo apt-get -qq update', {bSuppressStdErr => true, bSuppressError => true});
-    processExec(
-        'sudo DEBIAN_FRONTEND=noninteractive apt-get install -y meson libxml-checker-perl libyaml-perl', {bSuppressStdErr => true});
+    processExec('sudo apt-get install -y libxml-checker-perl libyaml-perl', {bSuppressStdErr => true});
     processEnd();
 
-    if (!$bNoTempFs)
-    {
-        processBegin('mount tmpfs');
-        processExec('mkdir -p -m 770 test');
-        processExec('sudo mount -t tmpfs -o size=2048m tmpfs test');
-        processExec('df -h test', {bShowOutputAsync => true});
-        processEnd();
-    }
+    processBegin('mount tmpfs');
+    processExec('mkdir -p -m 770 test');
+    processExec('sudo mount -t tmpfs -o size=2048m tmpfs test');
+    processExec('df -h test', {bShowOutputAsync => true});
+    processEnd();
 
     ################################################################################################################################
     # Build documentation
@@ -152,6 +149,15 @@ eval
 
     if ($ARGV[0] eq 'doc')
     {
+        if ($strVm eq VM_CO7)
+        {
+            processBegin('LaTeX install');
+            processExec(
+                'sudo apt-get install -y --no-install-recommends texlive-latex-base texlive-latex-extra texlive-fonts-recommended',
+                {bSuppressStdErr => true});
+            processExec('sudo apt-get install -y texlive-font-utils texlive-latex-recommended', {bSuppressStdErr => true});
+        }
+
         processBegin('remove sudo');
         processExec("sudo rm /etc/sudoers.d/${strUser}");
         processEnd();
@@ -171,14 +177,18 @@ eval
     elsif ($ARGV[0] eq 'test')
     {
         # Build list of packages that need to be installed
-        my $strPackage =
-            "gcc ccache python3-distutils git rsync zlib1g-dev libssl-dev libxml2-dev libpq-dev libyaml-dev pkg-config uncrustify" .
-            " libssh2-1-dev valgrind";
+        my $strPackage = "rsync zlib1g-dev libssl-dev libxml2-dev libpq-dev pkg-config";
+
+        # Add lcov when testing coverage
+        if (vmCoverageC($strVm))
+        {
+            $strPackage .= " lcov";
+        }
 
         # Extra packages required when testing without containers
         if ($strVm eq VM_NONE)
         {
-            $strPackage .= " liblz4-dev liblz4-tool zstd libzstd-dev bzip2 libbz2-dev";
+            $strPackage .= " valgrind liblz4-dev liblz4-tool zstd libzstd-dev bzip2 libbz2-dev";
         }
         # Else packages needed for integration tests on containers
         else
@@ -191,9 +201,7 @@ eval
         processEnd();
 
         processBegin('install test packages');
-        processExec(
-            "sudo DEBIAN_FRONTEND=noninteractive apt-get install --no-install-recommends -y ${strPackage}",
-            {bSuppressStdErr => true});
+        processExec("sudo apt-get install -y ${strPackage}", {bSuppressStdErr => true});
         processEnd();
 
         if (!$bSudo)
@@ -204,20 +212,17 @@ eval
         }
 
         # Build the container
-        my $strVmArchParam = defined($strVmArch) ? " --vm-arch=${strVmArch}" : '';
-
         if ($strVm ne VM_NONE)
         {
             processBegin("${strVm} build");
-            processExec(
-                "${strTestExe} --vm-build --vm=${strVm}${strVmArchParam}", {bShowOutputAsync => true, bOutLogOnError => false});
+            processExec("${strTestExe} --vm-build --vm=${strVm}", {bShowOutputAsync => true, bOutLogOnError => false});
             processEnd();
         }
 
         processBegin(($strVm eq VM_NONE ? "no container" : $strVm) . ' test');
         processExec(
-            "${strTestExe} --gen-check --log-level-test-file=off --no-coverage-report --vm-max=2 --vm=${strVm}${strVmArchParam}" .
-                (@stryParam != 0 ? " --" . join(" --", @stryParam) : ''),
+            "${strTestExe} --no-gen --log-level-test-file=off --no-coverage-report --vm-host=none --vm-max=2 --vm=${strVm}" .
+            (@stryParam != 0 ? " --" . join(" --", @stryParam) : ''),
             {bShowOutputAsync => true, bOutLogOnError => false});
         processEnd();
     }
