@@ -14,7 +14,7 @@ use English '-no_match_vars';
 use Exporter qw(import);
     our @EXPORT = qw();
 
-use pgBackRest::Common::Log;
+use pgBackRestDoc::Common::Log;
 
 use pgBackRestTest::Common::ExecuteTest;
 
@@ -27,8 +27,8 @@ sub codeCountScan
     my $strBasePath = shift;
 
     # Load YAML
-    require YAML::XS;
-    YAML::XS->import(qw(Load Dump));
+    require YAML::Any;
+    YAML::Any->import(qw(Load Dump));
 
     my $hCodeCount = {};
 
@@ -41,7 +41,7 @@ sub codeCountScan
         # Only interested in files
         next if $hManifest->{$strFile}{type} ne 'f';
 
-        # Only exclude these directories/files entirely
+        # Exclude these directories/files entirely
         next if ($strFile =~ '^\.' ||
                  $strFile =~ '\.DS_Store$' ||
                  $strFile =~ '\.gitignore$' ||
@@ -55,21 +55,17 @@ sub codeCountScan
                  $strFile =~ '\.eps$' ||
                  $strFile =~ '\.cache$' ||
                  $strFile =~ '^doc/site/' ||
-                 $strFile eq 'libc/typemap' ||
+                 $strFile =~ '^src/build/autom4te.cache/' ||
                  $strFile eq 'test/Vagrantfile' ||
                  $strFile =~ '^test/\.vagrant/' ||
                  $strFile =~ '^test/certificate/' ||
                  $strFile =~ '^test/code-count/' ||
-                 $strFile =~ '^test/coverage/' ||
                  $strFile =~ '^test/data/' ||
-                 $strFile =~ '^test/docker/' ||
                  $strFile =~ '^test/expect/' ||
-                 $strFile =~ '^test/lint/' ||
-                 $strFile =~ '^test/package/' ||
                  $strFile =~ '^test/patch/' ||
-                 $strFile =~ '^test/profile/' ||
+                 $strFile =~ '^test/result/' ||
                  $strFile eq 'test/scratch.txt' ||
-                 $strFile eq 'test/src/valgrind.suppress' ||
+                 $strFile =~ '^test/src/valgrind\.suppress\.' ||
                  $strFile eq 'test/src/lcov.conf');
 
         # Classify the source file
@@ -83,13 +79,12 @@ sub codeCountScan
         {
             $strClass = 'doc/core';
         }
-        elsif ($strFile =~ '^build/' || $strFile =~ '^libc/build/' || $strFile eq 'libc/Makefile.PL' ||
-               $strFile eq 'src/Makefile.in' || $strFile eq 'src/configure' || $strFile eq 'src/configure.ac')
+        elsif ($strFile =~ '^build/' ||  $strFile eq 'src/Makefile.in' || $strFile eq 'src/configure' ||
+               $strFile =~ '^src/build/')
         {
             $strClass = 'build';
         }
-        elsif ($strFile =~ '^test/lib/pgBackRestTest/Module/' || $strFile =~ '^test/src/module/' ||
-               $strFile =~ '^libc/t/')
+        elsif ($strFile =~ '^test/lib/pgBackRestTest/Module/' || $strFile =~ '^test/src/module/')
         {
             $strClass = 'test/module';
         }
@@ -104,6 +99,12 @@ sub codeCountScan
             $strClass .= '/auto';
         }
 
+        # Append vendor if a vendorized file
+        if ($strFile =~ '\.vendor\..$')
+        {
+            $strClass .= '/vendor';
+        }
+
         # Force unrecognized file types
         my $strForceLang = undef;
         my $strType = undef;
@@ -133,7 +134,7 @@ sub codeCountScan
             $strType = 'yaml';
             $strForceLang = 'YAML';
         }
-        elsif ($strFile =~ 'Makefile\.in$' || $strFile =~ '^src\/configure')
+        elsif ($strFile =~ 'Makefile\.in$' || $strFile =~ '^src\/configure' || $strFile =~ '^src\/build\/')
         {
             $strType = 'make';
             $strForceLang = 'make';
@@ -166,7 +167,6 @@ sub codeCountScan
         # Load the file counts
         my $strYaml = executeTest(
             "cloc --yaml ${strBasePath}/${strFile}" .
-            " --read-lang-def=${strBasePath}/test/code-count/code.def" .
             " --force-lang='${strForceLang}'", {bSuppressStdErr => true});
 
         # Error if the file was ignored
