@@ -244,12 +244,6 @@ sub certSetup
 
     if ($rhVm->{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_RHEL)
     {
-        if ($strOS eq VM_CO6)
-        {
-            $strScript .=
-                "    update-ca-trust enable && \\\n";
-        }
-
         $strScript .=
             "    cp /etc/fake-cert/pgbackrest-test-ca.crt /etc/pki/ca-trust/source/anchors && \\\n" .
             "    update-ca-trust extract";
@@ -307,8 +301,8 @@ sub containerBuild
     $oStorageDocker->pathCreate($strTempPath, {strMode => '0770', bIgnoreExists => true, bCreateParent => true});
 
     # Load container definitions from yaml
-    require YAML::XS;
-    YAML::XS->import(qw(Load));
+    require YAML::Any;
+    YAML::Any->import(qw(Load));
 
     $hContainerCache = Load(${$oStorageDocker->get($oStorageDocker->pathGet('test/container.yaml'))});
 
@@ -350,10 +344,10 @@ sub containerBuild
 
         if ($$oVm{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_RHEL)
         {
-            if ($strOS eq VM_CO6 || $strOS eq VM_CO7)
+            if ($strOS eq VM_CO7)
             {
                 $strScript .=
-                    "    yum -y install epel-release && \\\n";
+                    "    yum -y install centos-release-scl-rh epel-release && \\\n";
             }
 
             $strScript .=
@@ -361,16 +355,7 @@ sub containerBuild
                 "    yum -y install openssh-server openssh-clients wget sudo valgrind git \\\n" .
                 "        perl perl-Digest-SHA perl-DBD-Pg perl-YAML-LibYAML openssl \\\n" .
                 "        gcc make perl-ExtUtils-MakeMaker perl-Test-Simple openssl-devel perl-ExtUtils-Embed rpm-build \\\n" .
-                "        zlib-devel libxml2-devel lz4-devel lz4 bzip2-devel bzip2";
-
-            if ($strOS eq VM_CO6)
-            {
-                $strScript .= ' perl-Time-HiRes perl-parent perl-JSON';
-            }
-            else
-            {
-                $strScript .= ' perl-JSON-PP';
-            }
+                "        zlib-devel libxml2-devel lz4-devel lz4 bzip2-devel bzip2 perl-JSON-PP";
         }
         else
         {
@@ -475,21 +460,11 @@ sub containerBuild
                 $strScript .=
                     "    rpm --import http://yum.postgresql.org/RPM-GPG-KEY-PGDG && \\\n";
 
-                if ($strOS eq VM_CO6)
+                if ($strOS eq VM_CO7)
                 {
                     $strScript .=
                         "    rpm -ivh \\\n" .
-                        "        http://yum.postgresql.org/9.1/redhat/rhel-6-x86_64/pgdg-centos91-9.1-6.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.2/redhat/rhel-6-x86_64/pgdg-centos92-9.2-8.noarch.rpm \\\n" .
-                        "        https://download.postgresql.org/pub/repos/yum/11/redhat/rhel-6-x86_64/" .
-                            "pgdg-redhat-repo-latest.noarch.rpm && \\\n";
-                }
-                elsif ($strOS eq VM_CO7)
-                {
-                    $strScript .=
-                        "    rpm -ivh \\\n" .
-                        "        http://yum.postgresql.org/9.2/redhat/rhel-7-x86_64/pgdg-centos92-9.2-3.noarch.rpm \\\n" .
-                        "        https://download.postgresql.org/pub/repos/yum/11/redhat/rhel-7-x86_64/" .
+                        "        https://download.postgresql.org/pub/repos/yum/reporpms/EL-7-x86_64/" .
                             "pgdg-redhat-repo-latest.noarch.rpm && \\\n";
                 }
                 elsif ($strOS eq VM_F32)
@@ -506,7 +481,7 @@ sub containerBuild
             {
                 $strScript .=
                     "    echo 'deb http://apt.postgresql.org/pub/repos/apt/ " .
-                    $$oVm{$strOS}{&VM_OS_REPO} . '-pgdg main' . ($strOS eq VM_U18 ? ' 13' : '') .
+                    $$oVm{$strOS}{&VM_OS_REPO} . '-pgdg main' .
                         "' >> /etc/apt/sources.list.d/pgdg.list && \\\n" .
                     "    wget --quiet -O - https://www.postgresql.org/media/keys/ACCC4CF8.asc | apt-key add - && \\\n" .
                     "    apt-get update && \\\n" .
