@@ -16,22 +16,21 @@ use Exporter qw(import);
 use File::Basename qw(dirname);
 use Storable qw(dclone);
 
-use pgBackRest::Common::Exception;
-use pgBackRest::Common::Ini;
-use pgBackRest::Common::Log;
-use pgBackRest::Common::String;
-use pgBackRest::Common::Wait;
-use pgBackRest::Config::Config;
-use pgBackRest::DbVersion;
-use pgBackRest::Manifest;
-use pgBackRest::Protocol::Storage::Helper;
-use pgBackRest::Version;
+use pgBackRestDoc::Common::Exception;
+use pgBackRestDoc::Common::Ini;
+use pgBackRestDoc::Common::Log;
+use pgBackRestDoc::Common::String;
+use pgBackRestDoc::ProjectInfo;
 
-use pgBackRestTest::Env::Host::HostBackupTest;
-use pgBackRestTest::Env::Host::HostBaseTest;
+use pgBackRestTest::Common::DbVersion;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::HostGroupTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::StorageRepo;
+use pgBackRestTest::Common::Wait;
+use pgBackRestTest::Env::Host::HostBackupTest;
+use pgBackRestTest::Env::Host::HostBaseTest;
+use pgBackRestTest::Env::Manifest;
 
 ####################################################################################################################################
 # Test WAL size
@@ -72,7 +71,7 @@ sub new
 
     my $self = $class->SUPER::new(
         {
-            strName => $bStandby ? HOST_DB_STANDBY : HOST_DB_MASTER,
+            strName => $bStandby ? HOST_DB_STANDBY : HOST_DB_PRIMARY,
             strImage => $$oParam{strImage},
             strBackupDestination => $$oParam{strBackupDestination},
             oLogTest => $$oParam{oLogTest},
