@@ -139,7 +139,8 @@ sub setup
         strCompressType => $$oConfigParam{strCompressType},
         bHardlink => $bHostBackup ? undef : $$oConfigParam{bHardLink},
         bArchiveAsync => $$oConfigParam{bArchiveAsync},
-        strStorage => $oConfigParam->{strStorage}});
+        strStorage => $oConfigParam->{strStorage},
+        iRepoTotal => $oConfigParam->{iRepoTotal}});
 
     # Create backup config if backup host exists
     if (defined($oHostBackup))
@@ -147,7 +148,8 @@ sub setup
         $oHostBackup->configCreate({
             strCompressType => $$oConfigParam{strCompressType},
             bHardlink => $$oConfigParam{bHardLink},
-            strStorage => $oConfigParam->{strStorage}});
+            strStorage => $oConfigParam->{strStorage},
+            iRepoTotal => $oConfigParam->{iRepoTotal}});
     }
     # If backup host is not defined set it to db-primary
     else
@@ -171,7 +173,8 @@ sub setup
             strCompressType => $$oConfigParam{strCompressType},
             bHardlink => $bHostBackup ? undef : $$oConfigParam{bHardLink},
             bArchiveAsync => $$oConfigParam{bArchiveAsync},
-            strStorage => $oConfigParam->{strStorage}});
+            strStorage => $oConfigParam->{strStorage},
+            iRepoTotal => $oConfigParam->{iRepoTotal}});
     }
 
     # Create object storage
@@ -238,7 +241,7 @@ sub dbCatalogVersion
         &PG_VERSION_10 => 201707211,
         &PG_VERSION_11 => 201806231,
         &PG_VERSION_12 => 201909212,
-        &PG_VERSION_13 => 202005171,
+        &PG_VERSION_13 => 202007201,
     };
 
     if (!defined($hCatalogVersion->{$strPgVersion}))
