@@ -28,9 +28,11 @@ typedef struct HarnessInfoChecksumData
 } HarnessInfoChecksumData;
 
 static void
-harnessInfoChecksumCallback(void *callbackData, const String *section, const String *key, const String *value)
+harnessInfoChecksumCallback(
+    void *callbackData, const String *section, const String *key, const String *value, const Variant *valueVar)
 {
     HarnessInfoChecksumData *data = (HarnessInfoChecksumData *)callbackData;
+    (void)valueVar;
 
     // Calculate checksum
     if (data->sectionLast == NULL || !strEq(section, data->sectionLast))
@@ -63,6 +65,8 @@ harnessInfoChecksum(const String *info)
         FUNCTION_HARNESS_PARAM(STRING, info);
     FUNCTION_HARNESS_END();
 
+    ASSERT(info != NULL);
+
     Buffer *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -108,6 +112,8 @@ harnessInfoChecksumZ(const char *info)
         FUNCTION_HARNESS_PARAM(STRINGZ, info);
     FUNCTION_HARNESS_END();
 
+    ASSERT(info != NULL);
+
     FUNCTION_HARNESS_RESULT(BUFFER, harnessInfoChecksum(STR(info)));
 }
 
@@ -115,8 +121,8 @@ harnessInfoChecksumZ(const char *info)
 Test callback that logs the results to a string
 ***********************************************************************************************************************************/
 void
-harnessInfoLoadNewCallback(void *callbackData, const String *section, const String *key, const String *value)
+harnessInfoLoadNewCallback(void *callbackData, const String *section, const String *key, const Variant *value)
 {
     if (callbackData != NULL)
-        strCatFmt((String *)callbackData, "[%s] %s=%s\n", strPtr(section), strPtr(key), strPtr(value));
+        strCatFmt((String *)callbackData, "[%s] %s=%s\n", strZ(section), strZ(key), strZ(jsonFromVar(value)));
 }
