@@ -15,13 +15,11 @@ Log Test Harness
 #include "common/harnessDebug.h"
 #include "common/harnessTest.h"
 
-#ifndef NO_LOG
-
 /***********************************************************************************************************************************
 Expose log internal data for unit testing/debugging
 ***********************************************************************************************************************************/
 extern LogLevel logLevelFile;
-extern int logHandleFile;
+extern int logFdFile;
 extern bool logFileBanner;
 extern unsigned int logProcessId;
 extern void logAnySet(void);
@@ -77,7 +75,7 @@ harnessLogInit(void)
     logFileBanner = true;
 
     snprintf(logFile, sizeof(logFile), "%s/expect.log", testDataPath());
-    logHandleFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
+    logFdFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
     logAnySet();
 
     FUNCTION_HARNESS_RESULT_VOID();
@@ -149,14 +147,14 @@ harnessLogLoad(const char *logFile)
 
     harnessLogBuffer[0] = 0;
 
-    int handle = harnessLogOpen(logFile, O_RDONLY, 0);
+    int fd = harnessLogOpen(logFile, O_RDONLY, 0);
 
     size_t totalBytes = 0;
     ssize_t actualBytes = 0;
 
     do
     {
-        actualBytes = read(handle, harnessLogBuffer, sizeof(harnessLogBuffer) - totalBytes);
+        actualBytes = read(fd, harnessLogBuffer, sizeof(harnessLogBuffer) - totalBytes);
 
         if (actualBytes == -1)
             THROW_SYS_ERROR_FMT(FileOpenError, "unable to read log file '%s'", logFile);
@@ -165,7 +163,7 @@ harnessLogLoad(const char *logFile)
     }
     while (actualBytes != 0);
 
-    if (close(handle) == -1)
+    if (close(fd) == -1)
         THROW_SYS_ERROR_FMT(FileOpenError, "unable to close log file '%s'", logFile);
 
     // Remove final linefeed
@@ -223,7 +221,7 @@ hrnLogReplaceAdd(const char *expression, const char *expressionSub, const char *
     {
         MEM_CONTEXT_BEGIN(harnessLog.memContext)
         {
-            harnessLog.replaceList = lstNew(sizeof(HarnessLogReplace));
+            harnessLog.replaceList = lstNewP(sizeof(HarnessLogReplace));
         }
         MEM_CONTEXT_END();
     }
@@ -295,18 +293,18 @@ hrnLogReplace(void)
                         {
                             THROW_FMT(
                                 AssertError, "unable to find sub expression '%s' in '%s' extracted with expresion '%s'",
-                                strPtr(logReplace->expressionSub), strPtr(match), strPtr(logReplace->expression));
+                                strZ(logReplace->expressionSub), strZ(match), strZ(logReplace->expression));
                         }
 
                         // Find beginning of match
-                        begin += regExpMatchPtr(logReplace->regExpSub) - strPtr(match);
+                        begin += regExpMatchPtr(logReplace->regExpSub) - strZ(match);
 
                         // Get the match
                         match = regExpMatchStr(logReplace->regExpSub);
                     }
 
                     // Build replacement string.  If versioned then append the version number.
-                    String *replace = strNewFmt("[%s", strPtr(logReplace->replacement));
+                    String *replace = strNewFmt("[%s", strZ(logReplace->replacement));
 
                     if (logReplace->version)
                     {
@@ -332,7 +330,7 @@ hrnLogReplace(void)
 
                     // Move data from end of string enough to make room for the replacement and copy replacement
                     memmove(end + diff, end, strlen(end) + 1);
-                    memcpy(begin, strPtr(replace), strSize(replace));
+                    memcpy(begin, strZ(replace), strSize(replace));
                 }
             }
         }
@@ -364,12 +362,12 @@ harnessLogResult(const char *expected)
     if (strcmp(harnessLogBuffer, expected) != 0)
     {
         THROW_FMT(
-            AssertError, "\n\nactual log:\n\n%s\n\nbut diff is (- remove from expected, + add to expected):\n\n%s",
+            AssertError, "\nACTUAL LOG:\n\n%s\n\nBUT DIFF FROM EXPECTED IS (- remove from expected, + add to expected):\n\n%s",
             harnessLogBuffer, hrnDiff(expected, harnessLogBuffer));
     }
 
-    close(logHandleFile);
-    logHandleFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
+    close(logFdFile);
+    logFdFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
 
     FUNCTION_HARNESS_RESULT_VOID();
 }
@@ -408,8 +406,8 @@ harnessLogResultRegExp(const char *expression)
         if (regexec(&regExp, harnessLogBuffer, 0, NULL, 0) != 0)
             THROW_FMT(AssertError, "\n\nexpected log regexp:\n\n%s\n\nbut actual log was:\n\n%s\n\n", expression, harnessLogBuffer);
 
-        close(logHandleFile);
-        logHandleFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
+        close(logFdFile);
+        logFdFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
     }
     FINALLY()
     {
@@ -436,5 +434,3 @@ harnessLogFinal(void)
 
     FUNCTION_HARNESS_RESULT_VOID();
 }
-
-#endif
