@@ -24,6 +24,7 @@ typedef struct TestData
 static TestData testList[TEST_LIST_SIZE];
 
 static int testRun = 0;
+static int testRunSub = 0;
 static int testTotal = 0;
 static bool testFirst = true;
 
@@ -33,6 +34,7 @@ static const char *testExeData = NULL;
 static const char *testProjectExeData = NULL;
 static bool testContainerData = false;
 static unsigned int testIdxData = 0;
+static bool testTiming = true;
 static uint64_t testScaleData = 1;
 static const char *testPathData = NULL;
 static const char *testDataPathData = NULL;
@@ -43,10 +45,23 @@ static char testUserData[64];
 static char testGroupIdData[32];
 static char testGroupData[64];
 
+static struct HarnessTestLocal
+{
+    uint64_t logLastBeginTime;                                      // Store the begin time of the last log for deltas
+
+    struct HarnessTestResult
+    {
+        bool running;                                               // Is the test currently running?
+        const char *statement;                                      // statement that is being tested
+        int lineNo;                                                 // Line number the test is on
+        bool result;                                                // Is there a result or is it void?
+    } result;
+} harnessTestLocal;
+
 /***********************************************************************************************************************************
 Extern functions
 ***********************************************************************************************************************************/
-#ifndef NO_LOG
+#ifdef HRN_FEATURE_LOG
     void harnessLogInit(void);
     void harnessLogFinal(void);
 #endif
@@ -56,7 +71,7 @@ Initialize harness
 ***********************************************************************************************************************************/
 void
 hrnInit(
-    const char *testExe, const char *testProjectExe, bool testContainer, unsigned int testIdx, uint64_t testScale,
+    const char *testExe, const char *testProjectExe, bool testContainer, unsigned int testIdx, bool timing, uint64_t testScale,
     const char *testPath, const char *testDataPath, const char *testRepoPath)
 {
     FUNCTION_HARNESS_VOID();
@@ -66,6 +81,7 @@ hrnInit(
     testProjectExeData = testProjectExe;
     testContainerData = testContainer;
     testIdxData = testIdx;
+    testTiming = timing;
     testScaleData = testScale;
     testPathData = testPath;
     testDataPathData = testDataPath;
@@ -145,7 +161,7 @@ testBegin(const char *name)
 
     if (testList[testRun - 1].selected)
     {
-#ifndef NO_LOG
+#ifdef HRN_FEATURE_LOG
         if (!testFirst)
         {
             // Make sure there is nothing untested left in the log
@@ -153,7 +169,9 @@ testBegin(const char *name)
 
             // Clear out the test directory so the next test starts clean
             char buffer[2048];
-            snprintf(buffer, sizeof(buffer), "%srm -rf %s/" "*", testContainer() ? "sudo " : "", testPath());
+            snprintf(
+                buffer, sizeof(buffer), "%schmod -R 700 %s/" "*;%srm -rf %s/" "*", testContainer() ? "sudo " : "", testPath(),
+                testContainer() ? "sudo " : "", testPath());
 
             if (system(buffer) != 0)
             {
@@ -163,7 +181,9 @@ testBegin(const char *name)
             }
 
             // Clear out the data directory so the next test starts clean
-            snprintf(buffer, sizeof(buffer), "%srm -rf %s/" "*", testContainer() ? "sudo " : "", testDataPath());
+            snprintf(
+                buffer, sizeof(buffer), "%schmod -R 700 %s/" "*;%srm -rf %s/" "*", testContainer() ? "sudo " : "",
+                testDataPath(), testContainer() ? "sudo " : "", testDataPath());
 
             if (system(buffer) != 0)
             {
@@ -182,12 +202,13 @@ testBegin(const char *name)
         if (testRun != 1)
             printf("\n");
 
-        printf("run %03d - %s\n", testRun, name);
+        printf("run %d - %s\n", testRun, name);
         fflush(stdout);
 
+        testRunSub = 1;
         timeMSecBegin = testTimeMSec();
 
-#ifndef NO_LOG
+#ifdef HRN_FEATURE_LOG
         // Initialize logging
         harnessLogInit();
 #endif
@@ -195,6 +216,8 @@ testBegin(const char *name)
         result = true;
     }
 
+    harnessTestLocal.logLastBeginTime = 0;
+
     FUNCTION_HARNESS_RESULT(BOOL, result);
 }
 
@@ -206,7 +229,7 @@ hrnComplete(void)
 {
     FUNCTION_HARNESS_VOID();
 
-#ifndef NO_LOG
+#ifdef HRN_FEATURE_LOG
     // Make sure there is nothing untested left in the log
     harnessLogFinal();
 #endif
@@ -383,8 +406,339 @@ hrnDiff(const char *expected, const char *actual)
     FUNCTION_HARNESS_RESULT(STRINGZ, harnessDiffBuffer);
 }
 
+/**********************************************************************************************************************************/
+void
+hrnTestLogTitle(int lineNo)
+{
+    // Output run/test
+    char buffer[32];
+    int bufferSize = snprintf(buffer, sizeof(buffer), "%d/%d", testRun, testRunSub);
+
+    printf("\nrun %s ", buffer);
+
+    // Output dashes
+    for (int dashIdx = 0; dashIdx < 16 - bufferSize; dashIdx++)
+        putchar('-');
+
+    // Output line number
+    printf(" l%04d", lineNo);
+
+    // Increment testSub and reset log time
+    testRunSub++;
+}
+
+/**********************************************************************************************************************************/
+void
+hrnTestLogPrefix(int lineNo, bool padding)
+{
+    FUNCTION_HARNESS_BEGIN();
+        FUNCTION_HARNESS_PARAM(INT, lineNo);
+        FUNCTION_HARNESS_PARAM(BOOL, padding);
+    FUNCTION_HARNESS_END();
+
+    // Always indent at the beginning
+    printf("    ");
+
+    // Add timing if requested
+    if (testTiming)
+    {
+        uint64_t currentTime = testTimeMSec();
+
+        // Print elapsed time size the beginning of the test run
+        printf(
+            "%03" PRIu64 ".%03" PRIu64"s", ((currentTime - testTimeMSecBegin()) / 1000),
+            ((currentTime - testTimeMSecBegin()) % 1000));
+
+        // Print delta time since the last log message
+        if (harnessTestLocal.logLastBeginTime != 0)
+        {
+            printf(
+                " %03" PRIu64 ".%03" PRIu64"s ", ((currentTime - harnessTestLocal.logLastBeginTime) / 1000),
+                ((currentTime - harnessTestLocal.logLastBeginTime) % 1000));
+        }
+        else
+            printf("          ");
+
+        harnessTestLocal.logLastBeginTime = currentTime;
+    }
+
+    // Add line number and padding
+    printf("l%04d %s", lineNo, padding ? "    " : "");
+
+    FUNCTION_HARNESS_RESULT_VOID();
+}
+
+/**********************************************************************************************************************************/
+void
+hrnTestResultBegin(const char *statement, int lineNo, bool result)
+{
+    ASSERT(!harnessTestLocal.result.running);
+
+    // Set the line number for the current function in the stack trace
+    FUNCTION_HARNESS_STACK_TRACE_LINE_SET(lineNo);
+
+    // Set info to report if an error is thrown
+    harnessTestLocal.result =
+        (struct HarnessTestResult){.running = true, .statement = statement, .lineNo = lineNo, .result = result};
+}
+
+bool
+hrnTestResultException(void)
+{
+    FUNCTION_HARNESS_VOID();
+
+    if (harnessTestLocal.result.running)
+    {
+        THROW_FMT(
+#ifndef NDEBUG
+            TestError,
+#else
+            AssertError,
+#endif
+            "EXPECTED %sRESULT FROM STATEMENT: %s\n\nBUT GOT %s: %s\n\nTHROWN AT:\n%s",
+            harnessTestLocal.result.result ? "" : "VOID ",
+            harnessTestLocal.result.statement, errorName(), errorMessage(), errorStackTrace());
+    }
+
+    FUNCTION_HARNESS_RESULT(BOOL, false);
+}
+
+void
+hrnTestResultEnd(void)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    // Set the line number for the current function back to unknown
+    FUNCTION_HARNESS_STACK_TRACE_LINE_SET(0);
+
+    harnessTestLocal.result.running = false;
+}
+
+/**********************************************************************************************************************************/
+static void hrnTestResultDiff(const char *actual, const char *expected)
+{
+    if (actual != NULL && expected != NULL && (strstr(actual, "\n") != NULL || strstr(expected, "\n") != NULL))
+    {
+        THROW_FMT(
+#ifndef NDEBUG
+            TestError,
+#else
+            AssertError,
+#endif
+            "STATEMENT: %s\n\nRESULT IS:\n%s\n\nBUT DIFF FROM EXPECTED IS (- remove from expected, + add to expected):\n%s\n\n",
+            harnessTestLocal.result.statement, actual, hrnDiff(expected, actual));
+    }
+    else
+    {
+        THROW_FMT(
+#ifndef NDEBUG
+            TestError,
+#else
+            AssertError,
+#endif
+            "STATEMENT: %s\n\nRESULT IS:\n%s\n\nBUT EXPECTED:\n%s",
+            harnessTestLocal.result.statement, actual == NULL ? "NULL" : actual, expected == NULL ? "NULL" : expected);                                                 \
+    }
+}
+
+void hrnTestResultBool(int actual, int expected)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    if (actual < 0 || actual > 1 || expected < 0 || expected > 1 || actual != expected)
+    {
+        char actualZ[256];
+        char expectedZ[256];
+
+        if (actual < 0 || actual > 1)
+            snprintf(actualZ, sizeof(actualZ), "INVALID(%d)", actual);
+        else
+            actual ? strcpy(actualZ, "true") : strcpy(actualZ, "false");
+
+        if (expected < 0 || expected > 1)
+            snprintf(expectedZ, sizeof(expectedZ), "INVALID(%d)", expected);
+        else
+            expected ? strcpy(expectedZ, "true") : strcpy(expectedZ, "false");
+
+        hrnTestResultDiff(actualZ, expectedZ);
+    }
+
+    hrnTestResultEnd();
+}
+
+void hrnTestResultDouble(double actual, double expected)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    if (actual != expected)
+    {
+        char actualZ[256];
+        char expectedZ[256];
+
+        snprintf(actualZ, sizeof(actualZ), "%f", actual);
+        snprintf(expectedZ, sizeof(expectedZ), "%f", expected);
+
+        hrnTestResultDiff(actualZ, expectedZ);
+    }
+
+    hrnTestResultEnd();
+}
+
+void hrnTestResultInt64(int64_t actual, int64_t expected, HarnessTestResultOperation operation)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    bool result = false;
+
+    switch (operation)
+    {
+        case harnessTestResultOperationEq:
+            result = actual == expected;
+            break;
+
+        case harnessTestResultOperationNe:
+            result = actual != expected;
+            break;
+    }
+
+    if (!result)
+    {
+        char actualZ[256];
+        char expectedZ[256];
+
+        snprintf(actualZ, sizeof(actualZ), "%" PRId64, actual);
+        snprintf(expectedZ, sizeof(expectedZ), "%" PRId64, expected);
+
+        hrnTestResultDiff(actualZ, expectedZ);
+    }
+
+    hrnTestResultEnd();
+}
+
+void hrnTestResultPtr(const void *actual, const void *expected, HarnessTestResultOperation operation)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    bool result = false;
+
+    switch (operation)
+    {
+        case harnessTestResultOperationEq:
+            result = actual == expected;
+            break;
+
+        case harnessTestResultOperationNe:
+            result = actual != expected;
+            break;
+    }
+
+    if (!result)
+    {
+        char actualZ[256];
+        char expectedZ[256];
+
+        snprintf(actualZ, sizeof(actualZ), "%p", actual);
+        snprintf(expectedZ, sizeof(expectedZ), "%p", expected);
+
+        hrnTestResultDiff(actualZ, expectedZ);
+    }
+
+    hrnTestResultEnd();
+}
+
+#ifdef HRN_FEATURE_STRING
+
+void
+hrnTestResultStringList(const StringList *actual, const void *expected, HarnessTestResultOperation operation)
+{
+    // Return NULL if list is empty
+    if (strLstEmpty(actual))
+    {
+        hrnTestResultZ(NULL, expected, operation);
+        return;
+    }
+
+    hrnTestResultZ(strZ(strCatZ(strLstJoin(actual, "\n"), "\n")), expected, operation);
+}
+
+#endif
+
+void hrnTestResultUInt64(uint64_t actual, uint64_t expected, HarnessTestResultOperation operation)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    bool result = false;
+
+    switch (operation)
+    {
+        case harnessTestResultOperationEq:
+            result = actual == expected;
+            break;
+
+        case harnessTestResultOperationNe:
+            result = actual != expected;
+            break;
+    }
+
+    if (!result)
+    {
+        char actualZ[256];
+        char expectedZ[256];
+
+        snprintf(actualZ, sizeof(actualZ), "%" PRIu64, actual);
+        snprintf(expectedZ, sizeof(expectedZ), "%" PRIu64, expected);
+
+        hrnTestResultDiff(actualZ, expectedZ);
+    }
+
+    hrnTestResultEnd();
+}
+
+void hrnTestResultUInt64Int64(uint64_t actual, int64_t expected, HarnessTestResultOperation operation)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    if (actual <= INT64_MAX && expected >= 0)
+        hrnTestResultUInt64(actual, (uint64_t)expected, operation);
+    else
+    {
+        char actualZ[256];
+        char expectedZ[256];
+
+        snprintf(actualZ, sizeof(actualZ), "%" PRIu64, actual);
+        snprintf(expectedZ, sizeof(expectedZ), "%" PRId64, expected);
+
+        hrnTestResultDiff(actualZ, expectedZ);
+    }
+}
+
+void hrnTestResultZ(const char *actual, const char *expected, HarnessTestResultOperation operation)
+{
+    ASSERT(harnessTestLocal.result.running);
+
+    bool result = false;
+
+    switch (operation)
+    {
+        case harnessTestResultOperationEq:
+            result = (actual == NULL && expected == NULL) || (actual != NULL && expected != NULL && strcmp(actual, expected) == 0);
+            break;
+
+        case harnessTestResultOperationNe:
+            result =
+                (actual == NULL && expected != NULL) || (actual != NULL && expected == NULL) ||
+                (actual != NULL && expected != NULL && strcmp(actual, expected) == 0);
+            break;
+    }
+
+    if (!result)
+        hrnTestResultDiff(actual, expected);
+
+    hrnTestResultEnd();
+}
+
 /***********************************************************************************************************************************
-Getters
+Getters/Setters
 ***********************************************************************************************************************************/
 const char *
 testExe(void)
