@@ -3,8 +3,8 @@ Test Archive Push Command
 ***********************************************************************************************************************************/
 #include "common/io/bufferRead.h"
 #include "common/io/bufferWrite.h"
-#include "common/io/handleRead.h"
-#include "common/io/handleWrite.h"
+#include "common/io/fdRead.h"
+#include "common/io/fdWrite.h"
 #include "common/time.h"
 #include "postgres/version.h"
 #include "storage/posix/storage.h"
@@ -67,13 +67,13 @@ testRun(void)
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_wal/archive_status/000000010000000100000005.ready")), NULL);
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_wal/archive_status/000000010000000100000006.ready")), NULL);
 
-        TEST_RESULT_STR_Z(
-            strLstJoin(archivePushProcessList(strNewFmt("%s/db/pg_wal", testPath())), "|"),
-            "000000010000000100000002|000000010000000100000005|000000010000000100000006", "ready list");
+        TEST_RESULT_STRLST_Z(
+            archivePushProcessList(strNewFmt("%s/db/pg_wal", testPath())),
+            "000000010000000100000002\n000000010000000100000005\n000000010000000100000006\n", "ready list");
 
-        TEST_RESULT_STR_Z(
-            strLstJoin(strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc), "|"),
-            "000000010000000100000003.ok", "remaining status list");
+        TEST_RESULT_STRLST_Z(
+            strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc),
+            "000000010000000100000003.ok\n", "remaining status list");
 
         // Test drop
         // -------------------------------------------------------------------------------------------------------------------------
@@ -133,8 +133,9 @@ testRun(void)
                 "1={\"db-id\":5555555555555555555,\"db-version\":\"9.4\"}\n"));
 
         TEST_ERROR(
-            archivePushCheck(true, cipherTypeNone, NULL), ArchiveMismatchError,
-            "PostgreSQL version 9.6, system-id 18072658121562454734 do not match stanza version 9.4, system-id 5555555555555555555"
+            archivePushCheck(true), ArchiveMismatchError,
+            "PostgreSQL version 9.6, system-id 18072658121562454734 do not match repo1 stanza version 9.4, system-id"
+                " 5555555555555555555"
                 "\nHINT: are you archiving to the correct stanza?");
 
         // Fix the version
@@ -148,8 +149,9 @@ testRun(void)
                 "1={\"db-id\":5555555555555555555,\"db-version\":\"9.6\"}\n"));
 
         TEST_ERROR(
-            archivePushCheck(true, cipherTypeNone, NULL), ArchiveMismatchError,
-            "PostgreSQL version 9.6, system-id 18072658121562454734 do not match stanza version 9.6, system-id 5555555555555555555"
+            archivePushCheck(true), ArchiveMismatchError,
+            "PostgreSQL version 9.6, system-id 18072658121562454734 do not match repo1 stanza version 9.6, system-id"
+                " 5555555555555555555"
                 "\nHINT: are you archiving to the correct stanza?");
 
         // Fix archive info
@@ -163,12 +165,73 @@ testRun(void)
                 "1={\"db-id\":18072658121562454734,\"db-version\":\"9.6\"}\n"));
 
         ArchivePushCheckResult result = {0};
-        TEST_ASSIGN(result, archivePushCheck(true, cipherTypeNone, NULL), "get archive check result");
+        TEST_ASSIGN(result, archivePushCheck(true), "get archive check result");
 
         TEST_RESULT_UINT(result.pgVersion, PG_VERSION_96, "check pg version");
         TEST_RESULT_UINT(result.pgSystemId, 0xFACEFACEFACEFACE, "check pg system id");
-        TEST_RESULT_STR_Z(result.archiveId, "9.6-1", "check archive id");
-        TEST_RESULT_STR_Z(result.archiveCipherPass, NULL, "check archive cipher pass (not set in this test)");
+        TEST_RESULT_STR_Z(result.repoData[0].archiveId, "9.6-1", "check archive id");
+        TEST_RESULT_UINT(result.repoData[0].cipherType, cipherTypeNone, "check cipher type");
+        TEST_RESULT_STR_Z(result.repoData[0].cipherPass, NULL, "check cipher pass (not set in this test)");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("mismatched repos when pg-path not present");
+
+        argList = strLstNew();
+        strLstAddZ(argList, "--stanza=test");
+        strLstAdd(argList, strNewFmt("--repo2-path=%s/repo2", testPath()));
+        strLstAdd(argList, strNewFmt("--repo4-path=%s/repo4", testPath()));
+        harnessCfgLoad(cfgCmdArchivePush, argList);
+
+        // repo2 has correct info
+        storagePutP(
+            storageNewWriteP(storageTest, strNew("repo2/archive/test/archive.info")),
+            harnessInfoChecksumZ(
+                "[db]\n"
+                "db-id=1\n"
+                "\n"
+                "[db:history]\n"
+                "1={\"db-id\":18072658121562454734,\"db-version\":\"9.6\"}\n"));
+
+        // repo4 has incorrect info
+        storagePutP(
+            storageNewWriteP(storageTest, strNew("repo4/archive/test/archive.info")),
+            harnessInfoChecksumZ(
+                "[db]\n"
+                "db-id=1\n"
+                "\n"
+                "[db:history]\n"
+                "1={\"db-id\":5555555555555555555,\"db-version\":\"9.4\"}\n"));
+
+        TEST_ERROR(
+            archivePushCheck(false), ArchiveMismatchError,
+            "repo2 stanza version 9.6, system-id 18072658121562454734 do not match repo4 stanza version 9.4, system-id"
+                " 5555555555555555555"
+                "\nHINT: are you archiving to the correct stanza?");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("matched repos when pg-path not present");
+
+        // repo4 has correct info
+        storagePutP(
+            storageNewWriteP(storageTest, strNew("repo4/archive/test/archive.info")),
+            harnessInfoChecksumZ(
+                "[db]\n"
+                "db-id=2\n"
+                "\n"
+                "[db:history]\n"
+                "1={\"db-id\":5555555555555555555,\"db-version\":\"9.4\"}\n"
+                "2={\"db-id\":18072658121562454734,\"db-version\":\"9.6\"}\n"));
+
+        TEST_ASSIGN(result, archivePushCheck(false), "get archive check result");
+
+        TEST_RESULT_UINT(result.pgVersion, PG_VERSION_96, "check pg version");
+        TEST_RESULT_UINT(result.pgSystemId, 0xFACEFACEFACEFACE, "check pg system id");
+        TEST_RESULT_STR_Z(result.repoData[0].archiveId, "9.6-1", "check repo2 archive id");
+        TEST_RESULT_UINT(result.repoData[0].cipherType, cipherTypeNone, "check repo2 cipher pass");
+        TEST_RESULT_STR_Z(result.repoData[0].cipherPass, NULL, "check repo2 cipher pass (not set in this test)");
+        TEST_RESULT_STR_Z(result.repoData[1].archiveId, "9.6-2", "check repo4 archive id");
+        TEST_RESULT_UINT(result.repoData[1].cipherType, cipherTypeNone, "check repo4 cipher type");
+        TEST_RESULT_STR_Z(result.repoData[1].cipherPass, NULL, "check repo4 cipher pass (not set in this test)");
     }
 
     // *****************************************************************************************************************************
@@ -177,8 +240,8 @@ testRun(void)
         TEST_TITLE("command must be run on the pg host");
 
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "--" CFGOPT_PG1_HOST "=host");
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgHost, "host");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/pg");
         strLstAddZ(argList, "--" CFGOPT_STANZA "=test2");
         harnessCfgLoadRole(cfgCmdArchivePush, cfgCmdRoleDefault, argList);
 
@@ -233,11 +296,11 @@ testRun(void)
 
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_wal/000000010000000100000001")), walBuffer1);
 
-        THROW_ON_SYS_ERROR(chdir(strPtr(cfgOptionStr(cfgOptPgPath))) != 0, PathMissingError, "unable to chdir()");
+        THROW_ON_SYS_ERROR(chdir(strZ(cfgOptionStr(cfgOptPgPath))) != 0, PathMissingError, "unable to chdir()");
 
         TEST_ERROR(
             cmdArchivePush(), ArchiveMismatchError,
-            strPtr(
+            strZ(
                 strNewFmt(
                     "WAL file '%s/pg/pg_wal/000000010000000100000001' version 10, system-id 18072658121562454734 do not match"
                         " stanza version 11, system-id 18072658121562454734",
@@ -250,7 +313,7 @@ testRun(void)
 
         TEST_ERROR(
             cmdArchivePush(), ArchiveMismatchError,
-            strPtr(
+            strZ(
                 strNewFmt(
                     "WAL file '%s/pg/pg_wal/000000010000000100000001' version 11, system-id 17055110554209741999 do not match"
                         " stanza version 11, system-id 18072658121562454734",
@@ -261,6 +324,12 @@ testRun(void)
         memset(bufPtr(walBuffer1), 0, bufSize(walBuffer1));
         pgWalTestToBuffer((PgWal){.version = PG_VERSION_11, .systemId = 0xFACEFACEFACEFACE}, walBuffer1);
 
+        // Check sha1 checksum against fixed values once to make sure they are not getting munged. After this we'll calculate them
+        // directly from the buffers to reduce the cost of maintaining checksums.
+        const char *walBuffer1Sha1 = TEST_64BIT() ?
+            (TEST_BIG_ENDIAN() ? "1c5f963d720bb199d7935dbd315447ea2ec3feb2" : "aae7591a1dbc58f21d0d004886075094f622e6dd") :
+            "28a13fd8cf6fcd9f9a8108aed4c8bcc58040863a";
+
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_wal/000000010000000100000001")), walBuffer1);
 
         TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment");
@@ -268,15 +337,12 @@ testRun(void)
 
         TEST_RESULT_BOOL(
             storageExistsP(
-                storageTest,
-                strNewFmt(
-                    "repo/archive/test/11-1/0000000100000001/000000010000000100000001-%s.gz",
-                    TEST_64BIT() ? "aae7591a1dbc58f21d0d004886075094f622e6dd" : "28a13fd8cf6fcd9f9a8108aed4c8bcc58040863a")),
+                storageTest, strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000001-%s.gz", walBuffer1Sha1)),
             true, "check repo for WAL file");
 
         TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment again");
         harnessLogResult(
-            "P00   WARN: WAL file '000000010000000100000001' already exists in the archive with the same checksum\n"
+            "P00   WARN: WAL file '000000010000000100000001' already exists in the repo1 archive with the same checksum\n"
             "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
             "P00   INFO: pushed WAL file '000000010000000100000001' to the archive");
 
@@ -285,19 +351,20 @@ testRun(void)
         bufUsedSet(walBuffer2, bufSize(walBuffer2));
         memset(bufPtr(walBuffer2), 0xFF, bufSize(walBuffer2));
         pgWalTestToBuffer((PgWal){.version = PG_VERSION_11, .systemId = 0xFACEFACEFACEFACE}, walBuffer2);
+        const char *walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer2)));
 
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_wal/000000010000000100000001")), walBuffer2);
 
-        TEST_ERROR(cmdArchivePush(), ArchiveDuplicateError, "WAL file '000000010000000100000001' already exists in the archive");
+        TEST_ERROR(
+            cmdArchivePush(), ArchiveDuplicateError,
+            "WAL file '000000010000000100000001' already exists in the repo1 archive with a different checksum");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("WAL with absolute path and no pg1-path");
 
-        const char *sha1 = TEST_64BIT() ? "755defa48a0a0872767b6dea49bdd3b64902f147" : "9c2a6ec4491a2118bcdc9b653366581d8821c982";
-
         argListTemp = strLstNew();
         strLstAddZ(argListTemp, "--" CFGOPT_STANZA "=test");
-        strLstAdd(argListTemp, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
+        hrnCfgArgRawFmt(argListTemp, cfgOptRepoPath, "%s/repo", testPath());
         strLstAdd(argListTemp, strNewFmt("%s/pg/pg_wal/000000010000000100000002", testPath()));
         harnessCfgLoad(cfgCmdArchivePush, argListTemp);
 
@@ -309,7 +376,8 @@ testRun(void)
             storagePutP(
                 storageNewWriteP(
                     storageTest,
-                    strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz.pgbackrest.tmp", sha1)),
+                    strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz.pgbackrest.tmp",
+                    walBuffer2Sha1)),
                 BUFSTRDEF("PARTIAL")),
             "write WAL tmp file");
 
@@ -317,12 +385,13 @@ testRun(void)
         harnessLogResult("P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
 
         TEST_RESULT_BOOL(
-            storageExistsP(storageTest, strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz", sha1)),
+            storageExistsP(
+                storageTest, strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz", walBuffer2Sha1)),
             true, "check repo for WAL file");
         TEST_RESULT_BOOL(
             storageExistsP(
                 storageTest,
-                strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz.pgbackrest.tmp", sha1)),
+                strNewFmt("repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s.gz.pgbackrest.tmp", walBuffer2Sha1)),
             false, "check WAL tmp file is gone");
 
         // Push a history file
@@ -359,7 +428,7 @@ testRun(void)
 
         TEST_RESULT_VOID(cmdArchivePush(), "push WAL file again");
         harnessLogResult(
-            "P00   WARN: WAL file '000000010000000100000002' already exists in the archive with the same checksum\n"
+            "P00   WARN: WAL file '000000010000000100000002' already exists in the repo1 archive with the same checksum\n"
             "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
             "P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
 
@@ -367,20 +436,20 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         VariantList *paramList = varLstNew();
         varLstAdd(paramList, varNewStr(strNewFmt("%s/pg/pg_wal/000000010000000100000002", testPath())));
-        varLstAdd(paramList, varNewStrZ("11-1"));
         varLstAdd(paramList, varNewUInt64(PG_VERSION_11));
         varLstAdd(paramList, varNewUInt64(0xFACEFACEFACEFACE));
         varLstAdd(paramList, varNewStrZ("000000010000000100000002"));
-        varLstAdd(paramList, varNewUInt64(cipherTypeNone));
-        varLstAdd(paramList, NULL);
         varLstAdd(paramList, varNewBool(false));
         varLstAdd(paramList, varNewInt(6));
+        varLstAdd(paramList, varNewStrZ("11-1"));
+        varLstAdd(paramList, varNewUInt64(cipherTypeNone));
+        varLstAdd(paramList, NULL);
 
         TEST_RESULT_BOOL(
             archivePushProtocol(PROTOCOL_COMMAND_ARCHIVE_PUSH_STR, paramList, server), true, "protocol archive put");
         TEST_RESULT_STR_Z(
             strNewBuf(serverWrite),
-            "{\"out\":\"WAL file '000000010000000100000002' already exists in the archive with the same checksum"
+            "{\"out\":\"WAL file '000000010000000100000002' already exists in the repo1 archive with the same checksum"
                 "\\nHINT: this is valid in some recovery scenarios but may also indicate a problem.\"}\n",
             "check result");
 
@@ -390,11 +459,14 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_RESULT_BOOL(archivePushProtocol(strNew(BOGUS_STR), paramList, server), false, "invalid function");
 
-        // Create a new encrypted repo to test encryption
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("multiple repos, one encrypted");
+
+        // Remove old repo
         storagePathRemoveP(storageTest, strNew("repo"), .errorOnMissing = true, .recurse = true);
 
-        StorageWrite *infoWrite = storageNewWriteP(storageTest, strNew("repo/archive/test/archive.info"));
+        // repo2 is encrypted
+        StorageWrite *infoWrite = storageNewWriteP(storageTest, strNew("repo2/archive/test/archive.info"));
 
         ioFilterGroupAdd(
             ioWriteFilterGroup(storageWriteIo(infoWrite)), cipherBlockNew(cipherModeEncrypt, cipherTypeAes256Cbc,
@@ -412,25 +484,70 @@ testRun(void)
                 "[db:history]\n"
                 "1={\"db-id\":18072658121562454734,\"db-version\":\"11\"}"));
 
+        // repo3 is not encrypted
+        storagePutP(
+            storageNewWriteP(storageTest, strNew("repo3/archive/test/archive.info")),
+            harnessInfoChecksumZ(
+                "[db]\n"
+                "db-id=1\n"
+                "\n"
+                "[db:history]\n"
+                "1={\"db-id\":18072658121562454734,\"db-version\":\"11\"}"));
+
         // Push encrypted WAL segment
-        argListTemp = strLstDup(argList);
+        argListTemp = strLstNew();
+        hrnCfgArgRawZ(argListTemp, cfgOptStanza, "test");
+        hrnCfgArgKeyRawFmt(argListTemp, cfgOptPgPath, 1, "%s/pg", testPath());
+        hrnCfgArgKeyRawFmt(argListTemp, cfgOptRepoPath, 2, "%s/repo2", testPath());
+        hrnCfgArgKeyRawZ(argListTemp, cfgOptRepoCipherType, 2, CIPHER_TYPE_AES_256_CBC);
+        hrnCfgEnvKeyRawZ(cfgOptRepoCipherPass, 2, "badpassphrase");
+        hrnCfgArgKeyRawFmt(argListTemp, cfgOptRepoPath, 3, "%s/repo3", testPath());
+        hrnCfgArgRawNegate(argListTemp, cfgOptCompress);
         strLstAddZ(argListTemp, "pg_wal/000000010000000100000002");
-        strLstAddZ(argListTemp, "--repo1-cipher-type=aes-256-cbc");
-        strLstAddZ(argListTemp, "--no-compress");
-        setenv("PGBACKREST_REPO1_CIPHER_PASS", "badpassphrase", true);
         harnessCfgLoad(cfgCmdArchivePush, argListTemp);
-        unsetenv("PGBACKREST_REPO1_CIPHER_PASS");
+        hrnCfgEnvKeyRemoveRaw(cfgOptRepoCipherPass, 2);
 
         TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment");
         harnessLogResult("P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
 
         TEST_RESULT_BOOL(
             storageExistsP(
-                storageTest,
-                strNewFmt(
-                    "repo/archive/test/11-1/0000000100000001/000000010000000100000002-%s",
-                    TEST_64BIT() ? "755defa48a0a0872767b6dea49bdd3b64902f147" : "9c2a6ec4491a2118bcdc9b653366581d8821c982")),
-            true, "check repo for WAL file");
+                storageTest, strNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            true, "check repo2 for WAL file");
+
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo3/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            true, "check repo3 for WAL file");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("remove WAL from one repo and push again");
+
+        storageRemoveP(
+            storageTest, strNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1),
+            .errorOnMissing = true);
+
+        TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment");
+        harnessLogResult(
+            "P00   WARN: WAL file '000000010000000100000002' already exists in the repo3 archive with the same checksum\n"
+            "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
+            "P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
+
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo2/archive/test/11-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            true, "check repo2 for WAL file");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("WAL already exists in both repos");
+
+        TEST_RESULT_VOID(cmdArchivePush(), "push the WAL segment");
+        harnessLogResult(
+            "P00   WARN: WAL file '000000010000000100000002' already exists in the repo2 archive with the same checksum\n"
+            "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
+            "            WAL file '000000010000000100000002' already exists in the repo3 archive with the same checksum\n"
+            "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
+            "P00   INFO: pushed WAL file '000000010000000100000002' to the archive");
     }
 
     // *****************************************************************************************************************************
@@ -441,8 +558,8 @@ testRun(void)
         TEST_TITLE("command must be run on the pg host");
 
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "--" CFGOPT_PG1_HOST "=host");
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgHost, "host");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/pg");
         strLstAddZ(argList, "--" CFGOPT_SPOOL_PATH "=/spool");
         strLstAddZ(argList, "--" CFGOPT_STANZA "=test2");
         strLstAddZ(argList, "--" CFGOPT_ARCHIVE_ASYNC);
@@ -478,7 +595,7 @@ testRun(void)
         harnessCfgLoadRaw(strLstSize(argList), strLstPtr(argList));
 
         storagePathCreateP(storageTest, cfgOptionStr(cfgOptPgPath));
-        THROW_ON_SYS_ERROR(chdir(strPtr(cfgOptionStr(cfgOptPgPath))) != 0, PathMissingError, "unable to chdir()");
+        THROW_ON_SYS_ERROR(chdir(strZ(cfgOptionStr(cfgOptPgPath))) != 0, PathMissingError, "unable to chdir()");
 
         TEST_ERROR(
             cmdArchivePush(), ArchiveTimeoutError,
@@ -531,18 +648,19 @@ testRun(void)
         strLstAddZ(argListTemp, "--archive-timeout=1");
         harnessCfgLoad(cfgCmdArchivePush, argListTemp);
 
-        THROW_ON_SYS_ERROR(chdir(strPtr(cfgOptionStr(cfgOptPgPath))) != 0, PathMissingError, "unable to chdir()");
+        THROW_ON_SYS_ERROR(chdir(strZ(cfgOptionStr(cfgOptPgPath))) != 0, PathMissingError, "unable to chdir()");
 
         HARNESS_FORK_BEGIN()
         {
             HARNESS_FORK_CHILD_BEGIN(0, true)
             {
-                IoRead *read = ioHandleReadNew(strNew("child read"), HARNESS_FORK_CHILD_READ(), 2000);
+                IoRead *read = ioFdReadNew(strNew("child read"), HARNESS_FORK_CHILD_READ(), 2000);
                 ioReadOpen(read);
-                IoWrite *write = ioHandleWriteNew(strNew("child write"), HARNESS_FORK_CHILD_WRITE());
+                IoWrite *write = ioFdWriteNew(strNew("child write"), HARNESS_FORK_CHILD_WRITE(), 2000);
                 ioWriteOpen(write);
 
-                lockAcquire(cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), cfgLockType(), 30000, true);
+                lockAcquire(
+                    cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), STRDEF("555-fefefefe"), cfgLockType(), 30000, true);
 
                 // Let the parent know the lock has been acquired and wait for the parent to allow lock release
                 ioWriteStrLine(write, strNew(""));
@@ -555,9 +673,9 @@ testRun(void)
 
             HARNESS_FORK_PARENT_BEGIN()
             {
-                IoRead *read = ioHandleReadNew(strNew("parent read"), HARNESS_FORK_PARENT_READ_PROCESS(0), 2000);
+                IoRead *read = ioFdReadNew(strNew("parent read"), HARNESS_FORK_PARENT_READ_PROCESS(0), 2000);
                 ioReadOpen(read);
-                IoWrite *write = ioHandleWriteNew(strNew("parent write"), HARNESS_FORK_PARENT_WRITE_PROCESS(0));
+                IoWrite *write = ioFdWriteNew(strNew("parent write"), HARNESS_FORK_PARENT_WRITE_PROCESS(0), 2000);
                 ioWriteOpen(write);
 
                 // Wait for the child to acquire the lock
@@ -587,6 +705,7 @@ testRun(void)
         bufUsedSet(walBuffer1, bufSize(walBuffer1));
         memset(bufPtr(walBuffer1), 0xFF, bufSize(walBuffer1));
         pgWalTestToBuffer((PgWal){.version = PG_VERSION_94, .systemId = 0xAAAABBBBCCCCDDDD}, walBuffer1);
+        const char *walBuffer1Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer1)));
 
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_xlog/000000010000000100000001")), walBuffer1);
 
@@ -595,17 +714,14 @@ testRun(void)
 
         TEST_RESULT_BOOL(
             storageExistsP(
-                storageTest,
-                strNewFmt(
-                    "repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s",
-                    TEST_64BIT() ? "ce58ec898a080e59d71dce49051843b03a497d99" : "765ee94e4f65d081f2dc3b8e77556da95bc31cbf")),
+                storageTest, strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
             true, "check repo for WAL file");
 
         // Direct tests of the async function
         // -------------------------------------------------------------------------------------------------------------------------
         argList = strLstNew();
         strLstAddZ(argList, "--stanza=test");
-        strLstAddZ(argList, "--no-compress");
+        hrnCfgArgRawZ(argList, cfgOptCompressType, "none");
         strLstAdd(argList, strNewFmt("--spool-path=%s/spool", testPath()));
         strLstAddZ(argList, "--" CFGOPT_ARCHIVE_ASYNC);
         strLstAdd(argList, strNewFmt("--pg1-path=%s/pg", testPath()));
@@ -621,9 +737,6 @@ testRun(void)
         storagePathRemoveP(storageSpoolWrite(), STORAGE_SPOOL_ARCHIVE_OUT_STR, .recurse = true);
         storagePathCreateP(storageSpoolWrite(), STORAGE_SPOOL_ARCHIVE_OUT_STR);
 
-        storagePathRemoveP(storageRepoWrite(), strNew(STORAGE_REPO_ARCHIVE "/9.4-1"), .recurse = true);
-        storagePathCreateP(storageRepoWrite(), strNew(STORAGE_REPO_ARCHIVE "/9.4-1"));
-
         storagePathRemoveP(storagePgWrite(), strNew("pg_xlog/archive_status"), .recurse = true);
         storagePathCreateP(storagePgWrite(), strNew("pg_xlog/archive_status"));
 
@@ -636,45 +749,72 @@ testRun(void)
             strNewBuf(storageGetP(storageNewReadP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT "/global.error")))),
             "25\nno WAL files to process", "check global.error");
 
-        TEST_RESULT_STR_Z(
-            strLstJoin(strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc), "|"),
-            "global.error", "check status files");
+        TEST_RESULT_STRLST_Z(
+            strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc),
+            "global.error\n", "check status files");
 
-        // Push WAL
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("add repo, push already pushed WAL and new WAL");
+
+        // Add repo3
+        hrnCfgArgKeyRawFmt(argList, cfgOptRepoPath, 3, "%s/repo3", testPath());
+        harnessCfgLoadRole(cfgCmdArchivePush, cfgCmdRoleAsync, argList);
+
+        storagePutP(
+            storageNewWriteP(storageTest, strNew("repo3/archive/test/archive.info")),
+            harnessInfoChecksumZ(
+                "[db]\n"
+                "db-id=1\n"
+                "\n"
+                "[db:history]\n"
+                "1={\"db-id\":12297848147757817309,\"db-version\":\"9.4\"}\n"));
+
         // Recreate ready file for WAL 1
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_xlog/archive_status/000000010000000100000001.ready")), NULL);
 
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            true, "check repo1 for WAL 1 file");
+
         // Create a ready file for WAL 2 but don't create the segment yet -- this will test the file error
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_xlog/archive_status/000000010000000100000002.ready")), NULL);
 
         TEST_RESULT_VOID(cmdArchivePushAsync(), "push WAL segments");
         harnessLogResult(
-            strPtr(
+            strZ(
                 strNewFmt(
                     "P00   INFO: push 2 WAL file(s) to archive: 000000010000000100000001...000000010000000100000002\n"
+                    "P01   WARN: WAL file '000000010000000100000001' already exists in the repo1 archive with the same checksum\n"
+                    "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
                     "P01 DETAIL: pushed WAL file '000000010000000100000001' to the archive\n"
                     "P01   WARN: could not push WAL file '000000010000000100000002' to the archive (will be retried): "
                         "[55] raised from local-1 protocol: " STORAGE_ERROR_READ_MISSING,
-                    strPtr(strNewFmt("%s/pg/pg_xlog/000000010000000100000002", testPath())))));
+                    strZ(strNewFmt("%s/pg/pg_xlog/000000010000000100000002", testPath())))));
 
         TEST_RESULT_BOOL(
             storageExistsP(
-                storageTest,
-                strNewFmt(
-                    "repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s",
-                    TEST_64BIT() ? "ce58ec898a080e59d71dce49051843b03a497d99" : "765ee94e4f65d081f2dc3b8e77556da95bc31cbf")),
-            true, "check repo for WAL 1 file");
+                storageTest, strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            true, "check repo1 for WAL 1 file");
 
-        TEST_RESULT_STR_Z(
-            strLstJoin(strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc), "|"),
-            "000000010000000100000001.ok|000000010000000100000002.error", "check status files");
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000001-%s", walBuffer1Sha1)),
+            true, "check repo3 for WAL 1 file");
+
+        TEST_RESULT_STRLST_Z(
+            strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc),
+            "000000010000000100000001.ok\n000000010000000100000002.error\n", "check status files");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("create and push previously missing WAL");
 
         // Create WAL 2 segment
         Buffer *walBuffer2 = bufNew((size_t)16 * 1024 * 1024);
         bufUsedSet(walBuffer2, bufSize(walBuffer2));
         memset(bufPtr(walBuffer2), 0x0C, bufSize(walBuffer2));
         pgWalTestToBuffer((PgWal){.version = PG_VERSION_94, .systemId = 0xAAAABBBBCCCCDDDD}, walBuffer2);
+        const char *walBuffer2Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer2)));
 
         storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_xlog/000000010000000100000002")), walBuffer2);
 
@@ -689,15 +829,63 @@ testRun(void)
 
         TEST_RESULT_BOOL(
             storageExistsP(
-                storageTest,
-                strNewFmt(
-                    "repo/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s",
-                    TEST_64BIT() ? "251820f85b31554bbf33061fce7dabf18c16afe0" : "ae8cb1fe45d40bc364df6bc6e8803774f0e3eaeb")),
-            true, "check repo for WAL 2 file");
+                storageTest, strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            true, "check repo1 for WAL 2 file");
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000002-%s", walBuffer2Sha1)),
+            true, "check repo3 for WAL 2 file");
 
-        TEST_RESULT_STR_Z(
-            strLstJoin(strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc), "|"),
-            "000000010000000100000001.ok|000000010000000100000002.ok", "check status files");
+        TEST_RESULT_STRLST_Z(
+            strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc),
+            "000000010000000100000001.ok\n000000010000000100000002.ok\n", "check status files");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("push wal 2 again to get warnings from both repos");
+
+        // Remove the OK file so the WAL gets pushed again
+        storageRemoveP(storageSpoolWrite(), STRDEF(STORAGE_SPOOL_ARCHIVE_OUT "/000000010000000100000002.ok"));
+
+        TEST_RESULT_VOID(cmdArchivePushAsync(), "push WAL segments");
+        harnessLogResult(
+            "P00   INFO: push 1 WAL file(s) to archive: 000000010000000100000002\n"
+            "P01   WARN: WAL file '000000010000000100000002' already exists in the repo1 archive with the same checksum\n"
+            "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
+            "            WAL file '000000010000000100000002' already exists in the repo3 archive with the same checksum\n"
+            "            HINT: this is valid in some recovery scenarios but may also indicate a problem.\n"
+            "P01 DETAIL: pushed WAL file '000000010000000100000002' to the archive");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("create and push WAL 3 to both repos");
+
+        // Create WAL 3 segment
+        Buffer *walBuffer3 = bufNew((size_t)16 * 1024 * 1024);
+        bufUsedSet(walBuffer3, bufSize(walBuffer3));
+        memset(bufPtr(walBuffer3), 0x44, bufSize(walBuffer3));
+        pgWalTestToBuffer((PgWal){.version = PG_VERSION_94, .systemId = 0xAAAABBBBCCCCDDDD}, walBuffer3);
+        const char *walBuffer3Sha1 = strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA1_STR, walBuffer3)));
+
+        storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_xlog/000000010000000100000003")), walBuffer3);
+
+        // Create ready file
+        storagePutP(storageNewWriteP(storagePgWrite(), strNew("pg_xlog/archive_status/000000010000000100000003.ready")), NULL);
+
+        TEST_RESULT_VOID(cmdArchivePushAsync(), "push WAL segment");
+        harnessLogResult(
+            "P00   INFO: push 1 WAL file(s) to archive: 000000010000000100000003\n"
+            "P01 DETAIL: pushed WAL file '000000010000000100000003' to the archive");
+
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo/archive/test/9.4-1/0000000100000001/000000010000000100000003-%s", walBuffer3Sha1)),
+            true, "check repo1 for WAL 3 file");
+        TEST_RESULT_BOOL(
+            storageExistsP(
+                storageTest, strNewFmt("repo3/archive/test/9.4-1/0000000100000001/000000010000000100000003-%s", walBuffer3Sha1)),
+            true, "check repo3 for WAL 3 file");
+
+        // Remove the ready file to prevent WAL 3 from being considered for the next test
+        storageRemoveP(storagePgWrite(), strNew("pg_xlog/archive_status/000000010000000100000003.ready"), .errorOnMissing = true);
 
         // Check that drop functionality works
         // -------------------------------------------------------------------------------------------------------------------------
@@ -725,9 +913,9 @@ testRun(void)
                 storageGetP(storageNewReadP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT "/000000010000000100000002.ok")))),
             "0\ndropped WAL file '000000010000000100000002' because archive queue exceeded 16MB", "check WAL 2 warning");
 
-        TEST_RESULT_STR_Z(
-            strLstJoin(strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc), "|"),
-            "000000010000000100000001.ok|000000010000000100000002.ok", "check status files");
+        TEST_RESULT_STRLST_Z(
+            strLstSort(storageListP(storageSpool(), strNew(STORAGE_SPOOL_ARCHIVE_OUT)), sortOrderAsc),
+            "000000010000000100000001.ok\n000000010000000100000002.ok\n", "check status files");
         }
 
     FUNCTION_HARNESS_RESULT_VOID();
