@@ -105,8 +105,8 @@ testRun(void)
         storagePutP(storageNewWriteP(storageTest, strNew("repo/aaa"), .timeModified = 1578671569), BUFSTRDEF("TESTDATA"));
         storagePutP(storageNewWriteP(storageTest, strNew("repo/bbb/ccc")), BUFSTRDEF("TESTDATA2"));
 
-        ASSERT(system(strPtr(strNewFmt("ln -s ../bbb %s/repo/link", testPath()))) == 0);
-        ASSERT(system(strPtr(strNewFmt("mkfifo %s/repo/pipe", testPath()))) == 0);
+        ASSERT(system(strZ(strNewFmt("ln -s ../bbb %s/repo/link", testPath()))) == 0);
+        ASSERT(system(strZ(strNewFmt("mkfifo %s/repo/pipe", testPath()))) == 0);
 
         output = bufNew(0);
         cfgOptionSet(cfgOptOutput, cfgSourceParam, VARSTRDEF("text"));
@@ -138,7 +138,6 @@ testRun(void)
 
         // Recurse
         // -------------------------------------------------------------------------------------------------------------------------
-        cfgOptionValidSet(cfgOptRecurse, true);
         cfgOptionSet(cfgOptRecurse, cfgSourceParam, VARBOOL(true));
 
         output = bufNew(0);
@@ -148,7 +147,6 @@ testRun(void)
 
         // Filter
         // -------------------------------------------------------------------------------------------------------------------------
-        cfgOptionValidSet(cfgOptFilter, true);
         cfgOptionSet(cfgOptFilter, cfgSourceParam, VARSTRDEF("^aaa$"));
 
         output = bufNew(0);
@@ -172,7 +170,7 @@ testRun(void)
         int stdoutSave = dup(STDOUT_FILENO);
         String *stdoutFile = strNewFmt("%s/stdout.txt", testPath());
 
-        THROW_ON_SYS_ERROR(freopen(strPtr(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
+        THROW_ON_SYS_ERROR(freopen(strZ(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
 
         // Not in a test wrapper to avoid writing to stdout
         cmdStorageList();
@@ -296,7 +294,7 @@ testRun(void)
         TEST_TITLE("error when missing destination");
 
         StringList *argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
         harnessCfgLoad(cfgCmdRepoPut, argList);
 
         TEST_ERROR(storagePutProcess(ioBufferReadNew(fileBuffer)), ParamRequiredError, "destination file required");
@@ -313,8 +311,8 @@ testRun(void)
         TEST_TITLE("put an encrypted file with custom key");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom");
         strLstAdd(argList, fileEncCustomName);
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -325,8 +323,8 @@ testRun(void)
         TEST_TITLE("put a raw file");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--raw");
         strLstAdd(argList, fileRawName);
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -336,7 +334,7 @@ testRun(void)
         const String *stdinFile = storagePathP(storageRepo(), STRDEF("stdin.txt"));
         storagePutP(storageNewWriteP(storageRepoWrite(), stdinFile), fileRawBuffer);
 
-        THROW_ON_SYS_ERROR(freopen(strPtr(stdinFile), "r", stdin) == NULL, FileWriteError, "unable to reopen stdin");
+        THROW_ON_SYS_ERROR(freopen(strZ(stdinFile), "r", stdin) == NULL, FileWriteError, "unable to reopen stdin");
 
         TEST_RESULT_VOID(cmdStoragePut(), "put");
 
@@ -347,8 +345,8 @@ testRun(void)
         TEST_TITLE("put encrypted archive.info");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_ARCHIVE "/test/" INFO_ARCHIVE_FILE);
         harnessCfgLoad(cfgCmdRepoPut, argList);
 
@@ -358,8 +356,8 @@ testRun(void)
         TEST_TITLE("put encrypted archive.info.copy");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_ARCHIVE "/test/" INFO_ARCHIVE_FILE ".copy");
         harnessCfgLoad(cfgCmdRepoPut, argList);
 
@@ -369,8 +367,8 @@ testRun(void)
         TEST_TITLE("put encrypted backup.info");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/" INFO_BACKUP_FILE);
         harnessCfgLoad(cfgCmdRepoPut, argList);
 
@@ -380,8 +378,8 @@ testRun(void)
         TEST_TITLE("put encrypted backup.info.copy");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/" INFO_BACKUP_FILE ".copy");
         harnessCfgLoad(cfgCmdRepoPut, argList);
 
@@ -397,8 +395,8 @@ testRun(void)
         bufUsedSet(archiveFileBuffer, bufSize(archiveFileBuffer));
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom");
         strLstAdd(argList, strNew(STORAGE_PATH_ARCHIVE "/test/12-1/000000010000000100000001-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"));
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -412,8 +410,8 @@ testRun(void)
         TEST_TITLE("put encrypted backup.manifest");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom");
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/latest/" BACKUP_MANIFEST_FILE);
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -424,8 +422,8 @@ testRun(void)
         TEST_TITLE("put encrypted backup.manifest.copy");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom");
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/latest/" BACKUP_MANIFEST_FILE ".copy");
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -436,8 +434,8 @@ testRun(void)
         TEST_TITLE("put encrypted backup.history manifest");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom");
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/backup.history/2020/label.manifest.gz");
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -448,8 +446,8 @@ testRun(void)
         TEST_TITLE("put encrypted backup_label");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom2");
         strLstAdd(argList, strNew(STORAGE_PATH_BACKUP "/test/latest/pg_data/backup_label"));
         harnessCfgLoad(cfgCmdRepoPut, argList);
@@ -460,7 +458,7 @@ testRun(void)
         TEST_TITLE("error when missing source");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
         TEST_ERROR(storageGetProcess(ioBufferWriteNew(bufNew(0))), ParamRequiredError, "source file required");
@@ -479,8 +477,8 @@ testRun(void)
         TEST_TITLE("get a file with / repo");
 
         argList = strLstNew();
-        strLstAdd(argList, strNew("--" CFGOPT_REPO1_PATH "=/"));
-        strLstAdd(argList, strNewFmt("%s/repo/%s", testPath(), strPtr(fileName)));
+        hrnCfgArgRawZ(argList, cfgOptRepoPath, "/");
+        strLstAdd(argList, strNewFmt("%s/repo/%s", testPath(), strZ(fileName)));
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
         writeBuffer = bufNew(0);
@@ -491,8 +489,8 @@ testRun(void)
         TEST_TITLE("get an encrypted file with custom key");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--" CFGOPT_CIPHER_PASS "=custom");
         strLstAdd(argList, fileEncCustomName);
         harnessCfgLoad(cfgCmdRepoGet, argList);
@@ -505,8 +503,8 @@ testRun(void)
         TEST_TITLE("get a raw file");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "--raw");
         strLstAdd(argList, fileRawName);
         harnessCfgLoad(cfgCmdRepoGet, argList);
@@ -517,7 +515,7 @@ testRun(void)
         int stdoutSave = dup(STDOUT_FILENO);
         String *stdoutFile = strNewFmt("%s/repo/stdout.txt", testPath());
 
-        THROW_ON_SYS_ERROR(freopen(strPtr(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
+        THROW_ON_SYS_ERROR(freopen(strZ(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
 
         // Not in a test wrapper to avoid writing to stdout
         ASSERT(cmdStorageGet() == 0);
@@ -532,7 +530,7 @@ testRun(void)
         TEST_TITLE("ignore missing file");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
         strLstAddZ(argList, "--" CFGOPT_IGNORE_MISSING);
         strLstAddZ(argList, BOGUS_STR);
         harnessCfgLoad(cfgCmdRepoGet, argList);
@@ -544,37 +542,37 @@ testRun(void)
         TEST_TITLE("get file outside of the repo error");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, "/somewhere/" INFO_ARCHIVE_FILE);
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
         writeBuffer = bufNew(0);
         TEST_ERROR(
             storageGetProcess(ioBufferWriteNew(writeBuffer)), OptionInvalidValueError,
-            strPtr(strNewFmt("absolute path '/somewhere/%s' is not in base path '%s/repo'", INFO_ARCHIVE_FILE, testPath())));
+            strZ(strNewFmt("absolute path '/somewhere/%s' is not in base path '%s/repo'", INFO_ARCHIVE_FILE, testPath())));
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("get file in repo root directory error");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAdd(argList, fileEncCustomName);
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
         writeBuffer = bufNew(0);
         TEST_ERROR(
             storageGetProcess(ioBufferWriteNew(writeBuffer)), OptionInvalidValueError,
-            strPtr(strNewFmt("unable to determine cipher passphrase for '%s'", strPtr(fileEncCustomName))));
+            strZ(strNewFmt("unable to determine cipher passphrase for '%s'", strZ(fileEncCustomName))));
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("get encrypted archive.info - stanza mismatch");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
         strLstAddZ(argList, "--" CFGOPT_STANZA "=test2");
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_ARCHIVE "/test/" INFO_ARCHIVE_FILE);
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -586,8 +584,8 @@ testRun(void)
         TEST_TITLE("get encrypted archive.info");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_ARCHIVE "/test/" INFO_ARCHIVE_FILE);
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -599,9 +597,9 @@ testRun(void)
         TEST_TITLE("get encrypted archive.info.copy");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
         strLstAddZ(argList, "--" CFGOPT_STANZA "=test");
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_ARCHIVE "/test/" INFO_ARCHIVE_FILE ".copy");
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -613,8 +611,8 @@ testRun(void)
         TEST_TITLE("get encrypted backup.info");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/" INFO_BACKUP_FILE);
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -626,8 +624,8 @@ testRun(void)
         TEST_TITLE("get encrypted backup.info.copy");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/" INFO_BACKUP_FILE ".copy");
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -642,8 +640,8 @@ testRun(void)
         TEST_TITLE("get encrypted WAL archive file");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAdd(argList, strNewFmt(
             "%s/repo/" STORAGE_PATH_ARCHIVE "/test/12-1/000000010000000100000001-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",testPath()));
         harnessCfgLoad(cfgCmdRepoGet, argList);
@@ -656,8 +654,8 @@ testRun(void)
         TEST_TITLE("get encrypted backup.manifest");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/latest/" BACKUP_MANIFEST_FILE);
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -669,8 +667,8 @@ testRun(void)
         TEST_TITLE("get encrypted backup.manifest.copy");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/latest/" BACKUP_MANIFEST_FILE ".copy");
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -682,8 +680,8 @@ testRun(void)
         TEST_TITLE("get encrypted backup.history manifest");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAddZ(argList, STORAGE_PATH_BACKUP "/test/backup.history/2020/label.manifest.gz");
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
@@ -695,8 +693,8 @@ testRun(void)
         TEST_TITLE("get encrypted backup_label");
 
         argList = strLstNew();
-        strLstAdd(argList, strNewFmt("--" CFGOPT_REPO1_PATH "=%s/repo", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_REPO1_CIPHER_TYPE "=" CIPHER_TYPE_AES_256_CBC);
+        hrnCfgArgRawFmt(argList, cfgOptRepoPath, "%s/repo", testPath());
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, CIPHER_TYPE_AES_256_CBC);
         strLstAdd(argList, strNew(STORAGE_PATH_BACKUP "/test/latest/pg_data/backup_label"));
         harnessCfgLoad(cfgCmdRepoGet, argList);
 
