@@ -9,9 +9,10 @@ Test Ini
 Test callback to accumulate ini load results
 ***********************************************************************************************************************************/
 static void
-testIniLoadCallback(void *data, const String *section, const String *key, const String *value)
+testIniLoadCallback(void *data, const String *section, const String *key, const String *value, const Variant *valueVar)
 {
-    strCatFmt((String *)data, "%s:%s:%s\n", strPtr(section), strPtr(key), strPtr(value));
+    ASSERT(strEq(value, jsonFromVar(valueVar)));
+    strCatFmt((String *)data, "%s:%s:%s\n", strZ(section), strZ(key), strZ(value));
 }
 
 /***********************************************************************************************************************************
@@ -33,15 +34,6 @@ testRun(void)
         TEST_RESULT_VOID(iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), "load ini");
         TEST_RESULT_STR_Z(result, "", "    check ini");
 
-        // Invalid section
-        // -------------------------------------------------------------------------------------------------------------------------
-        iniBuf = BUFSTRZ(
-            "  [section  ");
-
-        TEST_ERROR(
-            iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), FormatError,
-            "ini section should end with ] at line 1: [section");
-
         // Key outside of section
         // -------------------------------------------------------------------------------------------------------------------------
         iniBuf = BUFSTRZ(
@@ -51,6 +43,14 @@ testRun(void)
             iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), FormatError,
             "key/value found outside of section at line 1: key=value");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("invalid JSON value");
+
+        iniBuf = BUFSTRZ("[section]\nkey=value\n");
+
+        TEST_ERROR(
+            iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), FormatError, "invalid JSON value at line 2: key=value");
+
         // Key outside of section
         // -------------------------------------------------------------------------------------------------------------------------
         iniBuf = BUFSTRZ(
@@ -65,49 +65,50 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         iniBuf = BUFSTRZ(
             "[section]\n"
-            "=value");
+            "=\"value\"");
 
         TEST_ERROR(
             iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), FormatError,
-            "key is zero-length at line 1: =value");
+            "key is zero-length at line 1: =\"value\"");
 
-        // One section
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("one section");
+
         iniBuf = BUFSTRZ(
-            "# comment\n"
             "[section1]\n"
-            "key1=value1\n"
-            "key2=value2");
+            " key1 =\"value1\"\n"
+            "key2=\"value2\"\n"
+            "key=3==\"value3\"\n"
+            "==\"=\"");
         result = strNew("");
 
         TEST_RESULT_VOID(iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), "load ini");
         TEST_RESULT_STR_Z(
             result,
-            "section1:key1:value1\n"
-                "section1:key2:value2\n",
+            "section1: key1 :\"value1\"\n"
+            "section1:key2:\"value2\"\n"
+            "section1:key=3=:\"value3\"\n"
+            "section1:=:\"=\"\n",
             "    check ini");
 
         // Two sections
         // -------------------------------------------------------------------------------------------------------------------------
         iniBuf = BUFSTRZ(
-            "# comment\n"
             "[section1]\n"
-            "key1=value1\n"
-            "key2=value2\n"
+            "[key1=\"value1\"\n"
+            "key2=\"value2\"\n"
             "\n"
             "[section2]\n"
-            "key1=\n"
             "\n"
-            "key2=value2");
+            "#key2=\"value2\"");
         result = strNew("");
 
         TEST_RESULT_VOID(iniLoad(ioBufferReadNew(iniBuf), testIniLoadCallback, result), "load ini");
         TEST_RESULT_STR_Z(
             result,
-            "section1:key1:value1\n"
-                "section1:key2:value2\n"
-                "section2:key1:\n"
-                "section2:key2:value2\n",
+            "section1:[key1:\"value1\"\n"
+            "section1:key2:\"value2\"\n"
+            "section2:#key2:\"value2\"\n",
             "    check ini");
     }
 
@@ -149,16 +150,16 @@ testRun(void)
         TEST_RESULT_STR_Z(iniGetDefault(ini, strNew("section3"), strNew("key3"), strNew("true")), "true", "get section, key, bool");
 
         TEST_RESULT_INT(strLstSize(iniSectionKeyList(ini, strNew("bogus"))), 0, "get keys for missing section");
-        TEST_RESULT_STR_Z(strLstJoin(iniSectionKeyList(ini, strNew("section1")), "|"), "key1|key2", "get keys for section");
+        TEST_RESULT_STRLST_Z(iniSectionKeyList(ini, strNew("section1")), "key1\nkey2\n", "get keys for section");
 
         TEST_RESULT_VOID(iniSet(ini, strNew("section2"), strNew("key2"), strNew("2")), "set section2, key");
         TEST_RESULT_INT(strLstSize(iniSectionList(ini)), 2, "number of sections");
-        TEST_RESULT_STR_Z(strLstJoin(iniSectionList(ini), "|"), "section1|section2", "get sections");
+        TEST_RESULT_STRLST_Z(iniSectionList(ini), "section1\nsection2\n", "get sections");
 
         TEST_RESULT_BOOL(iniSectionKeyIsList(ini, strNew("section1"), strNew("key1")), false, "single value is not list");
         TEST_RESULT_VOID(iniSet(ini, strNew("section2"), strNew("key2"), strNew("7")), "set section2, key");
         TEST_RESULT_BOOL(iniSectionKeyIsList(ini, strNew("section2"), strNew("key2")), true, "section2, key2 is a list");
-        TEST_RESULT_STR_Z(strLstJoin(iniGetList(ini, strNew("section2"), strNew("key2")), "|"), "2|7", "get list");
+        TEST_RESULT_STRLST_Z(iniGetList(ini, strNew("section2"), strNew("key2")), "2\n7\n", "get list");
         TEST_RESULT_PTR(iniGetList(ini, strNew("section2"), strNew("key-missing")), NULL, "get missing list");
 
         TEST_RESULT_VOID(iniFree(ini), "free ini");
