@@ -13,7 +13,7 @@ testFree(void *thisVoid)
 {
     MemContext *this = thisVoid;
 
-    TEST_RESULT_INT(this->state, memContextStateFreeing, "state should be freeing before memContextFree() in callback");
+    TEST_RESULT_BOOL(memContextFreeing(this), true, "state should be freeing before memContextFree() in callback");
     memContextFree(this);
     TEST_RESULT_INT(this->state, memContextStateFreeing, "state should still be freeing after memContextFree() in callback");
 
@@ -86,11 +86,15 @@ testRun(void)
         TEST_ERROR(memContextNew(""), AssertError, "assertion 'name[0] != '\\0'' failed");
 
         MemContext *memContext = memContextNew("test1");
+        memContextKeep();
         TEST_RESULT_Z(memContextName(memContext), "test1", "test1 context name");
         TEST_RESULT_PTR(memContext->contextParent, memContextTop(), "test1 context parent is top");
         TEST_RESULT_INT(memContextTop()->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE, "initial top context child list size");
 
-        TEST_RESULT_PTR(memContextSwitch(memContext), memContextTop(), "switch returns top as old context");
+        memContextSwitch(memContext);
+
+        TEST_ERROR(memContextKeep(), AssertError, "new context expected but current context 'test1' found");
+        TEST_ERROR(memContextDiscard(), AssertError, "new context expected but current context 'test1' found");
         TEST_RESULT_PTR(memContext, memContextCurrent(), "current context is now test1");
 
         // Create enough mem contexts to use up the initially allocated block
@@ -98,6 +102,7 @@ testRun(void)
         {
             memContextSwitch(memContextTop());
             memContextNew("test-filler");
+            memContextKeep();
             TEST_RESULT_BOOL(
                 memContextTop()->contextChildList[contextIdx]->state == memContextStateActive, true, "new context is active");
             TEST_RESULT_Z(memContextName(memContextTop()->contextChildList[contextIdx]), "test-filler", "new context name");
@@ -105,6 +110,7 @@ testRun(void)
 
         // This forces the child context array to grow
         memContextNew("test5");
+        memContextKeep();
         TEST_RESULT_INT(memContextTop()->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE * 2, "increased child context list size");
         TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, MEM_CONTEXT_INITIAL_SIZE + 1, "check context free idx");
 
@@ -117,6 +123,7 @@ testRun(void)
 
         // Create a new context and it should end up in the same spot
         memContextNew("test-reuse");
+        memContextKeep();
         TEST_RESULT_BOOL(
             memContextTop()->contextChildList[1]->state == memContextStateActive,
             true, "new context in same index as freed context is active");
@@ -125,17 +132,22 @@ testRun(void)
 
         // Next context will be at the end
         memContextNew("test-at-end");
+        memContextKeep();
         TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, MEM_CONTEXT_INITIAL_SIZE + 2, "check context free idx");
 
         // Create a child context to test recursive free
         memContextSwitch(memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]);
         memContextNew("test-reuse");
+        memContextKeep();
         TEST_RESULT_PTR_NE(
             memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]->contextChildList, NULL, "context child list is allocated");
         TEST_RESULT_INT(
             memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE,
             "context child list initial size");
 
+        // This test will change if the contexts above change
+        TEST_RESULT_UINT(memContextSize(memContextTop()), TEST_64BIT() ? 960 : 544, "check size");
+
         TEST_ERROR(
             memContextFree(memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]),
             AssertError, "cannot free current context 'test5'");
@@ -148,6 +160,7 @@ testRun(void)
             AssertError, "cannot free inactive context");
 
         MemContext *noAllocation = memContextNew("empty");
+        memContextKeep();
         noAllocation->allocListSize = 0;
         free(noAllocation->allocList);
         TEST_RESULT_VOID(memContextFree(noAllocation), "free context with no allocations");
@@ -156,10 +169,16 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("memContextAlloc(), memNew*(), memGrow(), and memFree()"))
     {
+        TEST_RESULT_UINT(sizeof(MemContextAlloc), 8, "check MemContextAlloc size (same for 32/64 bit)");
+        TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER((void *)1), (void *)(sizeof(MemContextAlloc) + 1), "check buffer macro");
+        TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_HEADER((void *)sizeof(MemContextAlloc)), (void *)0, "check header macro");
+
         memContextSwitch(memContextTop());
         memNewPtrArray(1);
 
         MemContext *memContext = memContextNew("test-alloc");
+        TEST_ERROR(memContextSwitchBack(), AssertError, "current context expected but new context 'test-alloc' found");
+        memContextKeep();
         memContextSwitch(memContext);
 
         for (int allocIdx = 0; allocIdx <= MEM_CONTEXT_ALLOC_INITIAL_SIZE; allocIdx++)
@@ -188,11 +207,10 @@ testRun(void)
 
         // Free memory
         TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 2, "check alloc free idx");
-        TEST_RESULT_VOID(memFree(memContextCurrent()->allocList[0].buffer), "free allocation");
-        TEST_ERROR(memFree(memContextCurrent()->allocList[0].buffer), AssertError, "unable to find allocation");
+        TEST_RESULT_VOID(memFree(MEM_CONTEXT_ALLOC_BUFFER(memContextCurrent()->allocList[0])), "free allocation");
         TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 0, "check alloc free idx");
 
-        TEST_RESULT_VOID(memFree(memContextCurrent()->allocList[1].buffer), "free allocation");
+        TEST_RESULT_VOID(memFree(MEM_CONTEXT_ALLOC_BUFFER(memContextCurrent()->allocList[1])), "free allocation");
         TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 0, "check alloc free idx");
 
         TEST_RESULT_VOID(memNew(3), "new allocation");
@@ -204,8 +222,17 @@ testRun(void)
         TEST_RESULT_VOID(memNew(3), "new allocation");
         TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 3, "check alloc free idx");
 
-        TEST_ERROR(memFree(NULL), AssertError, "assertion 'buffer != NULL' failed");
-        TEST_ERROR(memFree((void *)0x01), AssertError, "unable to find allocation");
+        // This test will change if the allocations above change
+        TEST_RESULT_UINT(memContextSize(memContextCurrent()), TEST_64BIT() ? 249 : 165, "check size");
+
+        TEST_ERROR(
+            memFree(NULL), AssertError,
+            "assertion '((MemContextAlloc *)buffer - 1) != NULL"
+                " && (uintptr_t)((MemContextAlloc *)buffer - 1) != (uintptr_t)-sizeof(MemContextAlloc)"
+                " && ((MemContextAlloc *)buffer - 1)->allocIdx <"
+                " memContextStack[memContextCurrentStackIdx].memContext->allocListSize"
+                " && memContextStack[memContextCurrentStackIdx].memContext->allocList[((MemContextAlloc *)buffer - 1)->allocIdx]'"
+                " failed");
         memFree(buffer);
 
         memContextSwitch(memContextTop());
@@ -219,6 +246,7 @@ testRun(void)
             memContextCallbackSet(memContextTop(), testFree, NULL), AssertError, "top context may not have a callback");
 
         MemContext *memContext = memContextNew("test-callback");
+        memContextKeep();
         memContextCallbackSet(memContext, testFree, memContext);
         TEST_ERROR(
             memContextCallbackSet(memContext, testFree, memContext), AssertError,
@@ -233,7 +261,8 @@ testRun(void)
 
         // Now test with an error
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_ASSIGN(memContext, memContextNew("test-callback-error"), "new mem context");
+        memContext = memContextNew("test-callback-error");
+        TEST_RESULT_VOID(memContextKeep(), "keep mem context");
         testFreeThrow = true;
         TEST_RESULT_VOID(memContextCallbackSet(memContext, testFree, memContext), "    set callback");
         TEST_ERROR(memContextFree(memContext), AssertError, "error in callback");
@@ -245,6 +274,7 @@ testRun(void)
     {
         memContextSwitch(memContextTop());
         MemContext *memContext = memContextNew("test-block");
+        memContextKeep();
 
         // Check normal block
         MEM_CONTEXT_BEGIN(memContext)
@@ -271,12 +301,12 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         MEM_CONTEXT_TEMP_RESET_BEGIN()
         {
-            TEST_RESULT_BOOL(MEM_CONTEXT_TEMP()->allocList[0].active, false, "nothing allocated");
+            TEST_RESULT_PTR(MEM_CONTEXT_TEMP()->allocList[0], NULL, "nothing allocated");
             memNew(99);
-            TEST_RESULT_BOOL(MEM_CONTEXT_TEMP()->allocList[0].active, true, "1 allocation");
+            TEST_RESULT_PTR_NE(MEM_CONTEXT_TEMP()->allocList[0], NULL, "1 allocation");
 
             MEM_CONTEXT_TEMP_RESET(1);
-            TEST_RESULT_BOOL(MEM_CONTEXT_TEMP()->allocList[0].active, false, "nothing allocated");
+            TEST_RESULT_PTR(MEM_CONTEXT_TEMP()->allocList[0], NULL, "nothing allocated");
         }
         MEM_CONTEXT_TEMP_END();
     }
@@ -289,6 +319,8 @@ testRun(void)
         const char *memContextTestName = "test-new-block";
         MemContext *memContext = NULL;
 
+        TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "context is now 'TOP'");
+
         MEM_CONTEXT_NEW_BEGIN(memContextTestName)
         {
             memContext = MEM_CONTEXT_NEW();
@@ -297,6 +329,7 @@ testRun(void)
         }
         MEM_CONTEXT_NEW_END();
 
+        TEST_RESULT_Z(memContextName(memContextCurrent()), "TOP", "context name is now 'TOP'");
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "context is now 'TOP'");
         TEST_RESULT_BOOL(memContext->state == memContextStateActive, true, "new mem context is still active");
         memContextFree(memContext);
@@ -343,6 +376,7 @@ testRun(void)
             MEM_CONTEXT_TEMP_BEGIN()
             {
                 memContextNew("not-to-be-moved");
+                memContextKeep();
 
                 MEM_CONTEXT_NEW_BEGIN("inner")
                 {
@@ -351,7 +385,7 @@ testRun(void)
                 }
                 MEM_CONTEXT_NEW_END();
 
-                TEST_RESULT_PTR(memContext->allocList[0].buffer, mem, "check memory allocation");
+                TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContext->allocList[0]), mem, "check memory allocation");
                 TEST_RESULT_PTR(memContextCurrent()->contextChildList[1], memContext, "check memory context");
 
                 // Null out the mem context in the parent so the move will fail
@@ -375,10 +409,10 @@ testRun(void)
             }
             MEM_CONTEXT_TEMP_END();
 
-            TEST_RESULT_PTR(memContext->allocList[0].buffer, mem, "check memory allocation");
+            TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContext->allocList[0]), mem, "check memory allocation");
             TEST_RESULT_PTR(memContextCurrent()->contextChildList[1], memContext, "check memory context");
 
-            TEST_RESULT_PTR(memContext2->allocList[0].buffer, mem2, "check memory allocation 2");
+            TEST_RESULT_PTR(MEM_CONTEXT_ALLOC_BUFFER(memContext2->allocList[0]), mem2, "check memory allocation 2");
             TEST_RESULT_PTR(memContextCurrent()->contextChildList[2], memContext2, "check memory context 2");
         }
         MEM_CONTEXT_NEW_END();
