@@ -72,6 +72,7 @@ testRun(void)
         TEST_ERROR(jsonToVar(strNew("")), JsonFormatError, "expected data");
         TEST_ERROR(jsonToVar(strNew(" \t\r\n ")), JsonFormatError, "expected data");
         TEST_ERROR(jsonToVar(strNew("z")), JsonFormatError, "invalid type at 'z'");
+        TEST_ERROR(jsonToVar(strNew("3 =")), JsonFormatError, "unexpected characters after JSON at '='");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_RESULT_STR_Z(varStr(jsonToVar(strNew(" \"test\""))), "test", "simple string");
@@ -113,7 +114,7 @@ testRun(void)
         TEST_ERROR(jsonToVarLst(strNew("[")), JsonFormatError, "expected data");
         TEST_ERROR(jsonToVarLst(strNew(" [] ZZZ")), JsonFormatError, "unexpected characters after array at 'ZZZ'");
 
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewVarLst(jsonToVarLst(strNew("[\"e1\", \"e2\"]"))), "|"), "e1|e2", "json list");
+        TEST_RESULT_STRLST_Z(strLstNewVarLst(jsonToVarLst(strNew("[\"e1\", \"e2\"]"))), "e1\ne2\n", "json list");
     }
 
     // *****************************************************************************************************************************
