@@ -7,6 +7,8 @@ Test Configuration Load
 #include "version.h"
 
 #include "common/harnessConfig.h"
+#include "storage/cifs/storage.h"
+#include "storage/posix/storage.h"
 
 /***********************************************************************************************************************************
 Test run
@@ -19,7 +21,7 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("cfgLoadLogSetting()"))
     {
-        cfgInit();
+        harnessCfgLoad(cfgCmdVersion, strLstNew());
 
         TEST_RESULT_VOID(cfgLoadLogSetting(), "load log settings all defaults");
 
@@ -32,102 +34,210 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("cfgLoadUpdateOption()"))
     {
-        String *exe = strNew("/path/to/pgbackrest");
-        String *exeOther = strNew("/other/path/to/pgbackrest");
+        TEST_TITLE("error if user passes pg/repo options when they are internal");
 
-        cfgInit();
-        cfgCommandSet(cfgCmdBackup, cfgCmdRoleDefault);
-        cfgExeSet(exe);
+        StringList *argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/pg1");
+        hrnCfgArgRawZ(argList, cfgOptRepo, "1");
+        TEST_ERROR(harnessCfgLoad(cfgCmdCheck, argList), OptionInvalidError, "option 'repo' not valid for command 'check'");
 
-        cfgOptionValidSet(cfgOptRepoHost, true);
-        cfgOptionValidSet(cfgOptRepoHostCmd, true);
-        cfgOptionValidSet(cfgOptPgHost, true);
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("error when repo option not set and repo total > 1 or first repo index != 1");
 
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "hosts are not set so don't update commands");
+        argList = strLstNew();
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 1, "/repo1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 4, "/repo4");
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/pg1");
+        TEST_ERROR(
+            harnessCfgLoad(cfgCmdStanzaDelete, argList), OptionRequiredError,
+            "stanza-delete command requires option: repo\n"
+            "HINT: this command requires a specific repository to operate on");
 
-        cfgOptionSet(cfgOptRepoHost, cfgSourceParam, varNewStrZ("repo-host"));
+        argList = strLstNew();
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 2, "/repo2");
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/pg1");
+        TEST_ERROR(
+            harnessCfgLoad(cfgCmdStanzaDelete, argList), OptionRequiredError,
+            "stanza-delete command requires option: repo\n"
+            "HINT: this command requires a specific repository to operate on");
+
+        argList = strLstNew();
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 1, "/repo1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 4, "/repo4");
+        TEST_RESULT_VOID(harnessCfgLoad(cfgCmdInfo, argList), "load info config -- option repo not required");
+        TEST_RESULT_BOOL(cfgCommand() == cfgCmdInfo, true, "    command is info");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("local default repo paths must be different");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgRawZ(argList, cfgOptRepo, "3");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionDiff, 4, "4");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionDiff, 3, "3");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 1, "/repo1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoHost, 2, "host2");
+        TEST_ERROR(
+            harnessCfgLoad(cfgCmdExpire, argList), OptionInvalidValueError,
+            "local repo3 and repo4 paths are both '/var/lib/pgbackrest' but must be different");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("local default repo paths for cifs repo type must be different");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptRepo, "2");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoType, 1, STORAGE_CIFS_TYPE);
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoType, 2, STORAGE_CIFS_TYPE);
+        TEST_ERROR(
+            harnessCfgLoad(cfgCmdInfo, argList), OptionInvalidValueError,
+            "local repo1 and repo2 paths are both '/var/lib/pgbackrest' but must be different");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("local repo paths same but types different");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptRepo, "1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoType, 1, STORAGE_POSIX_TYPE);
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoType, 2, STORAGE_CIFS_TYPE);
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoType, 3, "s3");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoS3Bucket, 3, "cool-bucket");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoS3Region, 3, "region");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoS3Endpoint, 3, "endpoint");
+        hrnCfgEnvKeyRawZ(cfgOptRepoS3Key, 3, "mykey");
+        hrnCfgEnvKeyRawZ(cfgOptRepoS3KeySecret, 3, "mysecretkey");
+        harnessCfgLoad(cfgCmdInfo, argList);
+
+        hrnCfgEnvKeyRemoveRaw(cfgOptRepoS3Key, 3);
+        hrnCfgEnvKeyRemoveRaw(cfgOptRepoS3KeySecret, 3);
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("repo-host-cmd is defaulted when null");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/pg1");
+        harnessCfgLoad(cfgCmdCheck, argList);
+
+        cfgOptionIdxSet(cfgOptRepoHost, 0, cfgSourceParam, varNewStrZ("repo-host"));
 
         TEST_RESULT_VOID(cfgLoadUpdateOption(), "repo remote command is updated");
-        TEST_RESULT_STR(cfgOptionStr(cfgOptRepoHostCmd), exe, "    check repo1-host-cmd");
+        TEST_RESULT_STR_Z(cfgOptionIdxStr(cfgOptRepoHostCmd, 0), testProjectExe(), "    check repo1-host-cmd");
 
-        cfgOptionSet(cfgOptRepoHostCmd, cfgSourceParam, varNewStr(exeOther));
+        cfgOptionIdxSet(cfgOptRepoHostCmd, 0, cfgSourceParam, VARSTRDEF("/other"));
 
         TEST_RESULT_VOID(cfgLoadUpdateOption(), "repo remote command was already set");
-        TEST_RESULT_STR(cfgOptionStr(cfgOptRepoHostCmd), exeOther, "    check repo1-host-cmd");
-
-        cfgOptionSet(cfgOptRepoHost, cfgSourceParam, NULL);
+        TEST_RESULT_STR_Z(cfgOptionIdxStr(cfgOptRepoHostCmd, 0), "/other", "    check repo1-host-cmd");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        cfgOptionValidSet(cfgOptPgHostCmd, true);
-        cfgOptionSet(cfgOptPgHost, cfgSourceParam, varNewStrZ("pg1-host"));
+        TEST_TITLE("pg-host-cmd is defaulted when null");
 
-        cfgOptionValidSet(cfgOptPgHost + 1, true);
-        cfgOptionSet(cfgOptPgHost + 1, cfgSourceParam, varNewStrZ("pg2-host"));
-        cfgOptionValidSet(cfgOptPgHostCmd + 1, true);
-        cfgOptionSet(cfgOptPgHostCmd + 1, cfgSourceParam, varNewStr(exeOther));
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgHost, 1, "pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 2, "/pg2");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgHost, 2, "pg2");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgHostCmd, 2, "pg2-exe");
+        harnessCfgLoad(cfgCmdCheck, argList);
 
-        cfgOptionValidSet(cfgOptPgHostCmd + 2, true);
-
-        cfgOptionValidSet(cfgOptPgHost + cfgOptionIndexTotal(cfgOptPgHost) - 1, true);
-        cfgOptionSet(cfgOptPgHost + cfgOptionIndexTotal(cfgOptPgHost) - 1, cfgSourceParam, varNewStrZ("pgX-host"));
-        cfgOptionValidSet(cfgOptPgHostCmd + cfgOptionIndexTotal(cfgOptPgHost) - 1, true);
-
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "pg remote command is updated");
-        TEST_RESULT_STR(cfgOptionStr(cfgOptPgHostCmd), exe, "    check pg1-host-cmd");
-        TEST_RESULT_STR(cfgOptionStr(cfgOptPgHostCmd + 1), exeOther, "    check pg2-host-cmd is already set");
-        TEST_RESULT_STR(cfgOptionStrNull(cfgOptPgHostCmd + 2), NULL, "    check pg3-host-cmd is not set");
-        TEST_RESULT_STR(cfgOptionStr(cfgOptPgHostCmd + cfgOptionIndexTotal(cfgOptPgHost) - 1), exe, "    check pgX-host-cmd");
+        TEST_RESULT_STR_Z(cfgOptionIdxStr(cfgOptPgHostCmd, 0), testProjectExe(), "    check pg1-host-cmd");
+        TEST_RESULT_STR_Z(cfgOptionIdxStr(cfgOptPgHostCmd, 1), "pg2-exe", "    check pg2-host-cmd");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        cfgInit();
+        TEST_TITLE("db-timeout set but not protocol timeout");
 
-        cfgOptionValidSet(cfgOptDbTimeout, true);
-        cfgOptionSet(cfgOptDbTimeout, cfgSourceParam, varNewDbl(100));
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "pg timeout set but not protocol timeout");
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgRawZ(argList, cfgOptDbTimeout, "100");
+        harnessCfgLoad(cfgCmdCheck, argList);
 
-        cfgOptionValidSet(cfgOptProtocolTimeout, true);
-        cfgOptionSet(cfgOptProtocolTimeout, cfgSourceDefault, varNewDbl(101));
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "protocol timeout > pg timeout");
+        cfgOptionInvalidate(cfgOptProtocolTimeout);
+        cfgLoadUpdateOption();
 
-        cfgOptionSet(cfgOptDbTimeout, cfgSourceParam, varNewDbl(100000));
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "protocol timeout set automatically");
-        TEST_RESULT_DOUBLE(cfgOptionDbl(cfgOptProtocolTimeout), 100030, "    check protocol timeout");
+        TEST_RESULT_UINT(cfgOptionUInt64(cfgOptDbTimeout), 100000, "check db-timeout");
 
-        cfgOptionValidSet(cfgOptProtocolTimeout, true);
-        cfgOptionSet(cfgOptProtocolTimeout, cfgSourceParam, varNewDbl(50.5));
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("protocol-timeout set but not db timeout");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgRawZ(argList, cfgOptProtocolTimeout, "100");
+        harnessCfgLoad(cfgCmdCheck, argList);
+
+        cfgOptionInvalidate(cfgOptDbTimeout);
+        cfgLoadUpdateOption();
+
+        TEST_RESULT_UINT(cfgOptionUInt64(cfgOptProtocolTimeout), 100000, "check protocol-timeout");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("protocol-timeout set automatically");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgRawZ(argList, cfgOptDbTimeout, "100000");
+        harnessCfgLoad(cfgCmdCheck, argList);
+
+        TEST_RESULT_UINT(cfgOptionUInt64(cfgOptProtocolTimeout), 100030000, "check protocol-timeout");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("error when db-timeout and protocol-timeout set but invalid");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgRawZ(argList, cfgOptDbTimeout, "100000");
+        hrnCfgArgRawZ(argList, cfgOptProtocolTimeout, "50.5");
         TEST_ERROR(
-            cfgLoadUpdateOption(), OptionInvalidValueError,
-            "'50.5' is not valid for 'protocol-timeout' option\n"
-                "HINT 'protocol-timeout' option (50.5) should be greater than 'db-timeout' option (100000).");
-
-        cfgOptionSet(cfgOptProtocolTimeout, cfgSourceParam, varNewDbl(45));
-        cfgOptionSet(cfgOptDbTimeout, cfgSourceDefault, varNewDbl(3600));
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "set default pg timeout to be less than protocol timeout");
-        TEST_RESULT_DOUBLE(cfgOptionDbl(cfgOptProtocolTimeout), 45, "    check protocol timeout");
-        TEST_RESULT_DOUBLE(cfgOptionDbl(cfgOptDbTimeout), 15, "    check db timeout");
-
-        cfgOptionSet(cfgOptProtocolTimeout, cfgSourceParam, varNewDbl(11));
-        cfgOptionSet(cfgOptDbTimeout, cfgSourceDefault, varNewDbl(3600));
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "set default pg timeout to be less than test protocol timeout");
-        TEST_RESULT_DOUBLE(cfgOptionDbl(cfgOptProtocolTimeout), 11, "    check protocol timeout");
-        TEST_RESULT_DOUBLE(cfgOptionDbl(cfgOptDbTimeout), 5.5, "    check db timeout");
+            harnessCfgLoad(cfgCmdCheck, argList), OptionInvalidValueError,
+            "'50500' is not valid for 'protocol-timeout' option\n"
+                "HINT 'protocol-timeout' option (50500) should be greater than 'db-timeout' option (100000000).");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        cfgInit();
-        cfgCommandSet(cfgCmdBackup, cfgCmdRoleDefault);
-        cfgExeSet(exe);
+        TEST_TITLE("very small protocol-timeout triggers db-timeout special handling");
 
-        cfgOptionValidSet(cfgOptPgHost, true);
-        TEST_RESULT_VOID(cfgLoadUpdateOption(), "only repo-host is valid");
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgRawZ(argList, cfgOptProtocolTimeout, "11");
+        harnessCfgLoad(cfgCmdCheck, argList);
 
-        cfgOptionValidSet(cfgOptRepoHost, true);
-        cfgOptionSet(cfgOptRepoHost, cfgSourceParam, varNewStrZ("repo-host"));
-        cfgOptionValidSet(cfgOptPgHost + 4, true);
-        cfgOptionSet(cfgOptPgHost + 4, cfgSourceParam, varNewStrZ("pg5-host"));
-        TEST_ERROR(cfgLoadUpdateOption(), ConfigError, "pg and repo hosts cannot both be configured as remote");
+        TEST_RESULT_UINT(cfgOptionUInt64(cfgOptProtocolTimeout), 11000, "check protocol-timeout");
+        TEST_RESULT_UINT(cfgOptionUInt64(cfgOptDbTimeout), 5500, "check db-timeout");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        StringList *argList = strLstNew();
+        TEST_TITLE("pg and repo cannot both be remote");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 4, "/pg4");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgHost, 4, "pg4");
+        hrnCfgArgRawZ(argList, cfgOptRepoHost, "repo1");
+        TEST_ERROR(harnessCfgLoad(cfgCmdCheck, argList), ConfigError, "pg and repo hosts cannot both be configured as remote");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("only pg can be remote");
+
+        // We'll have to cheat here and invalidate the repo-host option since there are currently no pg-only commands
+        cfgOptionInvalidate(cfgOptRepoHost);
+        cfgLoadUpdateOption();
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("only repo can be remote");
+
+        argList = strLstNew();
+        hrnCfgArgRawZ(argList, cfgOptRepoHost, "repo1");
+        hrnCfgArgRawZ(argList, cfgOptRepo, "1");
+        harnessCfgLoad(cfgCmdInfo, argList);
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
         strLstAdd(argList, strNew("backup"));
         strLstAdd(argList, strNew("process-max"));
 
@@ -231,30 +341,36 @@ testRun(void)
         TEST_RESULT_BOOL(cfgOptionTest(cfgOptRepoRetentionArchive), false, "    repo1-retention-archive not set");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        setenv("PGBACKREST_REPO1_S3_KEY", "mykey", true);
-        setenv("PGBACKREST_REPO1_S3_KEY_SECRET", "mysecretkey", true);
-
         // Invalid bucket name with verification enabled fails
         argList = strLstNew();
         strLstAdd(argList, strNew("--stanza=db"));
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
-        strLstAdd(argList, strNew("--repo1-type=s3"));
-        strLstAdd(argList, strNew("--repo1-s3-bucket=bogus.bucket"));
-        strLstAdd(argList, strNew("--repo1-s3-region=region"));
-        strLstAdd(argList, strNew("--repo1-s3-endpoint=endpoint"));
-        strLstAdd(argList, strNew("--repo1-path=/repo"));
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
+        strLstAdd(argList, strNew("--repo2-type=s3"));
+        strLstAdd(argList, strNew("--repo2-s3-bucket=bogus.bucket"));
+        strLstAdd(argList, strNew("--repo2-s3-region=region"));
+        strLstAdd(argList, strNew("--repo2-s3-endpoint=endpoint"));
+        strLstAdd(argList, strNew("--repo2-path=/repo"));
+        hrnCfgEnvKeyRawZ(cfgOptRepoS3Key, 2, "mykey");
+        hrnCfgEnvKeyRawZ(cfgOptRepoS3KeySecret, 2, "mysecretkey");
+        hrnCfgArgRawZ(argList, cfgOptRepo, "2");
 
         TEST_ERROR(
             harnessCfgLoad(cfgCmdArchiveGet, argList), OptionInvalidValueError,
-            "'bogus.bucket' is not valid for option 'repo1-s3-bucket'"
+            "'bogus.bucket' is not valid for option 'repo2-s3-bucket'"
                 "\nHINT: RFC-2818 forbids dots in wildcard matches."
                 "\nHINT: TLS/SSL verification cannot proceed with this bucket name."
                 "\nHINT: remove dots from the bucket name.");
 
+        hrnCfgEnvKeyRemoveRaw(cfgOptRepoS3Key, 2);
+        hrnCfgEnvKeyRemoveRaw(cfgOptRepoS3KeySecret, 2);
+
         // Invalid bucket name with verification disabled succeeds
+        hrnCfgEnvKeyRawZ(cfgOptRepoS3Key, 1, "mykey");
+        hrnCfgEnvKeyRawZ(cfgOptRepoS3KeySecret, 1, "mysecretkey");
+
         argList = strLstNew();
         strLstAdd(argList, strNew("--stanza=db"));
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
         strLstAdd(argList, strNew("--repo1-type=s3"));
         strLstAdd(argList, strNew("--repo1-s3-bucket=bogus.bucket"));
         strLstAdd(argList, strNew("--repo1-s3-region=region"));
@@ -268,7 +384,7 @@ testRun(void)
         // Valid bucket name
         argList = strLstNew();
         strLstAdd(argList, strNew("--stanza=db"));
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
         strLstAdd(argList, strNew("--repo1-type=s3"));
         strLstAdd(argList, strNew("--repo1-s3-bucket=cool-bucket"));
         strLstAdd(argList, strNew("--repo1-s3-region=region"));
@@ -369,6 +485,26 @@ testRun(void)
             storageRepoWrite(), AssertError, "unable to get writable storage in dry-run mode or before dry-run is initialized");
         lockRelease(true);
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("error on multi-repo");
+
+        argList = strLstNew();
+        strLstAddZ(argList, PROJECT_BIN);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "db");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 1, "/repo1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 2, "/repo2");
+        strLstAddZ(argList, CFGCMD_EXPIRE);
+
+        TEST_ERROR(cfgLoad(strLstSize(argList), strLstPtr(argList)), OptionInvalidValueError, "only repo1 may be configured");
+
+        argList = strLstNew();
+        strLstAddZ(argList, PROJECT_BIN);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "db");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoPath, 2, "/repo2");
+        strLstAddZ(argList, CFGCMD_EXPIRE);
+
+        TEST_ERROR(cfgLoad(strLstSize(argList), strLstPtr(argList)), OptionInvalidValueError, "only repo1 may be configured");
+
         // Command does not have umask and disables keep-alives
         // -------------------------------------------------------------------------------------------------------------------------
         argList = strLstNew();
@@ -390,7 +526,7 @@ testRun(void)
         argList = strLstNew();
         strLstAdd(argList, strNew("pgbackrest"));
         strLstAdd(argList, strNew("--stanza=db"));
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
         strLstAdd(argList, strNew("--log-level-console=off"));
         strLstAdd(argList, strNew("--log-level-stderr=off"));
         strLstAdd(argList, strNew("--log-level-file=off"));
@@ -407,7 +543,7 @@ testRun(void)
         argList = strLstNew();
         strLstAdd(argList, strNew("pgbackrest"));
         strLstAdd(argList, strNew("--stanza=db"));
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
         strLstAdd(argList, strNew("--no-neutral-umask"));
         strLstAdd(argList, strNew("--log-level-console=off"));
         strLstAdd(argList, strNew("--log-level-stderr=off"));
@@ -473,7 +609,8 @@ testRun(void)
         strLstAdd(argList, strNew("backup"));
 
         TEST_RESULT_VOID(cfgLoad(strLstSize(argList), strLstPtr(argList)), "lock and open log file");
-        TEST_RESULT_INT(lstat(strPtr(strNewFmt("%s/db-backup.log", testPath())), &statLog), 0, "   check log file exists");
+        TEST_RESULT_INT(lstat(strZ(strNewFmt("%s/db-backup.log", testPath())), &statLog), 0, "   check log file exists");
+        TEST_RESULT_PTR_NE(cfgOptionStr(cfgOptExecId), NULL, "   exec-id is set");
         TEST_RESULT_BOOL(socketLocal.init, true, "   check socketLocal.init");
         TEST_RESULT_BOOL(socketLocal.block, false, "   check socketLocal.block");
         TEST_RESULT_BOOL(socketLocal.keepAlive, true, "   check socketLocal.keepAlive");
@@ -489,16 +626,16 @@ testRun(void)
         strLstAdd(argList, strNew("pgbackrest"));
         strLstAdd(argList, strNew("--stanza=db"));
         strLstAdd(argList, strNewFmt("--log-path=%s", testPath()));
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to");
         strLstAdd(argList, strNew("--process=1"));
-        strLstAdd(argList, strNew("--host-id=1"));
         strLstAddZ(argList, "--" CFGOPT_REMOTE_TYPE "=" PROTOCOL_REMOTE_TYPE_REPO);
         strLstAdd(argList, strNew("--log-level-file=warn"));
+        hrnCfgArgRawZ(argList, cfgOptExecId, "1111-fe70d611");
         strLstAddZ(argList, CFGCMD_BACKUP ":" CONFIG_COMMAND_ROLE_LOCAL);
 
         TEST_RESULT_VOID(cfgLoad(strLstSize(argList), strLstPtr(argList)), "open log file");
-        TEST_RESULT_INT(
-            lstat(strPtr(strNewFmt("%s/db-backup-local-001.log", testPath())), &statLog), 0, "   check log file exists");
+        TEST_RESULT_INT(lstat(strZ(strNewFmt("%s/db-backup-local-001.log", testPath())), &statLog), 0, "   check log file exists");
+        TEST_RESULT_STR_Z(cfgOptionStr(cfgOptExecId), "1111-fe70d611", "   exec-id is preserved");
 
         // Remote command opens log file with special filename
         // -------------------------------------------------------------------------------------------------------------------------
@@ -512,8 +649,7 @@ testRun(void)
         strLstAddZ(argList, CFGCMD_INFO ":" CONFIG_COMMAND_ROLE_REMOTE);
 
         TEST_RESULT_VOID(cfgLoad(strLstSize(argList), strLstPtr(argList)), "open log file");
-        TEST_RESULT_INT(
-            lstat(strPtr(strNewFmt("%s/all-info-remote-000.log", testPath())), &statLog), 0, "   check log file exists");
+        TEST_RESULT_INT(lstat(strZ(strNewFmt("%s/all-info-remote-000.log", testPath())), &statLog), 0, "   check log file exists");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("remote command without archive-async option");
@@ -522,7 +658,7 @@ testRun(void)
         strLstAdd(argList, strNew("pgbackrest"));
         strLstAdd(argList, strNewFmt("--log-path=%s", testPath()));
         strLstAddZ(argList, "--" CFGOPT_STANZA "=test");
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
         strLstAddZ(argList, "--" CFGOPT_REMOTE_TYPE "=" PROTOCOL_REMOTE_TYPE_REPO);
         strLstAddZ(argList, "--" CFGOPT_LOG_LEVEL_FILE "=info");
         strLstAddZ(argList, "--" CFGOPT_LOG_SUBPROCESS);
@@ -531,7 +667,7 @@ testRun(void)
 
         TEST_RESULT_VOID(cfgLoad(strLstSize(argList), strLstPtr(argList)), "open log file");
         TEST_RESULT_INT(
-            lstat(strPtr(strNewFmt("%s/test-archive-get-remote-001.log", testPath())), &statLog), 0, "   check log file exists");
+            lstat(strZ(strNewFmt("%s/test-archive-get-remote-001.log", testPath())), &statLog), 0, "   check log file exists");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("local command with archive-async option");
@@ -549,7 +685,7 @@ testRun(void)
 
         TEST_RESULT_VOID(cfgLoad(strLstSize(argList), strLstPtr(argList)), "open log file");
         TEST_RESULT_INT(
-            lstat(strPtr(strNewFmt("%s/test-archive-push-async-local-001.log", testPath())), &statLog), 0,
+            lstat(strZ(strNewFmt("%s/test-archive-push-async-local-001.log", testPath())), &statLog), 0,
             "   check log file exists");
 
         // -------------------------------------------------------------------------------------------------------------------------
@@ -560,12 +696,12 @@ testRun(void)
         strLstAdd(argList, strNewFmt("--" CFGOPT_LOG_PATH "=%s", testPath()));
         strLstAdd(argList, strNewFmt("--lock-path=%s/lock", testDataPath()));
         strLstAddZ(argList, "--" CFGOPT_STANZA "=test");
-        strLstAddZ(argList, "--" CFGOPT_PG1_PATH "=/path/to/pg");
+        hrnCfgArgRawZ(argList, cfgOptPgPath, "/path/to/pg");
         strLstAddZ(argList, CFGCMD_ARCHIVE_GET ":" CONFIG_COMMAND_ROLE_ASYNC);
 
         TEST_RESULT_VOID(cfgLoad(strLstSize(argList), strLstPtr(argList)), "open log file");
         TEST_RESULT_INT(
-            lstat(strPtr(strNewFmt("%s/test-archive-get-async.log", testPath())), &statLog), 0, "   check log file exists");
+            lstat(strZ(strNewFmt("%s/test-archive-get-async.log", testPath())), &statLog), 0, "   check log file exists");
 
         lockRelease(true);
     }
