@@ -35,7 +35,7 @@ testRun(void)
         if (system("sudo pg_ctlcluster 11 test start") != 0)
             THROW(AssertError, "unable to start cluster");
 
-        if (system(strPtr(strNewFmt("sudo -u postgres psql -c 'create user %s superuser'", testUser()))) != 0)
+        if (system(strZ(strNewFmt("sudo -u postgres psql -c 'create user %s superuser'", testUser()))) != 0)
             THROW(AssertError, "unable to create superuser");
 #endif
 
@@ -105,7 +105,7 @@ testRun(void)
 #ifndef HARNESS_PQ_REAL
         harnessPqScriptSet((HarnessPq [])
         {
-            {.function = HRNPQ_CONNECTDB, .param = strPtr(
+            {.function = HRNPQ_CONNECTDB, .param = strZ(
                 strNewFmt("[\"dbname='postgres' port=5432 user='%s' host='/var/run/postgresql'\"]", testUser()))},
             {.function = HRNPQ_STATUS, .resultInt = CONNECTION_OK},
             {.function = NULL}
@@ -189,6 +189,25 @@ testRun(void)
         TEST_ERROR(pgClientQuery(client, query), DbQueryError, "unable to cancel query 'select pg_sleep(3000)': test error");
 #endif
 
+        // -------------------------------------------------------------------------------------------------------------------------
+#ifndef HARNESS_PQ_REAL
+        TEST_TITLE("PQgetCancel() returns NULL");
+
+        harnessPqScriptSet((HarnessPq [])
+        {
+            {.function = HRNPQ_SENDQUERY, .param = "[\"select 1\"]", .resultInt = 1},
+            {.function = HRNPQ_CONSUMEINPUT, .sleep = 600},
+            {.function = HRNPQ_ISBUSY, .resultInt = 1},
+            {.function = HRNPQ_CONSUMEINPUT},
+            {.function = HRNPQ_ISBUSY, .resultInt = 1},
+            {.function = HRNPQ_GETCANCEL, .resultNull = true},
+            {.function = NULL}
+        });
+
+        TEST_ERROR(
+            pgClientQuery(client, STRDEF("select 1")), DbQueryError, "unable to cancel query 'select 1': connection was lost");
+#endif
+
         // Execute do block and raise notice
         // -------------------------------------------------------------------------------------------------------------------------
 #ifndef HARNESS_PQ_REAL
