@@ -29,20 +29,18 @@ testRun(void)
     strLstAddZ(argList, "--stanza=db");
     strLstAddZ(argList, "--protocol-timeout=10");
     strLstAddZ(argList, "--buffer-size=16384");
+    hrnCfgArgKeyRawFmt(argList, cfgOptPgPath, 1, "%s/pg", testPath());
     strLstAddZ(argList, "--repo1-host=localhost");
     strLstAdd(argList, strNewFmt("--repo1-host-user=%s", testUser()));
     strLstAdd(argList, strNewFmt("--repo1-path=%s/repo", testPath()));
-    harnessCfgLoad(cfgCmdArchivePush, argList);
+    hrnCfgArgRawZ(argList, cfgOptRepo, "1");
+    harnessCfgLoadRole(cfgCmdArchiveGet, cfgCmdRoleLocal, argList);
 
     // Set type since we'll be running local and remote tests here
     cfgOptionSet(cfgOptRemoteType, cfgSourceParam, VARSTRDEF("repo"));
-    cfgOptionValidSet(cfgOptRemoteType, true);
 
-    // Set pg settings so we can run both db and backup remotes
+    // Set pg host so we can run both pg and repo remotes
     cfgOptionSet(cfgOptPgHost, cfgSourceParam, VARSTRDEF("localhost"));
-    cfgOptionValidSet(cfgOptPgHost, true);
-    cfgOptionSet(cfgOptPgPath, cfgSourceParam, VARSTR(strNewFmt("%s/pg", testPath())));
-    cfgOptionValidSet(cfgOptPgPath, true);
 
     // Start a protocol server to test the remote protocol
     Buffer *serverRead = bufNew(8192);
@@ -60,7 +58,7 @@ testRun(void)
     if (testBegin("storageNew()"))
     {
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), false), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, false), "get remote repo storage");
         TEST_RESULT_UINT(storageInterface(storageRemote).feature, storageInterface(storageTest).feature, "    check features");
         TEST_RESULT_BOOL(storageFeature(storageRemote, storageFeaturePath), true, "    check path feature");
         TEST_RESULT_BOOL(storageFeature(storageRemote, storageFeatureCompress), true, "    check compress feature");
@@ -86,13 +84,7 @@ testRun(void)
     if (testBegin("storageInfo()"))
     {
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
-
-        // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("storage types that are not tested elsewhere");
-
-        TEST_RESULT_UINT(storageRemoteInfoParseType('s'), storageTypeSpecial, "read special type");
-        TEST_ERROR(storageRemoteInfoParseType('z'), AssertError, "unknown storage type 'z'");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("missing file/path");
@@ -108,7 +100,7 @@ testRun(void)
         storagePathCreateP(storageTest, strNew("repo"));
         struct utimbuf utimeTest = {.actime = 1000000000, .modtime = 1555160000};
         THROW_ON_SYS_ERROR(
-            utime(strPtr(storagePathP(storageTest, strNew("repo"))), &utimeTest) != 0, FileWriteError, "unable to set time");
+            utime(strZ(storagePathP(storageTest, strNew("repo"))), &utimeTest) != 0, FileWriteError, "unable to set time");
 
         StorageInfo info = {.exists = false};
         TEST_ASSIGN(info, storageInfoP(storageRemote, NULL), "valid path");
@@ -145,7 +137,7 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("special info");
 
-        TEST_SYSTEM_FMT("mkfifo -m 666 %s", strPtr(storagePathP(storageTest, strNew("repo/fifo"))));
+        TEST_SYSTEM_FMT("mkfifo -m 666 %s", strZ(storagePathP(storageTest, strNew("repo/fifo"))));
 
         TEST_ASSIGN(info, storageInfoP(storageRemote, strNew("fifo")), "valid fifo");
         TEST_RESULT_STR(info.name, NULL, "    name is not set");
@@ -162,7 +154,7 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("link info");
 
-        TEST_SYSTEM_FMT("ln -s ../repo/test %s", strPtr(storagePathP(storageTest, strNew("repo/link"))));
+        TEST_SYSTEM_FMT("ln -s ../repo/test %s", strZ(storagePathP(storageTest, strNew("repo/link"))));
 
         TEST_ASSIGN(info, storageInfoP(storageRemote, strNew("link")), "valid link");
         TEST_RESULT_STR(info.name, NULL, "    name is not set");
@@ -192,24 +184,13 @@ testRun(void)
         TEST_RESULT_STR_Z(info.group, testGroup(), "    check group");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("protocol storage types that are not tested elsewhere");
-
-        TEST_RESULT_VOID(storageRemoteInfoWriteType(server, storageTypePath), "write path type");
-        TEST_RESULT_VOID(storageRemoteInfoWriteType(server, storageTypeSpecial), "write special type");
-
-        ioWriteFlush(serverWriteIo);
-        TEST_RESULT_STR_Z(strNewBuf(serverWrite), ".p\n.s\n", "check result");
-
-        bufUsedSet(serverWrite, 0);
-
-        // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("protocol output that is not tested elsewhere (basic)");
+        TEST_TITLE("protocol output that is not tested elsewhere (detail)");
 
         info = (StorageInfo){.level = storageInfoLevelDetail, .type = storageTypeLink, .linkDestination = STRDEF("../")};
         TEST_RESULT_VOID(storageRemoteInfoWrite(server, &info), "write link info");
 
         ioWriteFlush(serverWriteIo);
-        TEST_RESULT_STR_Z(strNewBuf(serverWrite), ".l\n.0\n.0\n.null\n.0\n.null\n.0\n.\"../\"\n", "check result");
+        TEST_RESULT_STR_Z(strNewBuf(serverWrite), ".2\n.0\n.0\n.null\n.0\n.null\n.0\n.\"../\"\n", "check result");
 
         bufUsedSet(serverWrite, 0);
 
@@ -232,7 +213,6 @@ testRun(void)
         // Do these tests against pg for coverage.  We're not really going to get a pg remote here because the remote for this host
         // id has already been created.  This will just test that the pg storage is selected to provide coverage.
         cfgOptionSet(cfgOptRemoteType, cfgSourceParam, VARSTRDEF("pg"));
-        cfgOptionValidSet(cfgOptRemoteType, true);
 
         paramList = varLstNew();
         varLstAdd(paramList, varNewStrZ(hrnReplaceKey("{[path]}/repo/test")));
@@ -244,7 +224,7 @@ testRun(void)
             strNewBuf(serverWrite),
             hrnReplaceKey(
                 "{\"out\":true}\n"
-                ".f\n.1555160001\n.6\n"
+                ".0\n.1555160001\n.6\n"
                 "{}\n"),
             "check result");
 
@@ -263,7 +243,7 @@ testRun(void)
             strNewBuf(serverWrite),
             hrnReplaceKey(
                 "{\"out\":true}\n"
-                ".f\n.1555160001\n.6\n.{[user-id]}\n.\"{[user]}\"\n.{[group-id]}\n.\"{[group]}\"\n.416\n"
+                ".0\n.1555160001\n.6\n.{[user-id]}\n.\"{[user]}\"\n.{[group-id]}\n.\"{[group]}\"\n.416\n"
                 "{}\n"),
             "check result");
 
@@ -274,7 +254,7 @@ testRun(void)
     if (testBegin("storageInfoList()"))
     {
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path not found");
@@ -299,7 +279,7 @@ testRun(void)
 
         // Path timestamp must be set after file is created since file creation updates it
         struct utimbuf utimeTest = {.actime = 1000000000, .modtime = 1555160000};
-        THROW_ON_SYS_ERROR(utime(strPtr(storagePathP(storageRemote, NULL)), &utimeTest) != 0, FileWriteError, "unable to set time");
+        THROW_ON_SYS_ERROR(utime(strZ(storagePathP(storageRemote, NULL)), &utimeTest) != 0, FileWriteError, "unable to set time");
 
         TEST_RESULT_BOOL(
             storageInfoListP(storageRemote, NULL, hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc),
@@ -312,7 +292,14 @@ testRun(void)
             "check content");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("check protocol function directly with a file");
+        TEST_TITLE("check protocol function directly with a path");
+
+        // Remove the file since ordering cannot be guaranteed in the protocol results
+        storageRemoveP(storageRemote, STRDEF("test"), .errorOnMissing = true);
+
+        // Path timestamp must be set after file is removed since file removal updates it
+        utimeTest = (struct utimbuf){.actime = 1000000000, .modtime = 1555160000};
+        THROW_ON_SYS_ERROR(utime(strZ(storagePathP(storageRemote, NULL)), &utimeTest) != 0, FileWriteError, "unable to set time");
 
         VariantList *paramList = varLstNew();
         varLstAdd(paramList, varNewStrZ(hrnReplaceKey("{[path]}/repo")));
@@ -322,8 +309,7 @@ testRun(void)
         TEST_RESULT_STR_Z(
             strNewBuf(serverWrite),
             hrnReplaceKey(
-                ".\".\"\n.p\n.1555160000\n.{[user-id]}\n.\"{[user]}\"\n.{[group-id]}\n.\"{[group]}\"\n.488\n"
-                ".\"test\"\n.f\n.1555160001\n.6\n.{[user-id]}\n.\"{[user]}\"\n.{[group-id]}\n.\"{[group]}\"\n.416\n"
+                ".\".\"\n.1\n.1555160000\n.{[user-id]}\n.\"{[user]}\"\n.{[group-id]}\n.\"{[group]}\"\n.488\n"
                 ".\n"
                 "{\"out\":true}\n"),
             "check result");
@@ -335,7 +321,7 @@ testRun(void)
     if (testBegin("storageNewRead()"))
     {
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), false), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, false), "get remote repo storage");
         storagePathCreateP(storageTest, strNew("repo"));
 
         Buffer *contentBuf = bufNew(32768);
@@ -346,9 +332,9 @@ testRun(void)
         bufUsedSet(contentBuf, bufSize(contentBuf));
 
         TEST_ERROR_FMT(
-            strPtr(strNewBuf(storageGetP(storageNewReadP(storageRemote, strNew("test.txt"))))), FileMissingError,
+            strZ(strNewBuf(storageGetP(storageNewReadP(storageRemote, strNew("test.txt"))))), FileMissingError,
             "raised from remote-0 protocol on 'localhost': " STORAGE_ERROR_READ_MISSING,
-            strPtr(strNewFmt("%s/repo/test.txt", testPath())));
+            strZ(strNewFmt("%s/repo/test.txt", testPath())));
 
         storagePutP(storageNewWriteP(storageTest, strNew("repo/test.txt")), contentBuf);
 
@@ -484,7 +470,7 @@ testRun(void)
         storagePathCreateP(storageTest, strNew("repo"));
 
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
 
         // Create buffer with plenty of data
         Buffer *contentBuf = bufNew(32768);
@@ -624,19 +610,19 @@ testRun(void)
         storagePathCreateP(storageTest, strNew("repo"));
 
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
 
         // Create a path via the remote. Check the repo via the local test storage to ensure the remote created it.
         TEST_RESULT_VOID(storagePathCreateP(storageRemote, path), "new path");
         StorageInfo info = {0};
-        TEST_ASSIGN(info, storageInfoP(storageTest, strNewFmt("repo/%s", strPtr(path))), "  get path info");
+        TEST_ASSIGN(info, storageInfoP(storageTest, strNewFmt("repo/%s", strZ(path))), "  get path info");
         TEST_RESULT_BOOL(info.exists, true, "  path exists");
         TEST_RESULT_INT(info.mode, STORAGE_MODE_PATH_DEFAULT, "  mode is default");
 
         // Check protocol function directly
         // -------------------------------------------------------------------------------------------------------------------------
         VariantList *paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(path))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(path))));
         varLstAdd(paramList, varNewBool(true));     // errorOnExists
         varLstAdd(paramList, varNewBool(true));     // noParentCreate (true=error if it does not have a parent, false=create parent)
         varLstAdd(paramList, varNewUInt64(0));      // path mode
@@ -649,7 +635,7 @@ testRun(void)
         // Error if parent path not exist
         path = strNew("parent/testpath");
         paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(path))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(path))));
         varLstAdd(paramList, varNewBool(false));    // errorOnExists
         varLstAdd(paramList, varNewBool(true));     // noParentCreate (true=error if it does not have a parent, false=create parent)
         varLstAdd(paramList, varNewUInt64(0));      // path mode
@@ -661,14 +647,14 @@ testRun(void)
 
         // Create parent and path with default mode
         paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(path))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(path))));
         varLstAdd(paramList, varNewBool(true));     // errorOnExists
         varLstAdd(paramList, varNewBool(false));    // noParentCreate (true=error if it does not have a parent, false=create parent)
         varLstAdd(paramList, varNewUInt64(0777));   // path mode
 
         TEST_RESULT_VOID(
             storageRemoteProtocol(PROTOCOL_COMMAND_STORAGE_PATH_CREATE_STR, paramList, server), "create parent and path");
-        TEST_ASSIGN(info, storageInfoP(storageTest, strNewFmt("repo/%s", strPtr(path))), "  get path info");
+        TEST_ASSIGN(info, storageInfoP(storageTest, strNewFmt("repo/%s", strZ(path))), "  get path info");
         TEST_RESULT_BOOL(info.exists, true, "  path exists");
         TEST_RESULT_INT(info.mode, 0777, "  mode is set");
         TEST_RESULT_STR_Z(strNewBuf(serverWrite), "{}\n", "  check result");
@@ -682,18 +668,18 @@ testRun(void)
         storagePathCreateP(storageTest, strNew("repo"));
 
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
         TEST_RESULT_VOID(storagePathCreateP(storageRemote, path), "new path");
 
         // Check the repo via the local test storage to ensure the remote wrote it, then remove via the remote and confirm removed
-        TEST_RESULT_BOOL(storagePathExistsP(storageTest, strNewFmt("repo/%s", strPtr(path))), true, "path exists");
+        TEST_RESULT_BOOL(storagePathExistsP(storageTest, strNewFmt("repo/%s", strZ(path))), true, "path exists");
         TEST_RESULT_VOID(storagePathRemoveP(storageRemote, path), "remote remove path");
-        TEST_RESULT_BOOL(storagePathExistsP(storageTest, strNewFmt("repo/%s", strPtr(path))), false, "path removed");
+        TEST_RESULT_BOOL(storagePathExistsP(storageTest, strNewFmt("repo/%s", strZ(path))), false, "path removed");
 
         // Check protocol function directly
         // -------------------------------------------------------------------------------------------------------------------------
         VariantList *paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(path))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(path))));
         varLstAdd(paramList, varNewBool(true));    // recurse
 
         TEST_RESULT_BOOL(
@@ -705,12 +691,12 @@ testRun(void)
 
         // Write the path and file to the repo and test the protocol
         TEST_RESULT_VOID(
-            storagePutP(storageNewWriteP(storageRemote, strNewFmt("%s/file.txt", strPtr(path))), BUFSTRDEF("TEST")),
+            storagePutP(storageNewWriteP(storageRemote, strNewFmt("%s/file.txt", strZ(path))), BUFSTRDEF("TEST")),
             "new path and file");
         TEST_RESULT_BOOL(
             storageRemoteProtocol(PROTOCOL_COMMAND_STORAGE_PATH_REMOVE_STR, paramList, server), true,
             "  protocol path recurse remove");
-        TEST_RESULT_BOOL(storagePathExistsP(storageTest, strNewFmt("repo/%s", strPtr(path))), false, "  recurse path removed");
+        TEST_RESULT_BOOL(storagePathExistsP(storageTest, strNewFmt("repo/%s", strZ(path))), false, "  recurse path removed");
         TEST_RESULT_STR_Z(strNewBuf(serverWrite), "{\"out\":true}\n", "  check result");
 
         bufUsedSet(serverWrite, 0);
@@ -722,21 +708,21 @@ testRun(void)
         storagePathCreateP(storageTest, strNew("repo"));
 
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
         String *file = strNew("file.txt");
 
         // Write the file to the repo via the remote so owner is pgbackrest
         TEST_RESULT_VOID(storagePutP(storageNewWriteP(storageRemote, file), BUFSTRDEF("TEST")), "new file");
 
         // Check the repo via the local test storage to ensure the remote wrote it, then remove via the remote and confirm removed
-        TEST_RESULT_BOOL(storageExistsP(storageTest, strNewFmt("repo/%s", strPtr(file))), true, "file exists");
+        TEST_RESULT_BOOL(storageExistsP(storageTest, strNewFmt("repo/%s", strZ(file))), true, "file exists");
         TEST_RESULT_VOID(storageRemoveP(storageRemote, file), "remote remove file");
-        TEST_RESULT_BOOL(storageExistsP(storageTest, strNewFmt("repo/%s", strPtr(file))), false, "file removed");
+        TEST_RESULT_BOOL(storageExistsP(storageTest, strNewFmt("repo/%s", strZ(file))), false, "file removed");
 
         // Check protocol function directly
         // -------------------------------------------------------------------------------------------------------------------------
         VariantList *paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(file))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(file))));
         varLstAdd(paramList, varNewBool(true));
 
         TEST_ERROR_FMT(
@@ -745,7 +731,7 @@ testRun(void)
             "[2] No such file or directory", testPath());
 
         paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(file))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(file))));
         varLstAdd(paramList, varNewBool(false));
 
         TEST_RESULT_BOOL(
@@ -759,7 +745,7 @@ testRun(void)
         TEST_RESULT_BOOL(
             storageRemoteProtocol(PROTOCOL_COMMAND_STORAGE_REMOVE_STR, paramList, server), true,
             "protocol file remove");
-        TEST_RESULT_BOOL(storageExistsP(storageTest, strNewFmt("repo/%s", strPtr(file))), false, "  confirm file removed");
+        TEST_RESULT_BOOL(storageExistsP(storageTest, strNewFmt("repo/%s", strZ(file))), false, "  confirm file removed");
         TEST_RESULT_STR_Z(strNewBuf(serverWrite), "{}\n", "  check result");
         bufUsedSet(serverWrite, 0);
     }
@@ -770,7 +756,7 @@ testRun(void)
         storagePathCreateP(storageTest, strNew("repo"));
 
         Storage *storageRemote = NULL;
-        TEST_ASSIGN(storageRemote, storageRepoGet(strNew(STORAGE_POSIX_TYPE), true), "get remote repo storage");
+        TEST_ASSIGN(storageRemote, storageRepoGet(0, true), "get remote repo storage");
 
         String *path = strNew("testpath");
         TEST_RESULT_VOID(storagePathCreateP(storageRemote, path), "new path");
@@ -779,7 +765,7 @@ testRun(void)
         // Check protocol function directly
         // -------------------------------------------------------------------------------------------------------------------------
         VariantList *paramList = varLstNew();
-        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strPtr(path))));
+        varLstAdd(paramList, varNewStr(strNewFmt("%s/repo/%s", testPath(), strZ(path))));
 
         TEST_RESULT_BOOL(
             storageRemoteProtocol(PROTOCOL_COMMAND_STORAGE_PATH_SYNC_STR, paramList, server), true,
@@ -792,7 +778,7 @@ testRun(void)
         TEST_ERROR_FMT(
             storageRemoteProtocol(PROTOCOL_COMMAND_STORAGE_PATH_SYNC_STR, paramList, server), PathMissingError,
             "raised from remote-0 protocol on 'localhost': " STORAGE_ERROR_PATH_SYNC_MISSING,
-            strPtr(strNewFmt("%s/repo/anewpath", testPath())));
+            strZ(strNewFmt("%s/repo/anewpath", testPath())));
     }
 
     protocolFree();
